Package: germinationmetrics
Type: Package
Title: Seed Germination Indices and Curve Fitting
Version: 0.1.10
Authors@R: c(
    person(given = "J.", family = "Aravind",
           email = "j.aravind@icar.org.in",
           role = c("aut", "cre"),
		       comment = c(ORCID = "0000-0002-4791-442X")),
	  person(given = "S.", family = "Vimala Devi",
           email = "vimala.devi@icar.gov.in",
		       role = "aut"),
    person(given = "J.", family = "Radhamani",
           email = "jalli.radhamani@icar.gov.in",
		       role = "aut"),
	  person(given = c("Sherry", "Rachel"), family = "Jacob",
           email = "sherry.jacob@icar.org.in",
		       role = "aut"),
    person(given = c("Kalyani", "Srinivasan"),
           email = "kalyani.srinivasan@icar.gov.in",
		       role = "aut"),
    person("ICAR-NBGPR",
           role = "cph",
		       comment = c(url = "https://nbpgr.org.in/")))
Description: Provides functions to compute various germination indices
    such as germinability, median germination time, mean germination time,
    mean germination rate, speed of germination, Timson's index,
    germination value, coefficient of uniformity of germination,
    uncertainty of germination process, synchrony of germination etc. from
    germination count data. Includes functions for fitting cumulative seed
    germination curves using four-parameter hill function and computation
    of associated parameters. See the vignette for more, including full
    list of citations for the methods implemented.
License: GPL-2 | GPL-3
URL: https://github.com/aravind-j/germinationmetrics,
        https://aravind-j.github.io/germinationmetrics/
        https://CRAN.R-project.org/package=germinationmetrics
        https://doi.org/10.5281/zenodo.1219630
BugReports: https://github.com/aravind-j/germinationmetrics/issues
Depends: R (>= 3.5.0)
Imports: broom, cli, data.table, dplyr, ggplot2, ggrepel, gslnls,
        mathjaxr, methods, plyr, Rdpack, rlang, stats, utils
Suggests: httr, knitr, pander, curl, reshape2, rmarkdown, testthat (>=
        3.0.0), XML
VignetteBuilder: knitr
RdMacros: mathjaxr, Rdpack
Copyright: 2017-2026, ICAR-NBPGR
Encoding: UTF-8
LazyData: true
Config/testthat/edition: 3
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-21 11:05:25 UTC; Aravind-DGC
Author: J. Aravind [aut, cre] (ORCID: <https://orcid.org/0000-0002-4791-442X>),
  S. Vimala Devi [aut],
  J. Radhamani [aut],
  Sherry Rachel Jacob [aut],
  Kalyani Srinivasan [aut],
  ICAR-NBGPR [cph] (url: https://nbpgr.org.in/)
Maintainer: J. Aravind <j.aravind@icar.org.in>
Repository: CRAN
Date/Publication: 2026-04-21 22:22:21 UTC
Built: R 4.6.0; ; 2026-04-30 11:46:42 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,FourPHFfit)
S3method(plot,FourPHFfit.bulk)
export(CUGerm)
export(CVG)
export(CVGermTime)
export(EmergenceRateIndex)
export(FirstGermTime)
export(FourPHF)
export(FourPHF_fixa)
export(FourPHF_fixa_fixy0)
export(FourPHF_fixy0)
export(FourPHFfit)
export(FourPHFfit.bulk)
export(GeomLine2)
export(GeomPath2)
export(GermIndex)
export(GermPercent)
export(GermRateGeorge)
export(GermRateRecip)
export(GermSpeed)
export(GermSpeedAccumulated)
export(GermSpeedCorrected)
export(GermSynchrony)
export(GermUncertainty)
export(GermValue)
export(LastGermTime)
export(MeanGermNumber)
export(MeanGermPercent)
export(MeanGermRate)
export(MeanGermTime)
export(PeakGermPercent)
export(PeakGermTime)
export(PeakValue)
export(RateofGerm)
export(SEGermRate)
export(SEGermTime)
export(TimeSpreadGerm)
export(TimsonsIndex)
export(VarGermRate)
export(VarGermTime)
export(WeightGermPercent)
export(draw_key_path2)
export(geom_line2)
export(geom_path2)
export(germination.indices)
export(t50)
import(data.table)
import(ggplot2)
import(ggrepel)
import(gslnls)
import(mathjaxr)
importFrom(Rdpack,reprompt)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(dplyr,bind_rows)
importFrom(grid,addGrob)
importFrom(grid,convertUnit)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,pointsGrob)
importFrom(grid,polylineGrob)
importFrom(grid,segmentsGrob)
importFrom(methods,is)
importFrom(plyr,mutate)
importFrom(plyr,round_any)
importFrom(rlang,`%||%`)
importFrom(rlang,list2)
importFrom(stats,ave)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,df)
importFrom(stats,integrate)
importFrom(stats,predict)
importFrom(utils,globalVariables)
importFrom(utils,tail)

## `germinationmetrics`: Seed Germination Indices and Curve Fitting <img src="https://raw.githubusercontent.com/aravind-j/germinationmetrics/master/inst/extdata/germinationmetrics.png" align="right" alt="logo" width="173" height = "200" style = "padding: 10px; border: none; float: right;">

###### Version : [0.1.10](https://aravind-j.github.io/germinationmetrics/articles/Introduction.html#version-history); Copyright (C) 2017-2026: [ICAR-NBPGR](https://nbpgr.org.in/); License: [GPL2\|GPL-3](https://www.r-project.org/Licenses/)

##### *Aravind, J., Vimala Devi, S., Radhamani, J., Jacob, S. R., and Kalyani Srinivasan*

ICAR-National Bureau of Plant Genetic Resources, New Delhi

------------------------------------------------------------------------

<!-- badges: start -->

[![minimal R
version](https://img.shields.io/badge/R%3E%3D-3.0.2-6666ff.svg?logo=R)](https://cran.r-project.org/)
[![License: GPL
v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version-last-release/germinationmetrics)](https://cran.r-project.org/package=germinationmetrics)
[![Dependencies](https://tinyverse.netlify.app/status/germinationmetrics)](https://cran.r-project.org/package=germinationmetrics)
[![rstudio mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/germinationmetrics?color=green)](https://CRAN.R-project.org/package=germinationmetrics)
[![develVersion](https://img.shields.io/badge/devel%20version-0.1.10-orange.svg)](https://github.com/aravind-j/germinationmetrics)
[![Github Code
Size](https://img.shields.io/github/languages/code-size/aravind-j/germinationmetrics.svg)](https://github.com/aravind-j/germinationmetrics)
[![R-CMD-check](https://github.com/aravind-j/germinationmetrics/workflows/R-CMD-check/badge.svg)](https://github.com/aravind-j/germinationmetrics/actions)
[![Project Status:
Active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![lifecycle](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![Last-changedate](https://img.shields.io/badge/last%20change-2026--04--17-yellowgreen.svg)](https://github.com/aravind-j/germinationmetrics/commits/master)
[![Zenodo
DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.1219630.svg)](https://doi.org/10.5281/zenodo.1219630)
[![Website -
pkgdown](https://img.shields.io/website-up-down-green-red/https/aravind-j.github.io/germinationmetrics.svg)](https://aravind-j.github.io/germinationmetrics/)
<!-- [![.](https://pro-pulsar-193905.appspot.com/UA-116205606-1/welcome-page)](https://github.com/aravind-j/google-analytics-beacon) -->
[![GoatCounter](https://germinationmetrics-gh.goatcounter.com/count?p=/test)](https://germinationmetrics-gh.goatcounter.com/)
<!-- [![packageversion](https://img.shields.io/badge/Package%20version-0.2.3.3-orange.svg)](https://github.com/aravind-j/germinationmetrics) -->
<!-- [![GitHub Download Count](https://github-basic-badges.herokuapp.com/downloads/aravind-j/germinationmetrics/total.svg)] -->
<!-- [![Rdoc](https://www.rdocumentation.org/badges/version/germinationmetrics)](https://www.rdocumentation.org/packages/germinationmetrics) -->
<!-- badges: end -->

------------------------------------------------------------------------

## Description

Provides functions to compute various germination indices such as
germinability, median germination time, mean germination time, mean
germination rate, speed of germination, Timson’s index, germination
value, coefficient of uniformity of germination, uncertainty of
germination process, synchrony of germination etc. from germination
count data. Includes functions for fitting cumulative seed germination
curves using four-parameter hill function and computation of associated
parameters. See the vignette for more, including full list of citations
for the methods implemented.

## Installation

The package can be installed from CRAN as follows:

``` r
if (!require('devtools')) install.packages('devtools')
install.packages('germinationmetrics', dependencies=TRUE)
```

The development version can be installed from github as follows:

``` r
devtools::install_github("aravind-j/germinationmetrics")
```

## Detailed tutorial

For a detailed tutorial (vignette) on how to used this package type:

``` r
browseVignettes(package = 'germinationmetrics')
```

The vignette for the latest version is also available
[online](https://aravind-j.github.io/germinationmetrics/articles/Introduction.html).

## What’s new

To know whats new in this version type:

``` r
news(package='germinationmetrics')
```

## Links

[CRAN page](https://cran.r-project.org/package=germinationmetrics)

[Github page](https://github.com/aravind-j/germinationmetrics)

[Documentation website](https://aravind-j.github.io/germinationmetrics/)

[Zenodo DOI](https://doi.org/10.5281/zenodo.1219630)

## CRAN checks

    Using libcurl 8.14.1 with Schannel

[![Linux](https://img.shields.io/badge/Linux-FCC624?style=for-the-badge&logo=linux&logoColor=black)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html)

|  |  |
|:---|----|
| r-devel-linux-x86_64-debian-clang | [![CRAN check - r-devel-linux-x86_64-debian-clang](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-debian-clang/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-devel-linux-x86_64-debian-gcc | [![CRAN check - r-devel-linux-x86_64-debian-gcc](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-debian-gcc/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-devel-linux-x86_64-fedora-clang | [![CRAN check - r-devel-linux-x86_64-fedora-clang](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-fedora-clang/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-devel-linux-x86_64-fedora-gcc | [![CRAN check - r-devel-linux-x86_64-fedora-gcc](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-fedora-gcc/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-patched-linux-x86_64 | [![CRAN check - r-patched-linux-x86_64](https://badges.cranchecks.info/flavor/r-patched-linux-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-release-linux-x86_64 | [![CRAN check - r-release-linux-x86_64](https://badges.cranchecks.info/flavor/r-release-linux-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |

[![Windows](https://img.shields.io/badge/Windows-0078D6?style=for-the-badge&logo=windows&logoColor=white)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html)

|  |  |
|:---|----|
| r-devel-windows-x86_64 | [![CRAN check - r-devel-windows-x86_64](https://badges.cranchecks.info/flavor/r-devel-windows-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-release-windows-x86_64 | [![CRAN check - r-release-windows-x86_64](https://badges.cranchecks.info/flavor/r-release-windows-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-oldrel-windows-x86_64 | [![CRAN check - r-oldrel-windows-x86_64](https://badges.cranchecks.info/flavor/r-oldrel-windows-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |

[![MacOS](https://img.shields.io/badge/mac%20os-000000?style=for-the-badge&logo=apple&logoColor=white)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html)

|  |  |
|:---|----|
| r-release-macos-x86_64 | [![CRAN check - r-release-macos-x86_64](https://badges.cranchecks.info/flavor/r-release-macos-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |
| r-oldrel-macos-x86_64 | [![CRAN check - r-oldrel-macos-x86_64](https://badges.cranchecks.info/flavor/r-oldrel-macos-x86_64/germinationmetrics.svg)](https://cran.r-project.org/web/checks/check_results_germinationmetrics.html) |

## Citing `germinationmetrics`

To cite the methods in the package use:

``` r
citation("germinationmetrics")
```

    To cite the R package 'germinationmetrics' in publications use:

      Aravind, J., Vimala Devi, S., Radhamani, J., Jacob, S. R., and Kalyani Srinivasan (2026).
      germinationmetrics: Seed Germination Indices and Curve Fitting. R package version 0.1.10,
      https://github.com/aravind-j/germinationmetrics https://cran.r-project.org/package=germinationmetrics.

    A BibTeX entry for LaTeX users is

      @Manual{,
        title = {germinationmetrics: Seed Germination Indices and Curve Fitting},
        author = {J. Aravind and S. {Vimala Devi} and J. Radhamani and Sherry Rachel Jacob and {Kalyani Srinivasan}},
        note = {R package version 0.1.10 https://aravind-j.github.io/germinationmetrics/ https://CRAN.R-project.org/package=germinationmetrics},
        year = {2026},
      }

    This free and open-source software implements academic research by the authors and co-workers. If you use
    it, please support the project by citing the package.

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Time for first, last and peak germination
#'
#' Compute the following metrics: \describe{ \item{\code{FirstGermTime}}{Time of
#' first germination or Germination time lag (\mjseqn{t_{0}})
#' \insertCite{edwards_temperature_1932,czabator_germination_1962,goloff_germination_1975,labouriau_germinacao_1983,ranal_effects_1999,quintanilla_effect_2000}{germinationmetrics}.}
#' \item{\code{LastGermTime}}{Time of last germination (\mjseqn{t_{g}})
#' \insertCite{edwards_temperature_1932,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
#' \item{\code{TimeSpreadGerm}}{Time spread of germination
#' \insertCite{al-mudaris_notes_1998,kader_comparison_2005}{germinationmetrics}
#' or Germination distribution
#' \insertCite{schrader_seed_2000}{germinationmetrics}.}
#' \item{\code{PeakGermTime}}{Peak time of germination or Modal time of
#' germination (\mjseqn{t_{peak}})
#' \insertCite{ranal_how_2006}{germinationmetrics}.} } \loadmathjax
#'
#' Time of first germination indicates time of germination of the faster seeds
#' in a seedlot.
#'
#' Lower value of time of first germination indicates faster initiation of
#' germination and lower value of time of last germination indicates faster
#' termination of germination.
#'
#' \mjsdeqn{t_{0} = \min \lbrace T_{i} : N_{i} \neq  0 \rbrace}
#'
#' \mjsdeqn{t_{g} = \max \lbrace T_{i} : N_{i} \neq  0 \rbrace}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval and \mjseqn{N_{i}} is the number of seeds germinated in
#' the \mjseqn{i}th time interval (not the accumulated number, but the number
#' corresponding to the \mjseqn{i}th interval).
#'
#'
#' Time spread of germination (\mjseqn{t_{g}-t_{0}}) indicates difference
#' between faster and slower germinating members of a sample.
#'
#' Peak time of germination is the time in which highest frequency of germinated
#' seeds are observed. Multiple peak times of germination are possible and if
#' detected are indicated by a warning message.
#'
#' \mjsdeqn{t_{peak} = \lbrace T_{i} : N_{i} = N_{max} \rbrace}
#'
#' Where, \mjseqn{N_{max}} is the maximum number of seeds germinated per
#' interval.
#'
#' @inheritParams MeanGermTime
#' @return For \code{FirstGermTime}, the time of first germination value in the
#'   same unit of time as specified in the argument \code{intervals}.
#'
#'   For \code{LastGermTime}, the time of last germination value in the same
#'   unit of time as specified in the argument \code{intervals}.
#'
#'   For \code{TimeSpreadGerm}, the time spread of germination value in the same
#'   unit of time as specified in the argument \code{intervals}.
#'
#'   For \code{PeakGermTime}, the time(s) of peak germination value(s) as a
#'   numeric vector in the same unit of time as specified in the argument
#'   \code{intervals}.
#'
#' @importFrom Rdpack reprompt
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name FirstGermTime
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' z <- c(0, 0, 0, 0, 11, 11, 9, 7, 1, 0, 1, 0, 0, 0)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' FirstGermTime(germ.counts = x, intervals = int)
#' LastGermTime(germ.counts = x, intervals = int)
#' TimeSpreadGerm(germ.counts = x, intervals = int)
#' PeakGermTime(germ.counts = x, intervals = int)
#' # For multiple peak germination times
#' PeakGermTime(germ.counts = z, intervals = int)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' FirstGermTime(germ.counts = y, intervals = int, partial = FALSE)
#' LastGermTime(germ.counts = y, intervals = int, partial = FALSE)
#' TimeSpreadGerm(germ.counts = y, intervals = int, partial = FALSE)
#' PeakGermTime(germ.counts = y, intervals = int, partial = FALSE)
#' # For multiple peak germination time
#' PeakGermTime(germ.counts = cumsum(z), intervals = int, partial = FALSE)
#'


#' @rdname FirstGermTime
#' @export
FirstGermTime <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  df <- data.frame(germ.counts, intervals)

  FGT <- intervals[min(which(df$germ.counts != 0))]

  return(FGT)
}

#' @rdname FirstGermTime
#' @export
LastGermTime <-  function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  df <- data.frame(germ.counts, intervals)

  LGT <- intervals[max(which(df$germ.counts != 0))]

  return(LGT)
}

#' @rdname FirstGermTime
#' @export
PeakGermTime <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts
  PGT <- intervals[which(x %in% max(x))]

  if (length(PGT) > 1) {
    warning("Multiple peak germination times exist.")
  }

  return(PGT)
}

#' @rdname FirstGermTime
#' @export
TimeSpreadGerm <-  function(germ.counts, intervals, partial = TRUE) {

  FGT <- FirstGermTime(germ.counts, intervals, partial)
  LGT <- LastGermTime(germ.counts, intervals, partial)

  TSG <- LGT - FGT

  return(TSG)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
z <- c(0, 0, 0, 0, 11, 11, 9, 7, 1, 0, 1, 0, 0, 0)
int <- 1:length(x)

test_that("FirstGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = FirstGermTime(germ.counts = x, intervals = int),
               expected = 5)
})

test_that("FirstGermTime works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = FirstGermTime(germ.counts = y, intervals = int,
                                      partial = FALSE),
               expected = 5)
})

test_that("FirstGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = LastGermTime(germ.counts = x, intervals = int),
               expected = 11)
})

test_that("FirstGermTime works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = LastGermTime(germ.counts = y, intervals = int,
                                     partial = FALSE),
               expected = 11)
})

test_that("FirstGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = TimeSpreadGerm(germ.counts = x, intervals = int),
               expected = 6)
})

test_that("FirstGermTime works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = TimeSpreadGerm(germ.counts = y, intervals = int,
                                       partial = FALSE),
               expected = 6)
})

test_that("FirstGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = PeakGermTime(germ.counts = x, intervals = int),
               expected = 6)
})

test_that("FirstGermTime works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = PeakGermTime(germ.counts = y, intervals = int,
                                     partial = FALSE),
               expected = 6)
})

test_that("PeakGermPercent with multiple peaks works; Partial germination counts", {

  # From partial germination counts
  expect_warning(object = out1 <- PeakGermTime(germ.counts = z,
                                               intervals = int),
                 regexp = "Multiple peak germination times exist.")
  expect_equal(object = out1, expected = c(5, 6))
})

test_that("PeakGermPercent with multiple peaks works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_warning(object = out2 <- PeakGermTime(germ.counts = cumsum(z),
                                                  intervals = int,
                                                  partial = FALSE),
                 regexp = "Multiple peak germination times exist.")
  expect_equal(object = out2, expected = c(5, 6))
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = FirstGermTime(germ.counts = x, intervals = int),
                   expected = FirstGermTime(germ.counts = y, intervals = int,
                                            partial = FALSE))
  expect_identical(object = LastGermTime(germ.counts = x, intervals = int),
                   expected = LastGermTime(germ.counts = y, intervals = int,
                                           partial = FALSE))
  expect_identical(object = TimeSpreadGerm(germ.counts = x, intervals = int),
                   expected = TimeSpreadGerm(germ.counts = y, intervals = int,
                                             partial = FALSE))
  expect_identical(object = PeakGermTime(germ.counts = x, intervals = int),
                   expected = PeakGermTime(germ.counts = y, intervals = int,
                                           partial = FALSE))

  expect_warning(object = out1 <- PeakGermTime(germ.counts = z,
                                               intervals = int),
                 regexp = "Multiple peak germination times exist.")
  expect_warning(object = out2 <- PeakGermTime(germ.counts = cumsum(z),
                                               intervals = int,
                                               partial = FALSE),
                 regexp = "Multiple peak germination times exist.")
  expect_identical(object = out1, expected = out2)

})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Weighted germination percentage
#'
#' Compute the Weighted germination percentage
#' \mjseqn{WGP} or Weighted germination index
#' \mjseqn{WGI}
#' \insertCite{reddy_effect_1978,reddy_effect_1985}{germinationmetrics}. \loadmathjax
#'
#'
#' Weighted germination percentage was calculated by giving maximum weight to
#' the seeds that germinate first and decreasing weight to the seeds that
#' germinate subsequently. It is computed as follows
#' \insertCite{reddy_effect_1978,reddy_effect_1985}{germinationmetrics}.
#'
#' \mjsdeqn{WGP =
#' \frac{\sum_{i=1}^{k}(k-i+1)N_{i}}{k \times N} \times 100}
#'
#' Where, \mjseqn{N_{i}}  is the  number
#' of seeds that germinated in the time interval
#' \mjseqn{i} (not cumulative, but partial count),
#' \mjseqn{N} is the total number of seeds tested,
#' and \mjseqn{k} is the  total number of time
#' intervals.
#'
#' @inheritParams MeanGermTime
#' @param total.seeds Total number of seeds tested.
#'
#' @return The weighted germination percentage (\%) value.
#' @export
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' WeightGermPercent(germ.counts = x, total.seeds = 50, intervals = int)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' WeightGermPercent(germ.counts = y, total.seeds = 50, intervals = int,
#'                   partial = FALSE)
#'
#'
WeightGermPercent <- function(germ.counts, total.seeds, intervals,
                              partial = TRUE) {
  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  invtimes <- rev(seq_along(intervals))

  WGP <- (sum(invtimes * germ.counts) / (max(invtimes) * total.seeds)) * 100
  return(WGP)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("MeanGermRate works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = WeightGermPercent(germ.counts = x, total.seeds = 50,
                                          intervals = int),
               expected = 47.4285714285714, tolerance = 1e-12)
})

test_that("MeanGermRate works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = WeightGermPercent(germ.counts = y, total.seeds = 50,
                                          intervals = int, partial = FALSE),
               expected = 47.4285714285714, tolerance = 1e-12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = WeightGermPercent(germ.counts = x,
                                              total.seeds = 50,
                                              intervals = int),
                   expected = WeightGermPercent(germ.counts = y,
                                                total.seeds = 50,
                                                intervals = int,
                                                partial = FALSE))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Plot the multiple four-parameter hill function fitted cumulative germination
#' curves from a \code{FourPHFfit.bulk} object
#'
#' \code{plot.FourPHFfit} plots multiple four-parameter hill function fitted
#' cumulative germination curves (FPHF curves) or  the rate of germination
#' curves (RoG curves) from a \code{FourPHFfit.bulk} object as an object of
#' class \code{ggplot}. Further, either one of the parameters such as time
#' required for 50\% of total seeds to germinate
#' (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Total}}}), time required for 50\% of
#' viable/germinated seeds to germinate
#' (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Germ}}}), Time at Maximum
#' Germination Rate (TMGR), Mean Germination Time (MGT), or uniformity value
#' (\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}})
#' can also be annotated in the plot as specified in the \code{annotate}
#' argument. \loadmathjax
#'
#' @param x An data frame object of class \code{FourPHFfit.bulk} obtained as
#'   output from the \code{\link[germinationmetrics]{FourPHFfit.bulk}} function.
#' @param rog If \code{TRUE}, plots the Rate of Germination curves (RoG) instead
#'   of the Four-Parameter Hill Function fitted cumulative germination curves
#'   (FPHF). Default is \code{FALSE}.
#' @param annotate The parameter to be annotated in the plot. Either
#'   \code{"t50.total"} (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Total}}}),
#'   \code{"t50.germ"} (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Germ}}}),
#'   \code{"tmgr"} (TMGR), \code{"mgt"} (MGT), or \code{"uniformity"}
#'   (\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}}).
#'    All the parameters are annotated as vertical lines, except for
#'   (\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}}),
#'    which is annotated as horizontal lines.
#' @param limits logical. If \code{TRUE}, set the limits of y axis (germination
#'   percentage) between 0 and 100 in the germination curve plot. If
#'   \code{FALSE}, limits are set according to the data. Default is \code{TRUE}.
#' @param group.col The names of the column in data frame \code{x} according to
#'   which grouping has to be done.
#' @param show.points logical. If \code{TRUE}, shows the original data points
#'   (only for the FPHF curve). Default is \code{TRUE}.
#' @param \dots Default plot arguments.
#'
#' @return The plot of the cumulative germination curve as an object of class
#'   \code{ggplot}.
#'
#' @seealso \code{\link[germinationmetrics]{FourPHFfit.bulk}}
#'
#' @import ggplot2
#' @importFrom dplyr bind_rows
#' @importFrom plyr mutate
#' @importFrom plyr round_any
#' @method plot FourPHFfit.bulk
#' @export
#'
#' @examples
#' \donttest{
#' data(gcdata)
#'
#' counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
#'                           "Day06", "Day07", "Day08", "Day09", "Day10",
#'                           "Day11", "Day12", "Day13", "Day14")
#'
#' fits <- FourPHFfit.bulk(gcdata, total.seeds.col = "Total Seeds",
#'                     counts.intervals.cols = counts.per.intervals,
#'                     intervals = 1:14, partial = TRUE,
#'                     fix.y0 = TRUE, fix.a = TRUE, xp = c(10, 60),
#'                     tmax = 20, tries = 3, umax = 90, umin = 10)
#'
#' # Plot FPHF curves
#' plot(fits, group.col = "Genotype")
#'
#' # Plot ROG curves
#' plot(fits, rog = TRUE, group.col = "Genotype")
#'
#' # Plot FPHF curves with points
#' plot(fits, group.col = "Genotype", show.points = TRUE)
#'
#' # Plot FPHF curves with annotations
#' plot(fits, group.col = "Genotype", annotate = "t50.total")
#' plot(fits, group.col = "Genotype", annotate = "t50.germ")
#' plot(fits, group.col = "Genotype", annotate = "tmgr")
#' plot(fits, group.col = "Genotype", annotate = "mgt")
#' plot(fits, group.col = "Genotype", annotate = "uniformity")
#'
#' # Plot ROG curves with annotations
#' plot(fits, rog = TRUE, group.col = "Genotype", annotate = "t50.total")
#' plot(fits, rog = TRUE, group.col = "Genotype", annotate = "t50.germ")
#' plot(fits, rog = TRUE, group.col = "Genotype", annotate = "tmgr")
#' plot(fits, rog = TRUE, group.col = "Genotype", annotate = "mgt")
#' plot(fits, rog = TRUE, group.col = "Genotype", annotate = "uniformity")
#'
#'
#'
#' # Change colour of curves using ggplot2 options
#' library(ggplot2)
#' curvesplot <- plot(fits, group.col = "Genotype")
#'
#' # 'Dark2' palette from RColorBrewer
#' curvesplot + scale_colour_brewer(palette = "Dark2")
#'
#' # Manual colours
#' curvesplot +
#'   scale_colour_manual(values = c("Coral", "Brown", "Blue",
#'                                  "Aquamarine", "Red"))
#' }
#'

plot.FourPHFfit.bulk <- function(x, rog = FALSE,
                                 annotate = c("none", "t50.total", "t50.germ",
                                              "tmgr", "mgt", "uniformity"),
                                 limits = TRUE, group.col,
                                 show.points = FALSE, ...){

  if (is.null(annotate)) {
    annotate == "none"
  }

  # checks
  annotate <- match.arg(annotate)

  # Check if group.col column present in x
  if (!(group.col %in% colnames(x))) {
    stop(paste('Column ', group.col,
               ' specified as the grouping column is not present in "x".',
               sep = ""))
  }

  if (annotate == "mgt") {
    acol <- "MGT"
  }
  if (annotate == "t50.total") {
    acol <- "t50.total"
  }
  if (annotate == "t50.germ") {
    acol <- "t50.Germinated"
  }
  if (annotate == "tmgr") {
    acol <- "TMGR"
  }
  if (annotate == "uniformity") {
    acol <- colnames(x)[grepl("Uniformity_", colnames(x))]
  }

  partial <- attributes(x)$arguments$partial
  intervals <- attributes(x)$arguments$intervals
  total.seeds.col <- attributes(x)$arguments$total.seeds.col
  counts.intervals.cols <- attributes(x)$arguments$counts.intervals.cols


  if (partial) {
    maxg <- max(rowSums(x[, counts.intervals.cols]))
  } else {
    maxg <- max(apply((x[, counts.intervals.cols]), 1, max))
  }

  # labels and unifromity positions
  if (limits == TRUE) {
    ypos <-  90
    ypos2 <-  80
  } else {
    ypos <-  plyr::round_any(maxg, 10, floor)
    ypos2 <- plyr::round_any(maxg, 10, floor)
  }

  # # Merge grouping columns
  # if (length(group.col) > 1) {
  #   x[, group.col] <- apply(x[, group.col, drop = F], MARGIN = 1,
  #                     FUN = function(i) paste(i, collapse = ""))
  # }

  x$curve <- 1:nrow(x)

  df <- x[, c(counts.intervals.cols,
              total.seeds.col, group.col, "curve")]
  coefs <- x[, c("a", "b", "c", "y0", group.col, "curve")]
  # coefs$curve <- 1:nrow(coefs)

  if (rog == FALSE) { # Plot FPHF curve
    # Calculate curves
    dfcurve <- merge(coefs, data.frame(intervals = seq(min(intervals),
                                                       max(intervals),
                                                       by = 0.1)))
    dfcurve[, c("a", "b", "c", "y0")] <-
      sapply(dfcurve[, c("a", "b", "c", "y0")], as.numeric)
    dfcurve <- plyr::mutate(dfcurve, csgp = FourPHF(intervals, a,
                                                    log(b, base = exp(1)),
                                                    c, y0))

    # Plot
    Gplot <- ggplot(data = dfcurve, aes(x = intervals, y = csgp,
                                        group = curve)) +
      geom_line(aes(colour = .data[[group.col]])) +
      labs(x = "Time", y = "Germination (%)") +
      theme_bw()

    if (show.points) {
      # Fetch cumulative germination data
      if (partial) {
        dfcsgp <- df
        dfcsgp[, counts.intervals.cols] <-
          t(apply(dfcsgp[, counts.intervals.cols],
                  1, cumsum))
        dfcsgp <- reshape2::melt(dfcsgp, id.vars = c(group.col,
                                                     total.seeds.col, "curve"),
                                 measure.vars = counts.intervals.cols,
                                 variable.name = "intervals")
        dfcsgp$csgp <- dfcsgp$value /
          dfcsgp[, total.seeds.col] * 100

        levels(dfcsgp$intervals) <- intervals
        dfcsgp$intervals <- as.numeric(as.character(dfcsgp$intervals))
      } else{
        dfcsgp <- reshape2::melt(df, id.vars = c(group.col,
                                                 total.seeds.col, "curve"),
                                 measure.vars = counts.intervals.cols,
                                 variable.name = "intervals")
        dfcsgp$csgp <- dfcsgp$value /
          dfcsgp[, total.seeds.col] * 100

        levels(dfcsgp$intervals) <- intervals
        dfcsgp$intervals <- as.numeric(as.character(dfcsgp$intervals))
      }

      dfcurve$curve <- as.factor(dfcurve$curve)
      dfcsgp$curve <- as.factor(dfcsgp$curve)

      dfcsgp$sel <- TRUE
      dfcurve$sel <- FALSE
      dfcurve2 <- dplyr::bind_rows(list(dfcurve, dfcsgp))

      dfcurve2$inc <- FALSE

      # dfcurve2$sel <- as.factor(dfcurve2$sel)

      # Gplot <- Gplot +
      #   # geom_point(data = dfcsgp, aes_string(x = "intervals", y = "csgp",
      #   #                                      colour = group.col),
      #   #            alpha = 0.5, inherit.aes = FALSE)
      #   geom_point(data = dfcsgp, aes(x = intervals, y = csgp,
      #                                 colour = .data[[group.col]]),
      #              alpha = 0.5, inherit.aes = FALSE)
      Gplot <-
        ggplot(data = dfcurve2, aes(x = intervals, y = csgp,
                                    group = curve,
                                    colour = .data[[group.col]],
                                    show.points = sel,
                                    include.points = inc)) +
        geom_line2() +
        labs(x = "Time", y = "Germination (%)") +
        theme_bw()
    }

  } else { # Plot ROG curve
    # Calculate curves
    dfcurve <- merge(coefs, data.frame(intervals = seq(min(intervals),
                                                       max(intervals),
                                                       by = 0.1)))
    dfcurve[, c("a", "b", "c", "y0")] <-
      sapply(dfcurve[, c("a", "b", "c", "y0")], as.numeric)
    dfcurve <- plyr::mutate(dfcurve, gp = RateofGerm(intervals, a, b, c))

    Gplot <- ggplot(data = dfcurve, aes(x = intervals, y = gp, group = curve)) +
      geom_line(aes(colour = .data[[group.col]])) +
      # geom_point(data = dfgp, aes_string(x = "intervals",
      #            y = "gp", colour = group.col),
      #            alpha = 0.5, inherit.aes = FALSE) +
      labs(x = "Time", y = "Germination (%)") +
      theme_bw()
  }

  if (annotate != "none") {
    dfannotate <- x[, c(group.col, acol)]
    dfannotate[, acol] <- sapply(dfannotate[, acol], as.numeric)

    if (annotate != "uniformity") {
      Gplot <- Gplot +
        geom_vline(data = dfannotate,
                   aes(xintercept = .data[[acol]], colour = .data[[group.col]]),
                   linetype = "dashed")
    } else {
      Gplot <- Gplot +
        geom_errorbar(data = dfannotate,
                      aes(xmin = .data[[acol[1]]], xmax = .data[[acol[2]]],
                          y = ypos2, colour = .data[[group.col]], width = 10),
                      inherit.aes = FALSE,
                      position = position_dodge(5))
    }

  }

  # plot limits
  if (limits == TRUE) {
    Gplot <- Gplot + coord_cartesian(xlim = c(0, max(intervals)),
                                     ylim = c(0, 100))
  }

  Gplot <- Gplot +
    theme(axis.text = element_text(colour = "black"))

  return(Gplot)

}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Germination rate
#'
#' Compute the following metrics: \describe{ \item{\code{MeanGermRate}}{Mean
#' germination rate (\mjseqn{\overline{V}})
#' \insertCite{labouriau_germination_1976,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.}
#' \item{\code{VarGermRate}}{Variance of germination rate (\mjseqn{s_{V}^{2}})
#' \insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}. }
#' \item{\code{SEGermRate}}{Standard error of germination rate
#' (\mjseqn{s_{\overline{V}}})
#' \insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}. }
#' \item{\code{CVG}}{Coefficient of velocity/rate of germination or Kotowski's
#' coefficient of velocity (\mjseqn{CVG})
#' \insertCite{kotowski_temperature_1926,nichols_two_1968,labouriau_uma_1983,scott_review_1984,bewley_seeds_1994}{germinationmetrics}.
#' } \item{\code{GermRateRecip}}{Germination rate as reciprocal of median time
#' (\mjseqn{v_{50}})
#' \insertCite{went_experimental_1957,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.}
#' } \loadmathjax
#'
#' \code{MeanGermRate} computes the mean germination rate
#' (\mjseqn{\overline{V}}) according to the following formula
#' \insertCite{labouriau_germination_1976,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{\overline{V} = \frac{\sum_{i=1}^{k}N_{i}}{\sum_{i=1}^{k}N_{i}T_{i}}}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
#' the \mjseqn{i}th time interval (not the accumulated number, but the number
#' corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
#' number of time intervals.
#'
#' It is the inverse of mean germination time (\mjseqn{\overline{T}}).
#'
#' \mjsdeqn{\overline{V} = \frac{1}{\overline{T}}}
#'
#' \code{VarGermRate} computes the variance of germination rate
#' (\mjseqn{s_{V}^{2}}) according to the following formula
#' \insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{s_{V}^{2} = \overline{V}^{4} \times s_{T}^{2}}
#'
#' Where, \mjseqn{s_{T}^{2}} is the variance of germination time.
#'
#' \code{SEGermRate} computes the standard error of germination time
#' (\mjseqn{s_{\overline{V}}}) according to the following formula
#' \insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{s_{\overline{V}} = \sqrt{\frac{s_{V}^{2}}{\sum_{i=1}^{k}N_{i}}}}
#'
#' Where, \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th
#' time interval (not the accumulated number, but the number corresponding to
#' the \mjseqn{i}th interval), and \mjseqn{k} is the total number of time
#' intervals.
#'
#' \code{CVG} computes the coefficient of velocity/rate of germination or
#' Kotowski's coefficient of velocity (\mjseqn{CVG}) according to the following
#' formula
#' \insertCite{kotowski_temperature_1926,nichols_two_1968,labouriau_uma_1983,scott_review_1984,bewley_seeds_1994}{germinationmetrics}.
#'
#' \mjsdeqn{CVG = \frac{\sum_{i=1}^{k}N_{i}}{\sum_{i=1}^{k}N_{i}T_{i}} \times
#' 100}
#'
#' \mjsdeqn{CVG = \overline{V} \times 100}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
#' the \mjseqn{i}th time interval (not the accumulated number, but the number
#' corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
#' number of time intervals.
#'
#' \code{GermRateRecip} computes the germination rate (\mjseqn{v_{50}}) as the
#' reciprocal of the median germination time (\mjseqn{t_{50}})
#' \insertCite{went_experimental_1957,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}
#' according to the methods of
#' \insertCite{coolbear_effect_1984;textual}{germinationmetrics} (Specified by
#' the argument \code{method = "coolbear"}) or
#' \insertCite{farooq_thermal_2005;textual}{germinationmetrics} (Specified by
#' the argument \code{method = "farooq"}) as follows.
#'
#' \mjsdeqn{v_{50} = \frac{1}{t_{50}}}
#'
#' @inheritParams MeanGermTime
#' @param method The method for computing median germination time. Either
#'   \code{"coolbear"} or \code{"farooq"}.
#'
#' @return For \code{MeanGermRate}, the mean germination rate value as
#'   \mjseqn{\mathrm{time^{-1}}}.
#'
#'   For \code{VarGermTime}, the variance of germination rate value as
#'   \mjseqn{\mathrm{time^{-2}}}.
#'
#'   For \code{SEGermTime}, the standard error of germination rate as
#'   \mjseqn{\mathrm{time^{-1}}}.
#'
#'   For \code{CVG}, the value of Coefficient of of velocity/rate of germination
#'   or Kotowski's coefficient of velocity as \% \mjseqn{\mathrm{time^{-1}}}.
#'
#'   For \code{GermRateRecip}, the value of germination rate as
#'   \mjseqn{\mathrm{time^{-1}}}.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name MeanGermRate
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' MeanGermRate(germ.counts = x, intervals = int)
#' CVG(germ.counts = x, intervals = int)
#' VarGermRate(germ.counts = x, intervals = int)
#' SEGermRate(germ.counts = x, intervals = int)
#' GermRateRecip(germ.counts = x, intervals = int, method = "coolbear")
#' GermRateRecip(germ.counts = x, intervals = int, method = "farooq")
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' MeanGermRate(germ.counts = y, intervals = int, partial = FALSE)
#' CVG(germ.counts = y, intervals = int, partial = FALSE)
#' VarGermRate(germ.counts = y, intervals = int, partial = FALSE)
#' SEGermRate(germ.counts = y, intervals = int, partial = FALSE)
#' GermRateRecip(germ.counts = y, intervals = int,
#'               method = "coolbear", partial = FALSE)
#' GermRateRecip(germ.counts = y, intervals = int,
#'               method = "farooq", partial = FALSE)
#'
#' @seealso \code{\link[germinationmetrics]{MeanGermTime}},
#'   \code{\link[germinationmetrics]{t50}}


#' @rdname MeanGermRate
#' @export
MeanGermRate <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts
  MGR <- sum(x)/sum(x*intervals)

  return(MGR)
}

#' @rdname MeanGermRate
#' @export
CVG <- function(germ.counts, intervals, partial = TRUE) {

  MGR <- MeanGermRate(germ.counts, intervals, partial)
  CVG <- MGR*100

  return(CVG)

}

#' @rdname MeanGermRate
#' @export
VarGermRate <- function(germ.counts, intervals, partial = TRUE) {

  MGR <- MeanGermRate(germ.counts, intervals, partial)
  VGT <- VarGermTime(germ.counts, intervals, partial)
  VGR <- (MGR^4)*VGT

  return(VGR)

}

#' @rdname MeanGermRate
#' @export
SEGermRate <- function(germ.counts, intervals, partial = TRUE) {

  VGR <- VarGermRate(germ.counts, intervals, partial)

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  SEGR <- sqrt(VGR/sum(germ.counts))

  return(SEGR)

}

#' @rdname MeanGermRate
#' @export
GermRateRecip <- function(germ.counts, intervals, partial = TRUE,
                          method = c("coolbear", "farooq")) {

  # Check method argument
  method <-  match.arg(method)

  warn <- NULL
  withCallingHandlers(t50 <- t50(germ.counts, intervals, partial, method),
           warning = function(w) {
             warn <<- w
             invokeRestart("muffleWarning")})

  if (!is.null(warn)) {
    warning(gsub("t50", "GermRateRecip", warn$message))
  }

  GR <- 1/t50

  return(GR)


}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("MeanGermRate works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = MeanGermRate(germ.counts = x, intervals = int),
               expected = 0.149253731343284, tolerance = 1e-12)
})

test_that("MeanGermRate works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = MeanGermRate(germ.counts = y, intervals = int,
                                     partial = FALSE),
               expected = 0.149253731343284, tolerance = 1e-12)
})

test_that("CVG works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = CVG(germ.counts = x, intervals = int),
               expected = 14.9253731343284, tolerance = 1e-12)
})

test_that("CVG works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = CVG(germ.counts = y, intervals = int,
                            partial = FALSE),
               expected = 14.9253731343284, tolerance = 1e-12)
})

test_that("VarGermRate works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = VarGermRate(germ.counts = x, intervals = int),
               expected = 0.000717654291368627, tolerance = 1e-12)
})

test_that("VarGermRate works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = VarGermRate(germ.counts = y, intervals = int,
                                   partial = FALSE),
               expected = 0.000717654291368627, tolerance = 1e-12)
})

test_that("SEGermRate works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = SEGermRate(germ.counts = x, intervals = int),
               expected = 0.00423572393862202, tolerance = 1e-12)
})

test_that("SEGermRate works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = SEGermRate(germ.counts = y, intervals = int,
                                   partial = FALSE),
               expected = 0.00423572393862202, tolerance = 1e-12)
})

test_that("GermRateRecip works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermRateRecip(germ.counts = x, intervals = int,
                                      method = "coolbear"),
               expected = 0.167487684729064, tolerance = 1e-12)
  expect_equal(object = GermRateRecip(germ.counts = x, intervals = int,
                                      method = "farooq"),
               expected = 0.168316831683168, tolerance = 1e-12)
})

test_that("GermRateRecip works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = GermRateRecip(germ.counts = y, intervals = int,
                                      method = "coolbear", partial = FALSE),
               expected = 0.167487684729064, tolerance = 1e-12)
  expect_equal(object = GermRateRecip(germ.counts = y, intervals = int,
                                      method = "farooq", partial = FALSE),
               expected = 0.168316831683168, tolerance = 1e-12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = MeanGermRate(germ.counts = x, intervals = int),
                   expected = MeanGermRate(germ.counts = y, intervals = int,
                                           partial = FALSE))
  expect_identical(object = CVG(germ.counts = x, intervals = int),
                   expected = CVG(germ.counts = y, intervals = int,
                                  partial = FALSE))
  expect_identical(object = VarGermRate(germ.counts = x, intervals = int),
                   expected = VarGermRate(germ.counts = y, intervals = int,
                                          partial = FALSE))
  expect_identical(object = SEGermRate(germ.counts = x, intervals = int),
                   expected = SEGermRate(germ.counts = y, intervals = int,
                                         partial = FALSE))
  expect_identical(object = GermRateRecip(germ.counts = x, intervals = int,
                                          method = "coolbear"),
                   expected = GermRateRecip(germ.counts = y, intervals = int,
                                            method = "coolbear",
                                            partial = FALSE))
  expect_identical(object = GermRateRecip(germ.counts = x, intervals = int,
                                          method = "farooq"),
                   expected = GermRateRecip(germ.counts = y, intervals = int,
                                            method = "farooq",
                                            partial = FALSE))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Median germination time
#'
#' Compute the median germination time (\mjseqn{t_{50}}). Median germination
#' time is the time to reach 50\% of final/maximum germination. \loadmathjax
#'
#' With argument \code{method} specified as \code{"coolbear"}, median
#' germination time is computed according to the formula by
#' \insertCite{coolbear_effect_1984;textual}{germinationmetrics} as follows.
#'
#' \mjsdeqn{t_{50}=T_{i}+
#' \frac{(\frac{N+1}{2}-N_{i})(T_{j}-T_{i})}{N_{j}-N_{i}}}
#'
#' Where, \mjseqn{t_{50}} is the median germination time, \mjseqn{N} is the
#' final number of germinated seeds, and \mjseqn{N_{i}} and \mjseqn{N_{j}} are
#' the total number of seeds germinated in adjacent counts at time
#' \mjseqn{T_{i}} and \mjseqn{T_{j}} respectively, when \mjteqn{N_{i} < \frac{N
#' + 1}{2} < N_{j}}{N_{i} &lt; \frac{N + 1}{2} &lt; N_{j}}{N_i < ((N + 1) / 2) <
#' N_j}.
#'
#' Similarly with argument \code{method} specified as \code{"farooq"}, median
#' germination time is computed according to the formula by  by
#' \insertCite{farooq_thermal_2005;textual}{germinationmetrics} as follows.
#'
#' \mjsdeqn{t_{50}=T_{i}+ \frac{(\frac{N}{2}-N_{i})(T_{j}-T_{i})}{N_{j}-N_{i}}}
#'
#' Where, \mjseqn{t_{50}} is the median germination time, \mjseqn{N} is the
#' final number of germinated seeds, and \mjseqn{N_{i}} and \mjseqn{N_{j}} are
#' the total number of seeds germinated in adjacent counts at time
#' \mjseqn{T_{i}} and \mjseqn{T_{j}} respectively, when \mjteqn{N_{i} <
#' \frac{N}{2} < N_{j}}{N_{i} &lt; \frac{N}{2} &lt; N_{j}}{N_i < (N / 2) < N_j}.
#'
#' @inheritParams MeanGermTime
#' @param method The method for computing median germination time. Either
#'   \code{"coolbear"} or \code{"farooq"}.
#'
#' @return The median germination time (\mjseqn{t_{50}}) value in the same unit
#'   of time as specified in the argument \code{intervals}.
#'
#' @export
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' t50(germ.counts = x, intervals = int, method = "coolbear")
#' t50(germ.counts = x, intervals = int, method = "farooq")
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' t50(germ.counts = y, intervals = int, partial = FALSE, method = "coolbear")
#' t50(germ.counts = y, intervals = int, partial = FALSE, method = "farooq")
#'
#' @seealso \code{\link[germinationmetrics]{MeanGermRate}}
#'
t50 <- function(germ.counts, intervals, partial = TRUE,
                method = c("coolbear", "farooq")) {

  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts
  csx <- cumsum(x)

  # Check method
  method <- match.arg(method)

  if (method == "coolbear") {
    xhalf <- (sum(x) + 1)/2
  }

  if (method == "farooq") {
    xhalf <- sum(x)/2
  }

  if (x[1] < xhalf) {
    if (length(csx[csx > xhalf]) == 0 |
        length(csx[csx < xhalf]) == 0) {
      nearest <- c(max(which(max(csx[csx <= xhalf]) == csx)),
                   min(which(min(csx[csx >= xhalf]) == csx)))
    } else {
      nearest <- c(max(which(max(csx[csx < xhalf]) == csx)),
                   min(which(min(csx[csx > xhalf]) == csx)))
    }
    flankg <- c(csx[nearest[1]], csx[nearest[2]])
    fltime <- c(intervals[nearest[1]], intervals[nearest[2]])

    t50 <- intervals[nearest[1]] +
      ((xhalf - flankg[1]) * (fltime[2] - fltime[1])) /
      (flankg[2] - flankg[1])

  } else {
    if (method == "coolbear") {
      cmt <- "((N + 1)/2) "
    }

    if (method == "farooq") {
      cmt <- "(N/2) "
    }

    warning("'t50' cannot be computed as more than half the seeds ",
            cmt,
            "have germinated at first interval.")
    t50 <- NA_real_
  }

  return(unname(t50))

}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("t50 works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = t50(germ.counts = x, intervals = int,
                            method = "coolbear"),
               expected = 5.97058823529412, tolerance = 1e-12)
  expect_equal(object = t50(germ.counts = x, intervals = int,
                            method = "farooq"),
               expected = 5.94117647058824, tolerance = 1e-12)
})

test_that("t50 works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = t50(germ.counts = y, intervals = int,
                            partial = FALSE, method = "coolbear"),
               expected = 5.97058823529412, tolerance = 1e-12)
  expect_equal(object = t50(germ.counts = y, intervals = int,
                            partial = FALSE, method = "farooq"),
               expected = 5.94117647058824, tolerance = 1e-12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = t50(germ.counts = x, intervals = int,
                                method = "coolbear"),
                   expected = t50(germ.counts = y, intervals = int,
                                  partial = FALSE, method = "coolbear"))

  expect_identical(object = t50(germ.counts = x, intervals = int,
                                method = "farooq"),
                   expected = t50(germ.counts = y, intervals = int,
                                  partial = FALSE, method = "farooq"))

})

test_that("Identical results with multiple matches of Ni & Nj present", {
  y1 <- c(0 , 0 , 0 ,39 ,41, 41)
  int1 <- c(0, 1, 2, 3, 4, 5)
  y2 <- c( 0 , 0 ,39 ,41, 41)
  int2 <- c( 1, 2, 3, 4, 5)

  expect_identical(object = t50(germ.counts = y1, intervals = int1,
                                partial = FALSE, method = "coolbear"),
                   expected = t50(germ.counts = y2, intervals = int2,
                                  partial = FALSE, method = "coolbear"))

  expect_identical(object = t50(germ.counts = y1, intervals = int1,
                                partial = FALSE, method = "farooq"),
                   expected = t50(germ.counts = y2, intervals = int2,
                                  partial = FALSE, method = "farooq"))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Germination Index
#'
#' Compute the Germination index according to
#' \insertCite{melville_seed_1980;textual}{germinationmetrics} and
#' its modification by
#' \insertCite{de_santana_alise_2004;textual}{germinationmetrics}
#' \insertCite{ranal_how_2006}{germinationmetrics}. \loadmathjax
#'
#' Germination index (\mjseqn{GI}) is computed as follows
#' \insertCite{melville_seed_1980}{germinationmetrics}.
#'
#' \mjsdeqn{GI = \sum_{i=1}^{k}\frac{\left | \left ( T_{k} - T_{i} \right )
#' N_{i}\right |}{N_{t}}}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval (day for the example), \mjseqn{N_{i}} is the number of
#' seeds germinated in the \mjseqn{i}th time interval (not the accumulated
#' number, but the number corresponding to the \mjseqn{i}th interval),
#' \mjseqn{N_{t}} is the total number of seeds used in the test, and \mjseqn{k}
#' is the total number of time intervals.
#'
#' A modified germination index (\mjseqn{GI_{mod}}) was proposed by
#' \insertCite{de_santana_alise_2004}{germinationmetrics} where total number of
#' seeds tested (\mjseqn{N_{t}}) is replaced by the total number of germinated
#' seeds (\mjseqn{N_{g}}) in the expression of
#' \insertCite{melville_seed_1980;textual}{germinationmetrics} to
#' remove the effect of sample size
#' \insertCite{ranal_how_2006}{germinationmetrics}. It is computed as follows
#' (Specified by the argument \code{modification = "santanaranal"}).
#'
#' \mjsdeqn{GI_{mod} = \sum_{i=1}^{k}\frac{\left | \left ( T_{k} - T_{i} \right
#' ) N_{i}\right |}{N_{g}}}
#'
#' The term Germination index (\mjseqn{GI}) has also been used to describe the
#' mathematical expression for Speed of germination
#' (\code{\link[germinationmetrics]{GermSpeed}}) by
#' \insertCite{aosa_seed_1983;textual}{germinationmetrics}.
#'
#' @inheritParams MeanGermTime
#' @param modification The modification to be applied. Either \code{"none"} or
#'   \code{"santanaranal"}. Default is \code{"none"} (see \strong{Details}).
#' @param total.seeds Total number of seeds.
#'
#' @return The value of the Germination index in the same unit of time as
#'   specified in the argument \code{intervals}.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' GermIndex(germ.counts = x, intervals = int, total.seeds = 50)
#' GermIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'           modification = "none")
#' GermIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'           modification = "santanaranal")
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' GermIndex(germ.counts = y, intervals = int, partial = FALSE,
#'           total.seeds = 50)
#' GermIndex(germ.counts = y, intervals = int, partial = FALSE,
#'           total.seeds = 50,
#'           modification = "none")
#' GermIndex(germ.counts = y, intervals = int, partial = FALSE,
#'           total.seeds = 50,
#'           modification = "santanaranal")
#'
#' @seealso \code{\link[germinationmetrics]{GermSpeed}}
#'
#' @importFrom utils tail
#'
#' @rdname GermIndex
#' @export
GermIndex <- function(germ.counts, intervals, partial = TRUE,
                      total.seeds,
                      modification = c("none", "santanaranal")) {

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  # Check modification
  modification <- match.arg(modification)

  Tk <- tail(intervals, n = 1)

  if (modification == "santanaranal") {
    GI <- sum(abs((Tk - intervals) * germ.counts)/sum(germ.counts))
  } else { # modification == "none"
    GI <- sum(abs((Tk - intervals) * germ.counts)/total.seeds)
  }

  return(GI)

}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("MeanGermRate works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermIndex(germ.counts = x, intervals = int,
                                  total.seeds = 50),
               expected = 5.84, tolerance = 1e-12)
  expect_equal(object = GermIndex(germ.counts = x, intervals = int,
                                  total.seeds = 50, modification = "none"),
               expected = 5.84, tolerance = 1e-12)
  expect_equal(object = GermIndex(germ.counts = x, intervals = int,
                                  total.seeds = 50,
                                  modification = "santanaranal"),
               expected = 7.3, tolerance = 1e-12)
})

test_that("MeanGermRate works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = GermIndex(germ.counts = y, intervals = int,
                                  partial = FALSE, total.seeds = 50),
               expected = 5.84, tolerance = 1e-12)
  expect_equal(object = GermIndex(germ.counts = y, intervals = int,
                                  partial = FALSE, total.seeds = 50,
                                  modification = "none"),
               expected = 5.84, tolerance = 1e-12)
  expect_equal(object = GermIndex(germ.counts = y, intervals = int,
                                  partial = FALSE, total.seeds = 50,
                                  modification = "santanaranal"),
               expected = 7.3, tolerance = 1e-12)
})


test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = GermIndex(germ.counts = x, intervals = int,
                                      total.seeds = 50),
                   expected = GermIndex(germ.counts = y, intervals = int,
                                        partial = FALSE, total.seeds = 50))
  expect_identical(object = GermIndex(germ.counts = x, intervals = int,
                                      total.seeds = 50, modification = "none"),
                   expected = GermIndex(germ.counts = y, intervals = int,
                                        partial = FALSE, total.seeds = 50,
                                        modification = "none"))
  expect_identical(object = GermIndex(germ.counts = x, intervals = int,
                                      total.seeds = 50,
                                      modification = "santanaranal"),
                   expected = GermIndex(germ.counts = y, intervals = int,
                                        partial = FALSE, total.seeds = 50,
                                        modification = "santanaranal"))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

wlcm <- paste0("\n",
               "--------------------------------------------------------------------------------\n",
               "Welcome to germinationmetrics version ", utils::packageDescription("germinationmetrics")$Version, "\n",
               "\n", "\n",
               "# To know how to use this package type:", "\n",
               "  browseVignettes(package = 'germinationmetrics')", "\n", "  for the package vignette.", "\n",
               "\n",
               "# To know whats new in this version type:", "\n",
               "  news(package='germinationmetrics')", "\n", "  for the NEWS file.", "\n",
               "\n",
               "# To cite the methods in the package type:", "\n",
               "  citation(package='germinationmetrics')","\n",
               "\n",
               "# To suppress this message use:", "\n",
               "  suppressPackageStartupMessages(library(germinationmetrics))", "\n",
               "--------------------------------------------------------------------------------\n")

.onAttach <- function(lib, pkg,...){
  packageStartupMessage(wlcm)

}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/


#' Mean germination percentage and number of seeds per time interval
#'
#' Compute the following metrics:\describe{ \item{\code{MeanGermPercent}}{
#' Mean/average germination percentage per unit time (\mjseqn{\overline{GP}})
#' \insertCite{czabator_germination_1962}{germinationmetrics}. }
#' \item{\code{MeanGermNumber}}{Number of seeds germinated per unit time
#' (\mjseqn{\overline{N}})
#' \insertCite{khamassi_optimal_2013}{germinationmetrics}.}} \loadmathjax
#'
#' Mean germination percentage per unit time (\mjseqn{\overline{GP}}) is
#' computed as follows
#' \insertCite{czabator_germination_1962}{germinationmetrics}.
#'
#' \mjsdeqn{\overline{GP} = \frac{GP}{T_{k}}}
#'
#' Where, \mjseqn{GP} is the final germination percentage, \mjseqn{T_{k}} is the
#' time at the \mjseqn{k}th time interval, and \mjseqn{k} is the total number of
#' time intervals required for final germination.
#'
#' Mean number of seeds germinated per unit time (\mjseqn{\overline{N}}) is
#' computed as follows \insertCite{khamassi_optimal_2013}{germinationmetrics}.
#'
#' \mjsdeqn{\overline{N} = \frac{N_{g}}{T_{k}}}
#'
#' Where, \mjseqn{N_{g}} is the number of germinated seeds at the end of the
#' germination test, \mjseqn{T_{k}} is the time at the \mjseqn{k}th time
#' interval, and \mjseqn{k} is the total number of time intervals required for
#' final germination.
#'
#' @inheritParams GermPercent
#' @inheritParams MeanGermTime
#'
#' @return The value of mean germination percentage as \%
#'   \mjseqn{\mathrm{time^{-1}}} or mean number of seeds per time interval as
#'   \mjseqn{\mathrm{count \, time^{-1}}}.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name MeanGermPercent
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' MeanGermPercent(germ.counts = x, total.seeds = 50, intervals = int)
#' MeanGermNumber(germ.counts = x, intervals = int)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' MeanGermPercent(germ.counts = y, total.seeds = 50, intervals = int, partial = FALSE)
#' MeanGermNumber(germ.counts = y, intervals = int, partial = FALSE)
#'
#' # From number of germinated seeds
#' #----------------------------------------------------------------------------
#' MeanGermPercent(germinated.seeds = 40, total.seeds = 50, intervals = int)
#'
#' @seealso \code{\link[germinationmetrics]{GermPercent}}

#' @rdname MeanGermPercent
#' @export
MeanGermPercent <- function(germinated.seeds, germ.counts, total.seeds, intervals,
                            partial = TRUE) {

  gcheck <- c(missing(germinated.seeds), missing(germ.counts))

  # Check if both germinated.seeds and germ.counts are provided
  if (identical(gcheck, c(FALSE, FALSE))) {
    stop("Provide only either one of the two arguments\n'germinated.seeds' or 'germ.counts' and not both")
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  if (!missing(germinated.seeds)) { # When germinated.seeds is provided
    # Check if argument germinated.seeds is of type numeric with unit length
    if (!is.numeric(germinated.seeds) || length(germinated.seeds) != 1) {
      stop("'germinated.seeds' should be a numeric vector of length 1.")
    }
  } else {# When germ.counts is provided

    # Check if argument germ.counts is of type numeric
    if (!is.numeric(germ.counts)) {
      stop("'germ.counts' should be a numeric vector.")
    }

    # Check if argument partial is of type logical with unit length
    if (!is.logical(partial) || length(partial) != 1) {
      stop("'partial' should be a logical vector of length 1.")
    }

    # Check if data is cumulative
    if (!partial) {
      if(is.unsorted(germ.counts)) {
        stop("'germ.counts' is not cumulative.")
      }
    }

    # Convert cumulative to partial
    if (!partial) {
      germ.counts <- c(germ.counts[1], diff(germ.counts))
    }
    germinated.seeds <- sum(germ.counts)
  }

  GP <- (germinated.seeds/total.seeds)*100

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  MGP <- GP/intervals[length(intervals)]

  return(MGP)
}


#' @rdname MeanGermPercent
#' @export
MeanGermNumber <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  MGN <- sum(germ.counts)/intervals[length(intervals)]

  return(MGN)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("MeanGermPercent works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = MeanGermPercent(germ.counts = x, total.seeds = 50,
                                        intervals = int),
               expected = 5.71428571428571, tolerance = 1e-12)
})

test_that("MeanGermPercent works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = MeanGermPercent(germ.counts = y, total.seeds = 50,
                                        intervals = int, partial = FALSE),
               expected = 5.71428571428571, tolerance = 1e-12)
})

test_that("MeanGermPercent works; Number of germinated seeds", {

  # From number of germinated seeds
  expect_equal(object = MeanGermPercent(germinated.seeds = 40,
                                        total.seeds = 50, intervals = int),
               expected = 5.71428571428571, tolerance = 1e-12)
})

test_that("MeanGermNumber works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = MeanGermNumber(germ.counts = x, intervals = int),
               expected = 2.85714285714286, tolerance = 1e-12)
})

test_that("MeanGermNumber works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = MeanGermNumber(germ.counts = y, intervals = int,
                                       partial = FALSE),
               expected = 2.85714285714286, tolerance = 1e-12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = MeanGermPercent(germ.counts = x, total.seeds = 50,
                                            intervals = int),
                   expected = MeanGermPercent(germ.counts = y, total.seeds = 50,
                                              intervals = int, partial = FALSE))
  expect_identical(object = MeanGermNumber(germ.counts = x, intervals = int),
                   expected = MeanGermNumber(germ.counts = y, intervals = int,
                                             partial = FALSE))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 2 of the License, or (at your option) any
# later version.
#
# germinationmetrics is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Synchrony and uncertainty of germination
#'
#' Compute the following metrics:\describe{ \item{\code{GermSynchrony}}{
#' Synchrony of germination (\mjseqn{Z} index)
#' \insertCite{primack_longevity_1985,ranal_how_2006}{germinationmetrics}. }
#' \item{\code{GermUncertainty}}{Synchronization index (\mjseqn{\overline{E}})
#' or Uncertainty of the germination process (\mjseqn{U}) or Informational
#' entropy (\mjseqn{H})
#' \insertCite{shannon_mathematical_1948,labouriau_germination_1976,labouriau_uma_1983}{germinationmetrics}.}
#' } \loadmathjax
#'
#' \code{GermSynchrony} computes the value of synchrony of germination
#' (\mjseqn{Z} index) as follows
#' \insertCite{primack_longevity_1985,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{Z=\frac{\sum_{i=1}^{k}C_{N_{i},2}}{C_{\Sigma N_{i},2}}}
#'
#' Where, \mjseqn{C_{N_{i},2}} is the partial combination of the two germinated
#' seeds from among \mjseqn{N_{i}}, the number of seeds germinated on the
#' \mjseqn{i}th time interval (estimated as
#' \mjseqn{C_{N_{i},2}=\frac{N_{i}(N_{i}-1)}{2}}), and \mjseqn{C_{\Sigma
#' N_{i},2}} is the partial combination of the two germinated seeds from among
#' the total number of seeds germinated at the final count, assuming that all
#' seeds that germinated did so simultaneously.
#'
#' \code{GermUncertainty} computes the value of synchronization index
#' (\mjseqn{\overline{E}}) or uncertainty of the germination process
#' (\mjseqn{U}) or informational entropy (\mjseqn{H}) as follows
#' \insertCite{shannon_mathematical_1948,labouriau_germination_1976,labouriau_uma_1983}{germinationmetrics}.
#'
#' \mjsdeqn{\overline{E} = -\sum_{i=1}^{k}f_{i}\log_{2}f_{i}}
#'
#' Where, \mjseqn{f_{i}} is the relative frequency of germination (estimated as
#' \mjseqn{f_{i}=\frac{N_{i}}{\sum_{i=1}^{k}N_{i}}}), \mjseqn{N_{i}} is the
#' number of seeds germinated on the \mjseqn{i}th time interval, and \mjseqn{k}
#' is the total number of time intervals.
#'
#' @inheritParams MeanGermTime
#'
#' @return For \code{GermUncertainty}, the value of uncertainty of germination
#'   process in \mjseqn{\mathrm{bit}}.
#'
#'   For \code{GermSynchrony}, the value of synchrony of germination.
#'
#' @name GermSynchrony
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' GermSynchrony(germ.counts = x, intervals = int)
#' GermUncertainty(germ.counts = x, intervals = int)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' GermSynchrony(germ.counts = y, intervals = int, partial = FALSE)
#' GermUncertainty(germ.counts = y, intervals = int, partial = FALSE)
#'
#' @seealso \code{\link[germinationmetrics]{CUGerm}}


#' @rdname GermSynchrony
#' @export
GermSynchrony <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }
  x <- germ.counts

  Z <- sum(x*(x - 1)/2)/(sum(x)*(sum(x) - 1)/2)

  return(Z)
}

#' @rdname GermSynchrony
#' @export
GermUncertainty <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts

  relfreq <- x/sum(x)

  GU <- -sum(relfreq*log2(relfreq), na.rm = TRUE)

  return(GU)
}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Fit four-parameter hill function
#'
#' Fit a four-parameter hill function
#' \insertCite{el-kassaby_seed_2008}{germinationmetrics} to cumulative
#' germination count data and compute the associated parameters. \loadmathjax
#'
#' The cumulative germination count data of a seed lot can be modelled to fit a
#' four-parameter hill function defined as follows
#' \insertCite{el-kassaby_seed_2008}{germinationmetrics}.
#'
#' \mjsdeqn{y = y_{0}+\frac{ax^{b}}{c^{b}+x^{b}}}
#'
#' Where, \mjseqn{y} is the cumulative germination percentage at time
#' \mjseqn{x}, \mjseqn{y_{0}} is the intercept on the y axis, \mjseqn{a} is the
#' asymptote, or maximum cumulative germination percentage, which is equivalent
#' to germination capacity, \mjseqn{b} is a mathematical parameter controlling
#' the shape and steepness of the germination curve (the larger the \mjseqn{b}
#' parameter, the steeper the rise toward the asymptote \mjseqn{a}, and the
#' shorter the time between germination onset and maximum germination), and
#' \mjseqn{c} is the "half-maximal activation level" which represents the time
#' required for 50\% of viable seeds to germinate (\mjseqn{c} is equivalent to
#' the germination speed).
#'
#' In \code{FourPHFfit}, this model has be reparameterized by substituting
#' \mjseqn{b} with \mjseqn{e^{\beta}} to constraint \mjseqn{b} to positive
#' values only.
#'
#' \mjsdeqn{y = y_{0}+\frac{ax^{e^{\beta}}}{c^{e^{\beta}}+x^{e^{\beta}}}}
#'
#' Where, \mjseqn{b = e^{\beta}} and \mjseqn{\beta = \log_{e}(b)}.
#'
#' The curve fitting is performed with nonlinear
#' \code{\link[gslnls:gsl_nls]{gslnls}} package, a R interface to the
#' least-squares optimization with the GNU Scientific Library (GSL) with the
#' Levenberg-Marquardt algorithm
#' \insertCite{galassi_gnu_2009,chau_gslnls_2023}{germinationmetrics}.
#'
#' Once this function is fitted to the curve, \code{FourPHFfit} computes the
#' time to 50\% germination of total seeds (\code{t50.total}) or viable seeds
#' (\code{t50.Germinated}). Similarly the time at any percentage of germination
#' (in terms of both total and viable seeds) as specified in argument \code{xp}
#' can be computed.
#'
#' The time at germination onset (\mjseqn{lag}) can be computed as follows.
#'
#' \mjsdeqn{lag = b\sqrt{\frac{-y_{0}c^{b}}{a + y_{0}}}}
#'
#' The value \mjseqn{D_{lag-50}} is defined as the duration between the time at
#' germination onset (lag) and that at 50\% germination (\mjseqn{c}).
#'
#' The time interval between the percentages of viable seeds specified in the
#' arguments \code{umin} and \code{umin} to germinate is computed as uniformity
#' (\mjseqn{U_{t_{max}-t_{min}}}).
#'
#' \mjsdeqn{U_{t_{max}-t_{min}} = t_{max} - t_{min}}
#'
#' The partial derivative of the four-parameter hill function gives the
#' instantaneous rate of germination (\mjseqn{s}) as follows.
#'
#' \mjsdeqn{s = \frac{\partial y}{\partial x} =
#' \frac{abc^{b}x^{b-1}}{(c^{b}+x^{b})^{2}}}
#'
#' From this function for instantaneous rate of germination, the time at maximum
#' germination rate (\mjseqn{TMGR}) can be estimated as follows.
#'
#' \mjsdeqn{TMGR = b \sqrt{\frac{c^{b}(b-1)}{b+1}}}
#'
#' TMGR represents the point in time when the instantaneous rate of germination
#' starts to decline.
#'
#' The area under the curve (\mjseqn{AUC}) is obined by integration of the
#' fitted curve between time 0 and time specified in the argument `tmax`.
#'
#' Integration of the fitted curve gives the value of mean germination time
#' (\mjseqn{MGT}) and the skewness of the germination curve is computed as the
#' ratio of \mjseqn{MGT} and the time for 50\% of viable seeds to germinate
#' (\mjseqn{t_{50}}).
#'
#' \mjsdeqn{Skewness = \frac{MGT}{t_{50}}}
#'
#' If final germination percentage is less than 10\%, a warning is given, as the
#' results may not be informative.
#'
#' @inheritParams MeanGermTime
#' @param total.seeds Total number of seeds.
#' @param fix.y0 Force the intercept of the y axis through 0.
#' @param fix.a Fix a as the actual maximum germination percentage at the end of
#'   the experiment.
#' @param tmax The time up to which AUC is to be computed.
#' @param xp Germination percentage value(s) for which the corresponding time is
#'   to be computed as a numeric vector. Default is \code{c(10, 60)}.
#' @param umin The minimum germination percentage value for computing
#'   uniformity. Default is \code{10}. See \strong{\code{Details}}.
#' @param umax The maximum germination percentage value for computing
#'   uniformity. Default is \code{90}. See \strong{\code{Details}}.
#' @param tries The number of tries to be attempted to fit the curve. Default is
#'   \code{3}.
#'
#' @return A list with the following components: \item{data}{A data frame with
#'   the data used for computing the model.} \item{Parameters}{A data frame of
#'   parameter estimates, standard errors and p value.}  \item{Fit}{A one-row
#'   data frame with estimates of model fitness such as log likelyhoods, Akaike
#'   Information Criterion, Bayesian Information Criterion, deviance and
#'   residual degrees of freedom.}  \item{a}{The asymptote or the maximum
#'   cumulative germination percentage.}  \item{b}{The mathematical parameter
#'   controlling the shape and steepness of the germination curve.} \item{c}{The
#'   half-maximal activation level.}  \item{y0}{The intercept on the y axis.}
#'   \item{lag}{Time at germination onset.} \item{Dlag50}{duration between the
#'   time at germination onset (lag) and that at 50\% germination.}
#'   \item{t50.total}{Time required for 50\% of total seeds to germinate. Will
#'   be \code{NaN} if more than 50\% of total seeds do not germinate.}
#'   \item{txp.total}{Time required for x\% (as specified in argument \code{xp})
#'   of total seeds to germinate. Will be \code{NaN} if more than x\% of total
#'   seeds do not germinate.} \item{t50.Germinated}{Time required for 50\%
#'   of viable/germinated seeds to germinate.} \item{txp.Germinated}{Time
#'   required for x\% (as specified in argument \code{xp}) of viable/germinated
#'   seeds to germinate.} \item{Uniformity}{Time interval between \code{umin}\%
#'   and \code{umax}\% of viable seeds to germinate.} \item{TMGR}{Time at
#'   maximum germination rate.} \item{AUC}{The estimate of area under the
#'   curve.} \item{MGT}{Mean germination time.} \item{Skewness}{Skewness of mean
#'   germination time.} \item{msg}{The message from
#'   \code{\link[gslnls]{gsl_nls}}.} \item{isConv}{Logical value indicating
#'   whether convergence was achieved.}
#'   \item{model}{The raw fitted model output as a list of class
#'   \code{gsl_nls}.}
#'
#' @import gslnls
#' @importFrom broom glance
#' @importFrom broom tidy
#' @importFrom methods is
#' @importFrom plyr round_any
#' @importFrom stats coef df integrate predict
#' @importFrom utils globalVariables
#'
#' @name FourPHFfit
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#'
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#' total.seeds = 50
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' FourPHFfit(germ.counts = x, intervals = int, total.seeds = 50, tmax = 20)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' FourPHFfit(germ.counts = y, intervals = int, total.seeds = 50, tmax = 20,
#'            partial = FALSE)
#'
#' @export
FourPHFfit <- function(germ.counts, intervals, total.seeds, partial = TRUE,
                       fix.y0 = TRUE, fix.a = TRUE, tmax, xp = c(10, 60),
                       umin = 10, umax = 90, tries = 3) {

  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check GP
  GP <- GermPercent(germ.counts = germ.counts, total.seeds = total.seeds,
                    partial = partial)

  if (GP == 0) {
    warning("Final germination percentage is 0%.\n",
            "The computation is not possible.")
  } else {
    if (GP < 10) {
      warning("Final germination percentage is less than 10%.\n",
              "The computation may not be appropriate.")
    }
  }

  # Check if umax is of type numeric with unit length
  if (!is.numeric(umax) || length(umax) != 1) {
    stop("'umax' should be a numeric vector of length 1.")
  }

  # Check if umin is of type numeric with unit length
  if (!is.numeric(umin) || length(umin) != 1) {
    stop("'umin' should be a numeric vector of length 1.")
  }

  # Check if argument xp is of type numeric
  if (!is.numeric(xp)) {
    stop("'xp' should be a numeric vector.")
  }

  # check if 0 < umax < 100
  if (umax > 100 || umax < 0) {
    stop('"umax" is not within range',
         ' (0 <= "umax" <= 100).')

    # check if 0 < umin < 100
    if (umin > 100 || umin < 0) {
      stop('"umin" is not within range',
           ' (0 <= "umin" <= 100).')
      # check if umax < umin
      if (umin > umax) {
        stop('"umin" is greater than "umax"')
      }

      if (umin == umax) {
        stop('"umin" and "umax" have same values')
      }

    }
  }

  # check if 0 < xp < 100
  if (any(!findInterval(xp, c(0, 100),
                        rightmost.closed = TRUE) == 1)) {
    stop('Values in "xp" are not within range',
         ' (0 <= "xp" <= 100).')
  }

  # name xp
  names(xp) <- xp

  # Check if tmax is of type numeric with unit length
  if (!is.numeric(tmax) || length(tmax) != 1) {
    stop("'tmax' should be a numeric vector of length 1.")
  }

  # Check if tmax is > 0
  if (tmax < 0) {
    stop("'tmax' should be greater than 0.")
  }

  # Check if tries is of type numeric with unit length
  if (!is.numeric(tries) || length(tries) != 1) {
    stop("'tries' should be a numeric vector of length 1.")
  }

  # Check if tries is > 0
  if (tries < 0) {
    stop("'tries' should be greater than 0.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  gp <- (germ.counts/total.seeds) * 100
  csgp <- cumsum(gp)

  df <- data.frame(gp, csgp, intervals)

  # Blank output
  parameters <- data.frame(term = c("a", "b", "c", "y0"),
                           estimate = rep(NA_integer_),
                           std.error = rep(NA_integer_, 4),
                           statistic = rep(NA_integer_, 4),
                           p.value = rep(NA_integer_, 4))

  fit <- data.frame(sigma = NA_integer_, isConv = NA, finTol = NA_integer_,
                    logLik = NA_integer_, AIC = NA_integer_, BIC = NA_integer_,
                    deviance = NA_integer_, df.residual = NA_integer_,
                    nobs = NA_integer_)

  a <- NA_integer_
  b <- NA_integer_
  c <- NA_integer_
  y0 <- NA_integer_
  lag <- NA_integer_
  Dlag50 <- NA_integer_
  t50.total <- NA_integer_
  t50.Germinated <- NA_integer_
  TMGR <- NA_integer_
  AUC <- NA_integer_
  MGT <- NA_integer_
  #MGTg <- NA_integer_
  Skewness <- NA_integer_
  isConv <- NA
  mod <- NULL

  uniformity = c(NA_integer_, NA_integer_, NA_integer_)
  names(uniformity) <- c(umax, umin, "uniformity")

  txp.total <-  txp.Germinated <- rep(NA_integer_, length(xp))
  names(txp.total) <- xp
  names(txp.Germinated) <- xp

  if (GP > 0) { # perform fit only if GP > 0

    peakg <- suppressWarnings(PeakGermTime(germ.counts, intervals,
                                           partial = TRUE))

    # Starting values for nls
    starta <- max(csgp)
    # startb <- 20
    startb  <- CVG(df$csgp, df$intervals, partial = TRUE)
    if (length(peakg) == 1 && peakg == 1 &&
        df[which(df$intervals == 1), ]$csgp >= 90) {
      startb <- 20
    }
    if (mean(peakg) <= 3) {
      startb <- 20
    }
    if (csgp[1] >= 10 || csgp[2] >= 10 && csgp[3] >= 60) {
      startb <- 10
    }
    startbta <- log(startb, base = exp(1))
    startc <- t50(germ.counts, intervals, partial = TRUE, method = "coolbear")
    starty0 <- 0

    # test starting values
    tryStart <-
      FourPHF_withWE(data = df,
                     fix.a = fix.a, fix.y0 = fix.y0,
                     starta = starta, startbta = startbta,
                     startc = startc, starty0 = starty0,
                     maxiter = 1024)
    # To check for singular gradient matrix as in gsl_nls only a warning is
    # shown instead of an error in gsl_nls and minpack.lm
    tryStart_summ <-  tryCatch(summary(tryStart),
                               error = function(e) e)

    # Test grid of starting values
    if (inherits(tryStart, "error") |
        inherits(tryStart_summ, "error")) {
      startgrid <- expand.grid(starta = starta,
                               startb = c(0.0000001, 2, 5,
                                          seq(from = 10, to = 100, by = 10)),
                               startc = startc, starty0 = starty0)
      startgrid$startbta <- log(startgrid$startb, base = exp(1))

      nls_fitlist <- vector("list", nrow(startgrid))

      for (i in 1:nrow(startgrid)) {

        nls_fitlist[[i]] <-
          FourPHF_withWE(data = df,
                         fix.a = fix.a, fix.y0 = fix.y0,
                         starta = startgrid[i, "starta"],
                         startbta = startgrid[i, "startbta"],
                         startc = startgrid[i, "startc"],
                         starty0 = startgrid[i, "starty0"], maxiter = 1024)
      }


      isnls <- unlist(lapply(nls_fitlist, function(x) is(x, "nls")))

      # Workaround for gls_nls summary error
      # when there is singular gradient matrix at parameter estimates
      isnlssumm <- unlist(lapply(nls_fitlist, function(x) {
        out <- tryCatch(summary(x), error = function(e) e)
        inherits(out, "error")
      }))

      isnls <- isnls & (!isnlssumm)

      if (any(isnls)) {
        nls_fitlist <- nls_fitlist[isnls]

        nls_fitdf <-
          lapply(nls_fitlist, function(x) {
            cdf <- data.frame(t(coef(x)))
            colnames(cdf) <- paste("start", colnames(cdf), sep = "")
            cbind(cdf, broom::glance(x))
          })

        nls_fitdf <- do.call(rbind, nls_fitdf)
        nls_fitdf <- nls_fitdf[nls_fitdf$isConv == TRUE, ]
        sel_ind <- which.min(nls_fitdf$deviance)

        starta <- ifelse("starta" %in% colnames(nls_fitdf),
                         nls_fitdf[sel_ind, "starta"],
                         starta)
        startbta <- ifelse("startbta" %in% colnames(nls_fitdf),
                           nls_fitdf[sel_ind, "startbta"],
                           startbta)
        startc <- ifelse("startc" %in% colnames(nls_fitdf),
                         nls_fitdf[sel_ind, "startc"],
                         startc)
        starty0 <- ifelse("starty0" %in% colnames(nls_fitdf),
                          nls_fitdf[sel_ind, "starty0"],
                          starty0)
      }

    }

    msg <- ""

    # lowera <- lowerb <- lowerc <- lowery0 <- -Inf
    # uppera <- upperb <- upperc <- uppery0 <- Inf
    #
    # if (TRUE %in% c(fix.y0, fix.a)) {
    #
    #   if (fix.y0 == TRUE) {
    #     lowery0 <- uppery0 <- 0
    #   }
    #   if (fix.a == TRUE) {
    #     lowera <- uppera <- max(csgp)
    #   }
    # }
    #
    # # test starting values
    # possibleError <- tryCatch(
    #
    #   nlsLM(
    #     csgp ~ FourPHF(x = intervals, a, b, c, y0),
    #     data = df,
    #     start = list(a = starta, b = startb, c = startc, y0 = starty0),
    #     lower = c(a = lowera, b = lowerb, c = lowerc, y0 = lowery0),
    #     upper = c(a = uppera, b = upperb, c = upperc, y0 = uppery0),
    #     control = list(maxiter = 1024, warnOnly = FALSE)),
    #
    #   error = function(e) e)
    #
    # if (inherits(possibleError, "error") & mean(peakg) <= 3) {
    #   csgp2 <- c(csgp, rep(max(csgp), 20))
    #   intervals2 <- seq_along(csgp2)
    #   df2 <- data.frame(csgp2, intervals2)
    #
    #   # grd <- data.frame(a=c(starta, starta),
    #   #                   b=c(55,85),
    #   #                   c=c(startc-1, startc+1),
    #   #                   y0=c(starty0, starty0))
    #
    #   # Modify start values v1
    #   tmod <- tryCatch(
    #
    #     suppressWarnings(nlsLM(
    #       csgp2 ~ FourPHF(x = intervals2, a, b, c, y0),
    #       data = df2,
    #       start = list(a = starta, b = startb, c = startc, y0 = starty0),
    #       lower = c(a = lowera, b = lowerb, c = lowerc, y0 = lowery0),
    #       upper = c(a = uppera, b = upperb, c = upperc, y0 = uppery0),
    #       control = list(maxiter = 1024, warnOnly = FALSE))),
    #
    #     # suppressWarnings(nls2(csgp ~ FourPHF(x = intervals, a, b, c, y0),
    #     #      data = df,
    #     #      start = grd,
    #     #      algorithm = "random",
    #     #      control = nls.control(maxiter = 20000), all = TRUE)),
    #
    #     error = function(e) e)
    #
    #   if (!inherits(tmod, "error")) {
    #
    #     tmod2 <- tryCatch(
    #
    #       suppressWarnings(nlsLM(
    #         csgp ~ FourPHF(x = intervals, a, b, c, y0),
    #         data = df,
    #         start = list(a = starta, b = startb, c = startc, y0 = starty0),
    #         lower = c(a = lowera, b = lowerb, c = lowerc, y0 = lowery0),
    #         upper = c(a = uppera, b = upperb, c = upperc, y0 = uppery0),
    #         control = list(maxiter = 1024, warnOnly = FALSE))),
    #
    #       error = function(e) e)
    #
    #     tmod3 <- tryCatch(
    #
    #       suppressWarnings(nlsLM(
    #         csgp ~ FourPHF(x = intervals, a, b, c, y0),
    #         data = df,
    #         start = list(a = starta, b = startb/2, c = startc, y0 = starty0),
    #         lower = c(a = lowera, b = lowerb, c = lowerc, y0 = lowery0),
    #         upper = c(a = uppera, b = upperb, c = upperc, y0 = uppery0),
    #         control = list(maxiter = 1024, warnOnly = FALSE))),
    #
    #       error = function(e) e)
    #
    #     if (!inherits(tmod2, "error")) {
    #       startb <-  unname(coef(tmod2)["b"])
    #       # startc <-  unname(coef(tmod)["c"])
    #     }
    #
    #     if (!inherits(tmod3, "error")) {
    #       startb <-  unname(coef(tmod3)["b"])
    #     }
    #
    #   }
    #
    # }
    # rm(possibleError)

    #===========================================================================
    # Fit the model - using gsl_nls
    #------------------------------
    for (i in 1:tries) {
      # check if convergence possible with start values
      possibleError <-
        FourPHF_withWE(data = df,
                       fix.a = fix.a, fix.y0 = fix.y0,
                       starta = starta, startbta = startbta,
                       startc = startc, starty0 = starty0,
                       maxiter = 1024)

      if (!inherits(possibleError, "error") &&
          possibleError$convInfo$isConv == TRUE) {

        mod <- possibleError
        rm(possibleError)

        filler <- ifelse(i == 1, "#", "\n#")
        msg <- paste(msg, filler, i, ". ", mod$convInfo$stopMessage, " ",
                     sep = "")
        break # break out of loop if convergence is successful

      } else {# In case of convergence failure with initial start values

        # suppress warnings and rerun above with warnOnly = TRUE

        mod <-
          FourPHF_withWE(data = df,
                         fix.a = fix.a, fix.y0 = fix.y0,
                         starta = starta, startbta = startbta,
                         startc = startc, starty0 = starty0,
                         maxiter = 1024, warnOnly = TRUE)

        # Extract convergence msg
        if (!inherits(mod, "error")) {

          filler <- ifelse(i == 1, "#", "\n#")
          msg <- paste(msg, filler, i, ". ", mod$convInfo$stopMessage,
                       " ", sep = "")

          # Get new start values
          starta <- coef(mod)["a"]
          startbta <- coef(mod)["bta"]
          startc <- coef(mod)["c"]
          starty0 <- 0

        } else {

          filler <- ifelse(i == 1, "#", "\n#")
          msg <- paste(msg, filler, i, ". ", mod$message,
                       " ", sep = "")

          break # break out of loop if convergence is NOT successful
        }

      }
    }

    # "exceeded max number of iterations"
    if (mod$convInfo$stopCode == 11) {

      starta <- ifelse(is.na(coef(mod)["a"]),
                       starta,
                       coef(mod)["a"])
      startbta <- ifelse(is.na(coef(mod)["bta"]),
                         startbta,
                         coef(mod)["bta"])
      startc <- ifelse(is.na(coef(mod)["c"]),
                       startc,
                       coef(mod)["c"])
      starty0 <- ifelse(is.na(coef(mod)["y0"]),
                        starty0,
                        coef(mod)["y0"])

      mod <- FourPHF_withWE(data = df,
                            fix.a = fix.a, fix.y0 = fix.y0,
                            starta = starta, startbta = startbta,
                            startc = startc, starty0 = starty0,
                            maxiter = 2048, algorithm = "lm")
      i = i + 1
      msg <- paste(msg, "\n#", i, ". ", mod$convInfo$stopMessage,
                   " ", sep = "")
    }

    # "iteration is not making progress towards solution"
    if (mod$convInfo$stopCode == 27) {

      starta <- ifelse(is.na(coef(mod)["a"]),
                       starta,
                       coef(mod)["a"])
      startbta <- ifelse(is.na(coef(mod)["bta"]),
                         startbta,
                         coef(mod)["bta"])
      startc <- ifelse(is.na(coef(mod)["c"]),
                       startc,
                       coef(mod)["c"])
      starty0 <- ifelse(is.na(coef(mod)["y0"]),
                        starty0,
                        coef(mod)["y0"])

      mod <- FourPHF_withWE(data = df,
                            fix.a = fix.a, fix.y0 = fix.y0,
                            starta = starta, startbta = startbta,
                            startc = startc, starty0 = starty0,
                            maxiter = 2048, algorithm = "lmaccel")
      i = i + 1
      msg <- paste(msg, "\n#", i, ". ", mod$convInfo$stopMessage,
                   " ", sep = "")
    }

    #===========================================================================

    if (!inherits(mod, "error")) {
      isConv <-  mod$convInfo$isConv

      # Parameters estimates
      parameters <- tidy(mod)

      # Fit quality
      fit <- glance(mod)

      # Asymptote or maximum cumulative germination percentage
      if (fix.a) {
        a <- max(csgp)
      } else {
        a <- unname(unlist(parameters[parameters$term == "a", "estimate"]))
      }
      # Shape and steepness
      bta <- unname(unlist(parameters[parameters$term == "bta", "estimate"]))
      b <- exp(bta)
      # Half-maximal activation level
      # Time required for 50% of viable seeds to germinate
      c <- unname(unlist(parameters[parameters$term == "c", "estimate"]))
      # Intercept on the y axis
      if (fix.y0) {
        y0 <- 0
      } else {
        y0 <- unname(unlist(parameters[parameters$term == "y0", "estimate"]))
      }

      # Time at germination onset (lag); will be 0 if y0 is constrained to zero
      lag <- ((-y0*(c^b))/(a + y0))^(1/b)

      # Duration between the time at germination onset (lag)
      # and that at 50% germination (c)
      Dlag50 <- c - lag

      # #-----------------------------------------------------------------------
      # findInt <- function(model, value) {
      #   function(x) {
      #     predict(mod, data.frame(intervals = x), type="response") - value
      #   }
      # }
      #
      # txp.total <- uniroot(findInt(model, xp), range(intervals))$root
      #
      # t50.total <- uniroot(findInt(model, 50), range(intervals))$root

      # txp.Germinated <- uniroot(findInt(model, (xp * max(csgp)/100)),
      #                           range(intervals))$root

      # #-----------------------------------------------------------------------

      # Time required for 50% of total seeds to germinate
      t50.total <- (((((50 - y0)/a))*(c^b))/(1 - (((50 - y0)/a))))^(1/b)

      # Time required for x% of total seeds to germinate
      txp.total <- (((((xp - y0)/a))*(c^b))/(1 - (((xp - y0)/a))))^(1/b)


      # Time required for 50%  of germinated/viable seeds to germinate
      t50.Germinated <- (((50 - y0)/100*(c^b))/(1 - ((50 - y0)/100)))^(1/b)

      # Time required for x%  of germinated/viable seeds to germinate
      # or time to reach x%  of germination
      txp.Germinated <- (((xp - y0)/100*(c^b))/(1 - ((xp - y0)/100)))^(1/b)

      # Uniformity
      UfmMax <- (((umax - y0)/100*(c^b))/(1 - ((umax - y0)/100)))^(1/b)
      UfmMin <- (((umin - y0)/100*(c^b))/(1 - ((umin - y0)/100)))^(1/b)
      Ufm <- UfmMax - UfmMin
      uniformity = c(max = UfmMax, min = UfmMin, uniformity = Ufm)
      names(uniformity) <- c(umax, umin, "uniformity")

      # Time at maximum germination rate
      # ie. peak of plot of daily rate of germination
      TMGR <- (((c^b)*(b - 1))/(b + 1))^(1/b)

      # Area under the curve - add argument max time
      AUC <- integrate(FourPHF, lower = 0, upper = tmax,
                       a = a, bta = bta, c = c, y0 = y0)$value

      # Mean Germination time
      FourPHFMGT <- function(x, a, b, c, y0)
      {
        ((y0 + a * (x ^ b) / (c ^ b + x ^ b )) -
           (y0 + a * ((x - 1) ^ b)/(c ^ b + (x - 1) ^ b))) * x
      }

      MGT <- integrate(FourPHFMGT, lower = 1, upper = max(intervals),
                       a = a, b = b, c = c, y0 = y0)$value / a

      #-------------------------------------------------------------------------
      # Mean Germination time
      # tmax <- max(intervals)
      # x <- 1
      # MGTg1 <- (y0 + ((a * ((x / 1000)^b)) /
      #                   ((c^b) + (((x - 1) / 1000)^b)))) * 0.001
      # x <- 2:(1000 * tmax)
      # z <- (y0 + ((a * ((x / 1000)^b)) / ((c^b) + (((x - 1) / 1000)^b))) -
      #         (y0 + ((a * (((x - 1) / 1000)^b)) /
      #                  ((c^b) + (((x - 2) / 1000)^b))))) * (x / 1000)
      # MGTg <- sum(c(MGTg1, z)) / a
      # rm(x, MGTg1, z)
      #-------------------------------------------------------------------------
      # plot(df$intervals, df$csgp, xlab = "Time", ylab = "Germination (%)")
      # abline(h = 50)
      # abline(v = c)
      # lines(predict(mod)~df$intervals, col="red")
      #
      # abline(h = xp, col="green")
      # abline(v = txp.total, col="green")
      # abline(v = t50.total , col="red")
      #
      # abline(v = TMGR, col = "blue")
      # curve(RateofGerm(x, a=a, b=b, c=c), xlim = c(0, 13), type = "l",
      #       add = TRUE, col = "blue")
    } else {

      warning("Convergence Error\n", paste(msg, collapse = "\n"))
    }


  } else {

    msg <- paste("Final germination percentage is 0%.",
                 "The computation is not possible.", sep = " ")

  }

  output <- list(data = df, Parameters = data.frame(parameters),
                 Fit = as.data.frame(fit),
                 a = a, b = b, c = c, y0 = y0,
                 lag = lag, Dlag50 = Dlag50,
                 t50.total = t50.total, txp.total = txp.total,
                 t50.Germinated = t50.Germinated,
                 txp.Germinated = txp.Germinated,
                 Uniformity = uniformity, TMGR = TMGR, AUC = AUC, MGT = MGT,
                 #MGTg = MGTg,
                 Skewness = MGT/t50.Germinated,
                 msg = msg, isConv = isConv, model = mod)

  # Set Class
  class(output) <- c("FourPHFfit", class(output))

  return(output)
  rm(mod)

}

#' Four paramter hill function
#'
#' To be used by
#' \code{\link[germinationmetrics]{FourPHFfit}}.
#'
#' @param x The explanatory/independent variable value.
#' @param a Parameter \mjseqn{a}.
#' @param bta Parameter \mjseqn{\beta} where \mjseqn{b = e^{\beta}}.
#' @param b Parameter \mjseqn{b}.
#' @param c Parameter \mjseqn{c}.
#' @param y0 Parameter \mjseqn{y_{0}}.
#'
#' @return The calculated response/dependent value value.
#'
#' @export
#'
#'
FourPHF <- function(x, a, bta, c, y0)
{
  y0 + ((a * (x ^ exp(bta))) /
          (c ^ exp(bta) + x ^ exp(bta))) # b = exp(bta) to enforce +ive slope
}

#' @rdname FourPHF
#' @export
# FPHF with a fixed to max(csgp)
FourPHF_fixa <- function(x, a = 100, bta, c, y0)
{
  y0 + ((a * (x ^ exp(bta))) /
          (c ^ exp(bta) + x ^ exp(bta)))
}

#' @rdname FourPHF
#' @export
# FPHF with y0 fixed to 0
FourPHF_fixy0 <- function(x, a, bta, c)
{
  0 + ((a * (x ^ exp(bta))) /
         (c ^ exp(bta) + x ^ exp(bta)))
}

#' @rdname FourPHF
#' @export
# FPHF with a fixed to max(csgp) & y0 fixed to 0
FourPHF_fixa_fixy0 <- function(x, a = 100, bta, c)
{
  0 + ((a * (x ^ exp(bta))) /
         (c ^ exp(bta) + x ^ exp(bta)))
}

#' @rdname FourPHF
#' @export
# Daily rate of germination function - partial derivative of 4PHF
RateofGerm <- function(x, a, b, c) {
  (a * b * (c ^ b) * (x ^ (b - 1))) /
    (((c ^ b) + (x ^ b)) ^ 2)
}


FourPHF_withWE <- function(data,
                           fix.a, fix.y0,
                           starta, startbta, startc, starty0,
                           maxiter, algorithm = "lm", warnOnly = FALSE) {
  tryCatch(

    if (TRUE %in% c(fix.y0, fix.a)) {

      if (fix.a == FALSE & fix.y0 == TRUE) {

        gsl_nls(
          csgp ~ FourPHF_fixy0(x = intervals, a, bta, c),
          data = data,
          algorithm = algorithm,
          start = list(a = starta, bta = startbta, c = startc),
          control = list(maxiter = maxiter, warnOnly = warnOnly,
                         scale = "levenberg"))

      }

      if (fix.a == TRUE & fix.y0 == FALSE) {

        gsl_nls(
          csgp ~ FourPHF_fixa(x = intervals, a = max(csgp), bta, c, y0),
          data = data,
          algorithm = algorithm,
          start = list(bta = startbta, c = startc, y0 = starty0),
          control = list(maxiter = maxiter, warnOnly = warnOnly,
                         scale = "levenberg"))

      }

      if (fix.a == TRUE & fix.y0 == TRUE) {

        gsl_nls(
          csgp ~ FourPHF_fixa_fixy0(x = intervals, a = max(csgp),
                                    bta, c),
          data = data,
          algorithm = algorithm,
          start = list(bta = startbta, c = startc),
          control = list(maxiter = maxiter, warnOnly = warnOnly,
                         scale = "levenberg"))
      }

    } else {

      gsl_nls(
        csgp ~ FourPHF(x = intervals, a, bta, c, y0),
        data = data,
        algorithm = algorithm,
        start = list(a = starta, bta = startbta,
                     c = startc, y0 = starty0),
        control = list(maxiter = maxiter, warnOnly = warnOnly,
                       scale = "levenberg"))

    },

    error = function(e) e)
}

Sys.setenv(R_TESTS="")

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)
total.seeds = 50

cols1 <- c("a", "b", "c", "y0", "lag", "Dlag50",
           "t50.total", "t50.Germinated",
           "TMGR", "AUC", "MGT", "Skewness")

fit_partial_fixa_fixy0 <-  FourPHFfit(germ.counts = x, intervals = int,
                                      total.seeds = 50,
                                      tmax = 20)
fit_cumulative_fixa_fixy0 <-  FourPHFfit(germ.counts = y, intervals = int,
                                         total.seeds = 50, tmax = 20,
                                         partial = FALSE)

test_that("FourPHFfit works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = fit_partial_fixa_fixy0[cols1],
               expected = list(a = 80, b = 9.88193689219798,
                               c = 6.03495355423453,
                               y0 = 0, lag = 0, Dlag50 = 6.03495355423453,
                               t50.total = 6.3551214973865,
                               t50.Germinated = 6.03495355423453,
                               TMGR = 5.91219440464896,
                               AUC = 1108.97550938733,
                               MGT = 6.63225196627282,
                               Skewness = 1.09897315806502),
               tolerance = 1e-6)
  expect_equal(object = fit_partial_fixa_fixy0[["txp.total"]],
               expected = c(`10` = 4.95626430994715, `60` = 6.7445983463311),
               tolerance = 1e-6)
  expect_equal(object = fit_partial_fixa_fixy0[["txp.Germinated"]],
               expected = c(`10` = 4.83180737938015, `60` = 6.28772357367537),
               tolerance = 1e-6)
  expect_equal(object = fit_partial_fixa_fixy0[["Uniformity"]],
               expected = c(`90` = 7.53768963497883, `10` = 4.83180737938015,
                            uniformity = 2.70588225559868 ),
               tolerance = 1e-6)
})

test_that("FourPHFfit works; Partial germination counts", {

  # From cumulative germination counts
  expect_equal(object = fit_cumulative_fixa_fixy0[cols1],
               expected = list(a = 80, b = 9.88193689219798,
                               c = 6.03495355423453,
                               y0 = 0, lag = 0, Dlag50 = 6.03495355423453,
                               t50.total = 6.3551214973865,
                               t50.Germinated = 6.03495355423453,
                               TMGR = 5.91219440464896,
                               AUC = 1108.97550938733,
                               MGT = 6.63225196627282,
                               Skewness = 1.09897315806502),
               tolerance = 1e-6)
  expect_equal(object = fit_cumulative_fixa_fixy0[["txp.total"]],
               expected = c(`10` = 4.95626430994715, `60` = 6.7445983463311),
               tolerance = 1e-6)
  expect_equal(object = fit_cumulative_fixa_fixy0[["txp.Germinated"]],
               expected = c(`10` = 4.83180737938015, `60` = 6.28772357367537),
               tolerance = 1e-6)
  expect_equal(object = fit_cumulative_fixa_fixy0[["Uniformity"]],
               expected = c(`90` = 7.53768963497883, `10` = 4.83180737938015,
                            uniformity = 2.70588225559868 ),
               tolerance = 1e-6)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = fit_partial_fixa_fixy0[cols1],
                   expected = fit_cumulative_fixa_fixy0[cols1])
  expect_identical(object = fit_partial_fixa_fixy0[["txp.total"]],
                   expected = fit_cumulative_fixa_fixy0[["txp.total"]])
  expect_identical(object = fit_partial_fixa_fixy0[["txp.Germinated"]],
                   expected = fit_cumulative_fixa_fixy0[["txp.Germinated"]])
  expect_identical(object = fit_partial_fixa_fixy0[["Uniformity"]],
                   expected = fit_cumulative_fixa_fixy0[["Uniformity"]])
})


z <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

test_that("Warning generation when germination % is 0 or less than 10%", {

  expect_warning(object = FourPHFfit(germ.counts = z, intervals = int,
                                     total.seeds = 50,
                                     tmax = 20),
                 regexp = "Final germination percentage is 0%.")
  expect_warning(object = FourPHFfit(germ.counts = z, intervals = int,
                                     total.seeds = 50, tmax = 20,
                                     partial = FALSE),
                 regexp = "Final germination percentage is 0%.")


})

m <- c(0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0)
n <- c(0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)

test_that("Warning generation when germination % is 0 or less than 10%", {

  expect_warning(object = FourPHFfit(germ.counts = m, intervals = int,
                                     total.seeds = 50,
                                     tmax = 20),
                 regexp = "Final germination percentage is less than 10%.")
  expect_warning(object = FourPHFfit(germ.counts = n, intervals = int,
                                     total.seeds = 50, tmax = 20,
                                     partial = FALSE),
                 regexp = "Final germination percentage is less than 10%.")


})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/


#' Coefficient of uniformity of germination
#'
#' Compute the Coefficient of uniformity of germination (CUG).
#'
#' CUG is computed as follows
#' \insertCite{heydecker_seed_1972,bewley_seeds_1994}{germinationmetrics}.
#' \loadmathjax
#'
#' \mjsdeqn{CUG =
#' \frac{\sum_{i=1}^{k}N_{i}}{\sum_{i=1}^{k}(\overline{T}-T_{i})^{2}N_{i}}}
#'
#' Where, \mjseqn{\overline{T}} is the the mean germination time, \mjseqn{T_{i}}
#' is the time from the start of the experiment to the \mjseqn{i}th interval,
#' \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th time
#' interval (not the accumulated number, but the number corresponding to the
#' \mjseqn{i}th interval), and \mjseqn{k} is the total number of time intervals.
#'
#' @inheritParams MeanGermTime
#'
#' @return The value of the coefficient of uniformity of germination as
#'   \mjseqn{\mathrm{time^{-2}}}.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @import mathjaxr
#' @export
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' CUGerm(germ.counts = x, intervals = int)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' CUGerm(germ.counts = y, intervals = int, partial = FALSE)
#'
#' @seealso \code{\link[germinationmetrics]{GermSynchrony}},
#'   \code{\link[germinationmetrics]{MeanGermTime}}
#'
CUGerm <- function(germ.counts, intervals, partial = TRUE) {

  MGT <- MeanGermTime(germ.counts, intervals, partial)

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  CUG <- sum(germ.counts)/sum((((MGT - intervals)^2)*germ.counts))

  return(CUG)

}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("CUGerm works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = CUGerm(germ.counts = x, intervals = int),
               expected = 0.709219858156028, tolerance = 1e+12)
})

test_that("CUGerm works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = CUGerm(germ.counts = y, intervals = int,
                               partial = FALSE),
               expected = 0.709219858156028, tolerance = 1e+12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = CUGerm(germ.counts = x, intervals = int),
                   expected = CUGerm(germ.counts = y, intervals = int,
                                     partial = FALSE))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Emergence Rate Index or Germination Rate Index
#'
#' Compute the Emergence rate index or Germination rate index according to
#' \itemize{ \item
#' \insertCite{shmueli_emergence_1971;textual}{germinationmetrics} and
#' it's modification by
#' \insertCite{de_santana_alise_2004;textual}{germinationmetrics}
#' \insertCite{ranal_how_2006}{germinationmetrics}; \item
#' \insertCite{bilbro_soil_1982;textual}{germinationmetrics}; and
#' \item
#' \insertCite{fakorede_relation_1980;textual}{germinationmetrics},
#' \insertCite{fakorede_variability_1981;textual}{germinationmetrics},
#' and
#' \insertCite{fakorede_heterotic_1983;textual}{germinationmetrics}.}
#' \loadmathjax
#'
#' @section Details: Several germination indices are referred to as Emergence
#'   rate Index (\mjseqn{ERI}).
#'
#'   \subsection{\insertCite{shmueli_emergence_1971;textual}{germinationmetrics}}{
#'    With argument \code{method} specified as \code{"shmueligoldberg"}
#'   \mjseqn{ERI} is estimated as follows.
#'
#'   \mjsdeqn{ERI = \sum_{i=i_{0}}^{k-1}N_{i}(k-i)}
#'
#'   Where, \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th
#'   time interval (not the accumulated number, but the number corresponding to
#'   the \mjseqn{i}th interval), \mjseqn{i_{0}} is the time interval when
#'   emergence/germination started, and \mjseqn{k} is the total number of time
#'   intervals.
#'
#'   A modification for the same was suggested by
#'   \insertCite{de_santana_alise_2004;textual}{germinationmetrics}
#'   \insertCite{ranal_how_2006}{germinationmetrics} and is estimated by
#'   dividing Emergence rate index (\mjseqn{ERI}) by total number of emerged
#'   seedlings (or germinated seeds) as follows with argument \code{method}
#'   specified as \code{"sgsantanaranal"}).
#'
#'   \mjsdeqn{ERI_{mod} = \frac{\sum_{i=i_{0}}^{k-1}N_{i}(k-i)}{N_{g}} =
#'   \frac{ERI}{N_{g}}}
#'
#'   Where, \mjseqn{N_{g}} is the total number of germinated seeds at the end of
#'   the test, \mjseqn{N_{i}} is the number of seeds germinated in the
#'   \mjseqn{i}th time interval (not the accumulated number, but the number
#'   corresponding to the \mjseqn{i}th interval), \mjseqn{i_{0}} is the time
#'   interval when emergence/germination started, and \mjseqn{k} is the total
#'   number of time intervals. }
#'
#'   \subsection{\insertCite{bilbro_soil_1982;textual}{germinationmetrics}}{
#'    With argument \code{method} specified as \code{"bilbrowanjura"}
#'   \mjseqn{ERI} is the estimated as follows.
#'
#'   \mjsdeqn{ERI = \frac{\sum_{i=1}^{k}N_{i}}{\overline{T}} =
#'   \frac{N_{g}}{\overline{T}}}
#'
#'   Where, \mjseqn{N_{g}} is the total number of germinated seeds at the end of
#'   the test, \mjseqn{N_{i}} is the number of seeds germinated in the
#'   \mjseqn{i}th time interval (not the accumulated number, but the number
#'   corresponding to the \mjseqn{i}th interval), and \mjseqn{\overline{T}} is
#'   the mean germination time or mean emergence time. }
#'
#'   \subsection{\insertCite{fakorede_relation_1980,fakorede_variability_1981,fakorede_heterotic_1983;textual}{germinationmetrics}}{
#'    With argument \code{method} specified as \code{"fakorede"} \mjseqn{ERI} is
#'   the estimated as follows.
#'
#'   \mjsdeqn{ERI = \frac{\overline{T}}{FGP/100}}
#'
#'   Where, \mjseqn{\overline{T}} is the Mean germination time and \mjseqn{FGP}
#'   is the final germination time. }
#'
#'   \subsection{Others}{ Apart from the above the above, the name Emergence
#'   rate index (\mjseqn{ERI}) has also been used to for the mathematical
#'   expressions for Speed of germination
#'   (\code{\link[germinationmetrics]{GermSpeed}})
#'   \insertCite{allan_seedling_1962,erbach_tillage_1982}{germinationmetrics},
#'    Timson's index (\code{\link[germinationmetrics]{TimsonsIndex}})
#'   \insertCite{chaudhary_effect_1970}{germinationmetrics},
#'   and George's index
#'   (\code{\link[germinationmetrics:TimsonsIndex]{GermRateGeorge}})
#'   \insertCite{chopra_effect_1980}{germinationmetrics}. }
#'
#' @inheritParams MeanGermTime
#' @param method The method to be used. Either \code{"shmueligoldberg"},
#'   \code{"sgsantanaranal"}, \code{"bilbrowanjura"} or \code{"fakorede"}.
#'   Default is \code{"shmueligoldberg"} (see \strong{Details}).
#' @param total.seeds Total number of seeds.
#'
#' @return The value of the Emergence rate index with the units according to
#'   \code{method} as follows.
#'
#'   \item{\code{shmueligoldberg}}{\mjseqn{\mathrm{count}}}
#'   \item{\code{sgsantanaranal}}{\mjseqn{\mathrm{no \, unit}}}
#'   \item{\code{bilbrowanjura}}{\mjseqn{\mathrm{count} \, time^{-1}}}
#'   \item{\code{fakorede}}{\mjseqn{time \, \mathrm{count}^{-1}}}
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50)
#' EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'                    method = "shmueligoldberg")
#' EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'                    method = "sgsantanaranal")
#' EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'                    method = "bilbrowanjura")
#' EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'                    method = "fakorede")
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
#'                    partial = FALSE)
#' EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
#'                    partial = FALSE,
#'                    method = "shmueligoldberg")
#' EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
#'                    partial = FALSE,
#'                    method = "sgsantanaranal")
#' EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
#'                    partial = FALSE,
#'                    method = "bilbrowanjura")
#' EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
#'                    partial = FALSE,
#'                    method = "fakorede")
#'
#' @seealso \code{\link[germinationmetrics]{GermSpeed}},
#'   \code{\link[germinationmetrics]{TimsonsIndex}},
#'   \code{\link[germinationmetrics:TimsonsIndex]{GermRateGeorge}}
#'
#' @rdname EmergenceRateIndex
#' @export
EmergenceRateIndex <- function(germ.counts, intervals, partial = TRUE,
                               total.seeds = NULL,
                               method = c("shmueligoldberg",
                                          "sgsantanaranal",
                                          "bilbrowanjura",
                                          "fakorede")) {

  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Check if GP is 0
  GP <- GermPercent(germ.counts = germ.counts, total.seeds = total.seeds,
                    partial = partial)

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  # Warning for depreciated method choices
  if (length(method) == 1 && method == "melville") {
    method <- "shmueligoldberg"
    warning('Argument `method = "melville"` is depreciated as of 0.1.6\n',
            'Please use `method = "shmueligoldberg"` instead.')
  }

  if (length(method) == 1 && method == "melvillesantanaranal") {
    method <- "sgsantanaranal"
    warning('Argument `method = "melvillesantanaranal"` is depreciated as of 0.1.6\n',
            'Please use `method = "sgsantanaranal"` instead.')
  }

  # Check method
  method <- match.arg(method)

  # Warning when GP is 0
  if (method == "shmueligoldberg") {
    if (GP == 0) {
      warning("Final germination percentage is 0%.\n",
              "The computation of 'EmergenceRateIndex'",
              "with method == 'shmueligoldberg' is not possible.")
    }
  }
  if (method == "sgsantanaranal") {
    if (GP == 0) {
      warning("Final germination percentage is 0%.\n",
              "The computation of 'EmergenceRateIndex'",
              "with method == 'sgsantanaranal' is not possible.")
    }
  }

  if (method == "fakorede") {
    # Check if argument total.seeds is of type numeric with unit length
    if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
      stop("'total.seeds' should be a numeric vector of length 1.")
    }
  }

  ERI <- NA_integer_

  if (GP > 0) {
    if (method == "shmueligoldberg") {
      startindex <- min(which(germ.counts != 0))
      Ni <- germ.counts[startindex:(length(germ.counts) - 1)]
      kminusi <- (length(germ.counts) - seq_along(intervals))
      kminusi <- kminusi[startindex:(length(germ.counts) - 1)]
      ERI <- sum(Ni * kminusi)
    }

    if (method == "sgsantanaranal") {
      startindex <- min(which(germ.counts != 0))
      Ni <- germ.counts[startindex:(length(germ.counts) - 1)]
      kminusi <- (length(germ.counts) - seq_along(intervals))
      kminusi <- kminusi[startindex:(length(germ.counts) - 1)]
      ERI <- sum(Ni * kminusi)/sum(germ.counts)
    }
  }

  if (method == "bilbrowanjura") {
    MGT <- MeanGermTime(germ.counts = germ.counts, intervals = intervals)
    ERI <- sum(germ.counts)/MGT
  }

  if (method == "fakorede") {
    MGT <- MeanGermTime(germ.counts = germ.counts, intervals = intervals)
    FGP <- GermPercent(germ.counts = germ.counts, total.seeds = total.seeds)
    ERI <- MGT/(FGP/100)
  }

  return(ERI)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)
total.seeds <- 50

test_that("EmergenceRateIndex works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                           total.seeds = 50),
               expected = 292, tolerance = 1e+12)
  expect_equal(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                           total.seeds = 50,
                                           method = "shmueligoldberg"),
               expected = 292, tolerance = 1e+12)
  expect_equal(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                           total.seeds = 50,
                                           method = "sgsantanaranal"),
               expected = 7.3, tolerance = 1e+12)
  expect_equal(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                           total.seeds = 50,
                                           method = "bilbrowanjura"),
               expected = 5.97014925373134, tolerance = 1e+12)
})

test_that("EmergenceRateIndex works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = EmergenceRateIndex(germ.counts = y, intervals = int,
                                           total.seeds = 50,
                                           partial = FALSE),
               expected = 292, tolerance = 1e+12)
  expect_equal(object = EmergenceRateIndex(germ.counts = y, intervals = int,
                                           total.seeds = 50,
                                           partial = FALSE,
                                           method = "shmueligoldberg"),
               expected = 292, tolerance = 1e+12)
  expect_equal(object = EmergenceRateIndex(germ.counts = y, intervals = int,
                                           total.seeds = 50,
                                           partial = FALSE,
                                           method = "sgsantanaranal"),
               expected = 7.3, tolerance = 1e+12)
  expect_equal(object = EmergenceRateIndex(germ.counts = y, intervals = int,
                                           total.seeds = 50,
                                           partial = FALSE,
                                           method = "bilbrowanjura"),
               expected = 5.97014925373134, tolerance = 1e+12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                               total.seeds = 50),
                   expected = EmergenceRateIndex(germ.counts = y, intervals = int,
                                                 total.seeds = 50,
                                                 partial = FALSE))
  expect_identical(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                               total.seeds = 50,
                                               method = "shmueligoldberg"),
                   expected = EmergenceRateIndex(germ.counts = y, intervals = int,
                                                 total.seeds = 50,
                                                 partial = FALSE,
                                                 method = "shmueligoldberg"))
  expect_identical(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                               total.seeds = 50,
                                               method = "sgsantanaranal"),
                   expected = EmergenceRateIndex(germ.counts = y, intervals = int,
                                                 total.seeds = 50,
                                                 partial = FALSE,
                                                 method = "sgsantanaranal"))
  expect_identical(object = EmergenceRateIndex(germ.counts = x, intervals = int,
                                               total.seeds = 50,
                                               method = "bilbrowanjura"),
                   expected = EmergenceRateIndex(germ.counts = y, intervals = int,
                                                 total.seeds = 50,
                                                 partial = FALSE,
                                                 method = "bilbrowanjura"))
})

z <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

test_that("Warning generated for shmueligoldberg when germination % is 0", {

  expect_warning(object = EmergenceRateIndex(germ.counts = z,
                                                       intervals = int,
                                                       total.seeds = 50,
                                                       method = "shmueligoldberg"),
                           regexp = "Final germination percentage is 0%.")
  expect_warning(object = EmergenceRateIndex(germ.counts = z,
                                             intervals = int,
                                             partial = FALSE,
                                             total.seeds = 50,
                                             method = "shmueligoldberg"),
                 regexp = "Final germination percentage is 0%.")

})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Fit four-parameter hill function for multiple samples
#'
#' Fit four-parameter hill function from a data frame of germination counts
#' recorded at specific time intervals for multiple samples in batch.
#'
#' @param data A data frame with the germination count data. It should possess
#'   columns with \itemize{ \item Partial or cumulative germination counts per
#'   time interval (to be indicated by the argument \code{counts.intervals.cols}
#'   and \item Total number of seeds tested (to be indicated by the argument
#'   \code{total.seeds.col}). }
#' @param total.seeds.col The name of the column in \code{data} with the total
#'   number of seeds tested.
#' @param counts.intervals.cols The names of columns in \code{data} with the
#'   germination counts (partial or cumulative, as indicated by the argument
#'   \code{partial}) per time interval (indicated by the argument
#'   \code{intervals}).
#' @inheritParams FourPHFfit
#'
#' @return A data frame with the original data along with the various parameters
#'   of the respective fitted four-parameter hill function.
#'
#' @references
#'
#' \insertRef{el-kassaby_seed_2008}{germinationmetrics}
#'
#' @seealso This function is a wrapper around the
#'   \code{\link[germinationmetrics]{FourPHFfit}} function for fitting
#'   four-parameter hill curve.
#'
#' @export
#'
#' @examples
#'
#' \donttest{
#' data(gcdata)
#'
#' counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
#'                           "Day06", "Day07", "Day08", "Day09", "Day10",
#'                           "Day11", "Day12", "Day13", "Day14")
#'
#' FourPHFfit.bulk(gcdata, total.seeds.col = "Total Seeds",
#'                     counts.intervals.cols = counts.per.intervals,
#'                     intervals = 1:14, partial = TRUE,
#'                     fix.y0 = TRUE, fix.a = TRUE, xp = c(10, 60),
#'                     tmax = 20, tries = 3, umax = 90, umin = 10)
#' }
#'
FourPHFfit.bulk <- function(data, total.seeds.col, counts.intervals.cols,
                            intervals, partial = TRUE,
                            fix.y0 = TRUE, fix.a = TRUE,
                            tmax, xp = c(10, 60),
                            umin = 10, umax = 90, tries = 3) {

  # Check if data.frame
  if (!is.data.frame(data)) {
    stop('"data" should be a data frame object.')
  }

  if (any(c("tbl_dataf", "tbl") %in% class(data))) {
    warning('"data" is of type tibble\nCoercing to data frame.')
    data <- as.data.frame(data)
  }

  # Check if total.seeds.col column present in data
  if (!(total.seeds.col %in% colnames(data))) {
    stop(paste('Column ', total.seeds.col,
               ' specified as the total seeds column is not present in "data".',
               sep = ""))
  }

  # Check if intervals.cols columns present in data
  if (FALSE %in% (counts.intervals.cols %in% colnames(data))) {
    stop(paste('The following column(s) specified as germination count per interval are not present in "data":\n',
               paste(counts.intervals.cols[!(counts.intervals.cols %in% colnames(data))], collapse = ", "),
               sep = ""))
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals and counts.intervals.cols are of equal length
  if (length(intervals) != length(counts.intervals.cols)) {
    stop('"intervals" and "counts.intervals.cols" are not of equal length.')
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if total.seeds.col is of type numeric
  if (!is.numeric(data[, total.seeds.col])) {
    stop('"total.seeds.col" is not of type numeric.')
  }

  # Check if total.seeds.col is of type integer
  if (!isTRUE(all.equal(data[, total.seeds.col],
                        as.integer(data[, total.seeds.col])))) {
    warning('"total.seeds.col" possesses non-integer data.')
  }

  # Check for missing values
  missvcols <- unlist(lapply(data[, c(counts.intervals.cols,
                                      total.seeds.col)],
                             function(x) TRUE %in% is.na(x)))
  if (TRUE %in% missvcols) {
    stop(paste('The following column(s) in "data" have missing values:\n',
               paste(names(missvcols[missvcols]), collapse = ", ")))
  }

  # Check if counts.intervals.cols are of type integer or numeric
  inttraitcols <- unlist(lapply(data[, counts.intervals.cols],
                                function(x) FALSE %in% (is.vector(x, mode = "integer") | is.vector(x, mode = "numeric"))))
  if (TRUE %in% inttraitcols) {
    stop(paste('The following column(s) specified as germination count per interval in "data" are not of type integer:\n',
               paste(names(inttraitcols[inttraitcols]), collapse = ", ")))
  }

  # Warning if counts.intervals.cols are not of type integer
  inttraitcols2 <- unlist(lapply(data[, counts.intervals.cols],
                                 function(x) isTRUE(all.equal(x, as.integer(x)))))
  if (FALSE %in% inttraitcols2) {
    warning(paste('The following column(s) specified as germination count per interval in "data" possess non-integer data:\n',
                  paste(names(inttraitcols2[!inttraitcols2]), collapse = ", ")))
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Convert cumulative to partial
  if (!partial) {
    data[, counts.intervals.cols] <- t(apply(data[, counts.intervals.cols], 1,
                                             function(x) c(x[1], diff(x))))
  }

  # Check if total.seeds > no. of germinated seeds
  if (any(apply(data[, counts.intervals.cols], 1,
                sum) > data[, total.seeds.col])) {
    inderrorrows <- (1:nrow(data))[apply(data[, counts.intervals.cols], 1,
                                         sum) > data[, total.seeds.col]]
    warning(paste('For the following rows in "data", the total',
                  'number of seeds tested ("total.seeds.col")',
                  'is less than the total number of germinated seeds:',
                  paste(inderrorrows, collapse = ", ")))
  }


  data <- data.table::as.data.table(data)

  fphfnames <- c("a", "b", "c", "y0", "lag", "Dlag50", "t50.total",
                 "txp.total", #list
                 "t50.Germinated",
                 "txp.Germinated", "Uniformity", #list
                 "TMGR", "AUC", "MGT", "Skewness",
                 "msg", "isConv",
                 "Fit" #list
                 )

  out <- data[, .(outlist = list(FourPHFfit(germ.counts = unlist(mget(counts.intervals.cols)),
                                            intervals = intervals,
                                            partial = TRUE, fix.y0 = fix.y0,
                                            fix.a = fix.a, tmax = tmax,
                                            xp = xp, umin = umin, umax = umax,
                                            tries = tries,
                                            total.seeds = unlist(mget(total.seeds.col)))[fphfnames])),
              by = 1:nrow(data)]

  out <-
    rbindlist(lapply(out$outlist,
                     function(x) {

                       xptdf <- as.data.frame.list(x$txp.total)
                       colnames(xptdf) <-
                         gsub("(^X)(\\d+$)", "t\\2.total", colnames(xptdf))
                       xpgdf <- as.data.frame.list(x$txp.Germinated)
                       colnames(xpgdf) <-
                         gsub("(^X)(\\d+$)", "t\\2.Germinated", colnames(xpgdf))
                       udf <- as.data.frame.list(x$Uniformity)
                       colnames(udf) <-
                         gsub("(^X)(\\d+$)", "Uniformity_\\2", colnames(udf))

                       fout <- cbind(
                         data.frame(x[c("a", "b", "c", "y0", "lag",
                                        "Dlag50", "t50.total")]),
                         xptdf,
                         data.frame(x[c("t50.Germinated")]),
                         xpgdf,
                         udf,
                         data.frame(x[c("TMGR", "AUC",  "MGT", "Skewness")]),
                         data.frame(x[c("msg")]),
                         as.data.frame.list(x$Fit))

                       return(fout)

                     }
    ))

  setnames(out, old = "uniformity", new = "Uniformity")
  # setnames(out, old = colnames(out),
  #          new = gsub("Fit.", "Fit_", colnames(out)))

  out <- out[, isConv := NULL]

  data <- cbind(data, out)

  setDF(data)

  # Convert partial back to cumulative if original data was cumulative
  if (!partial) {
    data[, counts.intervals.cols] <- t(apply(data[, counts.intervals.cols], 1,
                                             function(x) cumsum(x)))
  }

  arguments <- list(total.seeds.col = total.seeds.col,
                    counts.intervals.cols = counts.intervals.cols,
                    intervals = intervals, partial = partial, fix.y0 = fix.y0,
                    fix.a = fix.a, tmax = tmax, xp = xp,
                    umin = umin, umax = umax)

  attr(data, "arguments") <- arguments

  class(data) <- c("FourPHFfit.bulk", class(data))
  return(data)

}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Germination Indices
#'
#' Compute germination indices from a data frame of germination counts recorded
#' at specific time intervals for multiple samples in batch. \loadmathjax
#'
#' @param data A data frame with the germination count data. It should possess
#'   columns with \itemize{ \item Partial or cumulative germination counts per
#'   time interval (to be indicated by the argument \code{counts.intervals.cols}
#'   and \item Total number of seeds tested (to be indicated by the argument
#'   \code{total.seeds.col}). }
#' @param total.seeds.col The name of the column in \code{data} with the total
#'   number of seeds tested.
#' @param counts.intervals.cols The names of columns in \code{data} with the
#'   germination counts (partial or cumulative, as indicated by the argument
#'   \code{partial}) per time interval (indicated by the argument
#'   \code{intervals}).
#' @param intervals The time intervals at which germination counts (in the
#'   columns specified in argument \code{counts.intervals.cols})  were recorded.
#' @param partial logical. If \code{TRUE}, germination counts in
#'   \code{counts.intervals.cols} is considered as partial and if \code{FALSE},
#'   it is considered as cumulative. Default is \code{TRUE}.
#' @param PeakGermPercent logical. If \code{TRUE}, then the Peak germination
#'   percentage
#'   \insertCite{vallance_studies_1950,roh_maturity_2004}{germinationmetrics} is
#'   computed. Default is \code{TRUE}.
#' @param FirstGermTime logical. If \code{TRUE}, the Time of first germination
#'   or Germination time lag (\mjseqn{t_{0}})
#'   \insertCite{edwards_temperature_1932,czabator_germination_1962,goloff_germination_1975,labouriau_germinacao_1983,ranal_effects_1999,quintanilla_effect_2000}{germinationmetrics}
#'    is computed. Default is \code{TRUE}.
#' @param LastGermTime logical. If \code{TRUE}, the Time of last germination
#'   (\mjseqn{t_{g}})
#'   \insertCite{edwards_temperature_1932,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
#'    is computed. Default is \code{TRUE}.
#' @param TimeSpreadGerm logical. If \code{TRUE}, the Time spread of germination
#'   \insertCite{al-mudaris_notes_1998,kader_comparison_2005}{germinationmetrics}
#'    or Germination distribution
#'   \insertCite{schrader_seed_2000}{germinationmetrics}. is computed. Default
#'   is \code{TRUE}.
#' @param PeakGermTime logical. If \code{TRUE}, the Peak time of germination or
#'   Modal time of germination \insertCite{ranal_how_2006}{germinationmetrics}
#'   is computed. Default is \code{TRUE}.
#' @param t50 logical. If \code{TRUE}, the Median germination time
#'   (\mjseqn{t_{50}})
#'   \insertCite{coolbear_effect_1984,farooq_thermal_2005}{germinationmetrics}
#'   is computed. Default is \code{TRUE}.
#' @param MeanGermTime logical. If \code{TRUE}, the Mean germination time
#'   (\mjseqn{\overline{T}}) or Mean length of incubation time
#'   \insertCite{edmond_effects_1958,czabator_germination_1962,ellis_improved_1980,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
#'    or Sprouting index (\mjseqn{SI})
#'   \insertCite{smith_germinating_1964}{germinationmetrics} or Germination
#'   Resistance (\mjseqn{GR})
#'   \insertCite{gordon_observations_1969,gordon_germination_1971;textual}{germinationmetrics}
#'    is computed. Default is \code{TRUE}.
#' @param VarGermTime logical. If \code{TRUE}, the Variance of germination time
#'   (\mjseqn{s_{T}^{2}})
#'   \insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}
#'   is computed. Default is \code{TRUE}.
#' @param SEGermTime logical. If \code{TRUE}, the Standard error of germination
#'   time (\mjseqn{s_{\overline{T}}})
#'   \insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
#'   is computed. Default is \code{TRUE}.
#' @param CVGermTime logical. If \code{TRUE}, the Coefficient of variation of
#'   the germination time (\mjseqn{CV_{T}})
#'   \insertCite{ranal_how_2006}{germinationmetrics} is computed. Default is
#'   \code{TRUE}.
#' @param MeanGermRate logical. If \code{TRUE}, the Mean germination rate
#'   (\mjseqn{\overline{V}})
#'   \insertCite{labouriau_germination_1976,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.
#'    is computed. Default is \code{TRUE}.
#' @param VarGermRate logical. If \code{TRUE}, the Variance of germination rate
#'   (\mjseqn{s_{V}^{2}})
#'   \insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics} is
#'   computed. Default is \code{TRUE}.
#' @param SEGermRate logical. If \code{TRUE}, the Standard error of germination
#'   rate (\mjseqn{s_{\overline{V}}})
#'   \insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics} is
#'   computed. Default is \code{TRUE}.
#' @param CVG logical. If \code{TRUE}, the Coefficient of velocity/rate of
#'   germination or Kotowski's coefficient of velocity (\mjseqn{CVG})
#'   \insertCite{kotowski_temperature_1926,nichols_two_1968,labouriau_uma_1983,scott_review_1984,bewley_seeds_1994}{germinationmetrics}.
#'    is computed. Default is \code{TRUE}.
#' @param GermRateRecip logical. If \code{TRUE}, the Germination rate as
#'   reciprocal of median time (\mjseqn{v_{50}})
#'   \insertCite{went_experimental_1957,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}
#'    is computed.
#' @param GermSpeed logical. If \code{TRUE}, the Speed of germination or
#'   Germination rate index or Index of velocity of germination or Germination
#'   index or Emergence rate index
#'   \insertCite{throneberry_relation_1955,maguire_speed_1962,allan_seedling_1962,kendrick_photocontrol_1969,bouton_germination_1976,aosa_seed_1983,khandakar_jute_1983,bradbeer_seed_1988,wardle_allelopathic_1991}{germinationmetrics}
#'    considering both counts and percentages is computed.
#' @param GermSpeedAccumulated logical. If \code{TRUE}, the Speed of accumulated
#'   germination
#'   \insertCite{bradbeer_seed_1988,wardle_allelopathic_1991,haugland_experiments_1996,de_santana_alise_2004}{germinationmetrics}
#'    considering both counts and percentages is computed.
#' @param GermSpeedCorrected logical. If \code{TRUE}, the Corrected speed of
#'   germination or Corrected germination rate index and the Corrected speed of
#'   accumulated germination
#'   \insertCite{evetts_germination_1972}{germinationmetrics} is computed.
#' @param WeightGermPercent logical. If \code{TRUE}, the Weighted germination
#'   percentage \mjseqn{WGP} or Weighted germination index \mjseqn{WGI}
#'   \insertCite{reddy_effect_1978,reddy_effect_1985}{germinationmetrics} is
#'   computed.
#' @param MeanGermPercent logical. If \code{TRUE}, the Mean/average germination
#'   percentage per unit time (\mjseqn{\overline{GP}})
#'   \insertCite{czabator_germination_1962}{germinationmetrics} is computed.
#' @param MeanGermNumber logical. If \code{TRUE}, the Number of seeds germinated
#'   per unit time (\mjseqn{\overline{N}})
#'   \insertCite{khamassi_optimal_2013}{germinationmetrics} is computed.
#' @param TimsonsIndex logical. If \code{TRUE}, the Timson's index or Timson's
#'   germination velocity index
#'   \insertCite{grose_laboratory_1958,timson_new_1965,brown_representing_1988,baskin_seeds_1998}{germinationmetrics}
#'    and it's modifications by Labouriau
#'   \insertCite{ranal_how_2006}{germinationmetrics} and
#'   \insertCite{khan_effect_1984}{germinationmetrics} is computed.
#' @param GermRateGeorge logical. If \code{TRUE}, the George's index
#'   \insertCite{george_influence_1961;textual}{germinationmetrics} is computed.
#' @param max.int The maximum interval value up to which Timson's index/George's
#'   germination rate is to be computed.
#' @param GermIndex logical. If \code{TRUE}, the Germination index according to
#'   \insertCite{melville_seed_1980;textual}{germinationmetrics}
#'   and its modification by
#'   \insertCite{de_santana_alise_2004;textual}{germinationmetrics}
#'   \insertCite{ranal_how_2006}{germinationmetrics} are computed.
#' @param EmergenceRateIndex logical. If \code{TRUE}, the Emergence rate index
#'   or Germination rate index are computed according to \itemize{ \item
#'   \insertCite{shmueli_emergence_1971;textual}{germinationmetrics}
#'   and it's modification by
#'   \insertCite{de_santana_alise_2004;textual}{germinationmetrics}
#'   \insertCite{ranal_how_2006}{germinationmetrics}; \item
#'   \insertCite{bilbro_soil_1982;textual}{germinationmetrics}; and
#'   \item
#'   \insertCite{fakorede_relation_1980;textual}{germinationmetrics},
#'   \insertCite{fakorede_variability_1981;textual}{germinationmetrics},
#'    and
#'   \insertCite{fakorede_heterotic_1983;textual}{germinationmetrics}.}
#'
#'
#' @param PeakValue logical. If \code{TRUE}, the Peak value (\mjseqn{PV}) or
#'   Emergence Energy (\mjseqn{EE})
#'   \insertCite{czabator_germination_1962,bonner_ideal_1967}{germinationmetrics}
#'    is computed.
#' @param GermValue logical. If \code{TRUE}, the Germination value
#'   \insertCite{czabator_germination_1962,djavanshir_germination_1976}{germinationmetrics}
#'    and its modification considering duration from start of test instead of
#'   from onset of germination
#'   \insertCite{brown_representing_1988}{germinationmetrics} is computed.
#' @param gv.k The k Constant for computation of germination value. Default is
#'   10.
#' @param CUGerm logical. If \code{TRUE}, the Coefficient of uniformity of
#'   germination (CUG)
#'   \insertCite{heydecker_seed_1972,bewley_seeds_1994}{germinationmetrics} is
#'   computed.
#' @param GermSynchrony logical. If \code{TRUE}, the Synchrony of germination
#'   (\mjseqn{Z} index)
#'   \insertCite{primack_longevity_1985,ranal_how_2006}{germinationmetrics} is
#'   computed.
#' @param GermUncertainty logical. If \code{TRUE}, the Synchronization index
#'   (\mjseqn{\overline{E}}) or Uncertainty of the germination process
#'   (\mjseqn{U}) or Informational entropy (\mjseqn{H})
#'   \insertCite{shannon_mathematical_1948,labouriau_germination_1976,labouriau_uma_1983}{germinationmetrics}
#'    is computed.
#'
#' @return A data frame with the original data along with the various
#'   single-value germination indices specified in the function arguments.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#'
#' \donttest{
#' data(gcdata)
#'
#' counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
#'                           "Day06", "Day07", "Day08", "Day09", "Day10",
#'                           "Day11", "Day12", "Day13", "Day14")
#' germination.indices(gcdata, total.seeds.col = "Total Seeds",
#'                     counts.intervals.cols = counts.per.intervals,
#'                     intervals = 1:14, partial = TRUE, max.int = 5)
#' }
#'
#' @seealso This function is a wrapper around the different functions for
#'   computation of single-value germination indices in
#'   \href{https://cran.r-project.org/package=germinationmetrics}{germinationmetrics}
#'    (\code{\link[germinationmetrics]{PeakGermPercent}},
#'   \code{\link[germinationmetrics]{FirstGermTime}},
#'   \code{\link[germinationmetrics:FirstGermTime]{LastGermTime}},
#'   \code{\link[germinationmetrics:FirstGermTime]{PeakGermTime}},
#'   \code{\link[germinationmetrics:FirstGermTime]{TimeSpreadGerm}},
#'   \code{\link[germinationmetrics]{t50}},
#'   \code{\link[germinationmetrics]{MeanGermTime}},
#'   \code{\link[germinationmetrics:MeanGermTime]{VarGermTime}},
#'   \code{\link[germinationmetrics:MeanGermTime]{SEGermTime}},
#'   \code{\link[germinationmetrics:MeanGermTime]{CVGermTime}},
#'   \code{\link[germinationmetrics]{MeanGermRate}},
#'   \code{\link[germinationmetrics:MeanGermRate]{CVG}},
#'   \code{\link[germinationmetrics:MeanGermRate]{VarGermRate}},
#'   \code{\link[germinationmetrics:MeanGermRate]{SEGermRate}},
#'   \code{\link[germinationmetrics:MeanGermRate]{GermRateRecip}},
#'   \code{\link[germinationmetrics]{GermSpeed}},
#'   \code{\link[germinationmetrics:GermSpeed]{GermSpeedAccumulated}},
#'   \code{\link[germinationmetrics:GermSpeed]{GermSpeedCorrected}},
#'   \code{\link[germinationmetrics]{WeightGermPercent}},
#'   \code{\link[germinationmetrics]{MeanGermPercent}},
#'   \code{\link[germinationmetrics:GermSpeed]{MeanGermPercent}},
#'   \code{\link[germinationmetrics]{TimsonsIndex}},
#'   \code{\link[germinationmetrics:TimsonsIndex]{GermRateGeorge}},
#'   \code{\link[germinationmetrics]{GermIndex}},
#'   \code{\link[germinationmetrics]{EmergenceRateIndex}},
#'   \code{\link[germinationmetrics]{GermValue}},
#'   \code{\link[germinationmetrics:GermValue]{PeakValue}},
#'   \code{\link[germinationmetrics]{CUGerm}},
#'   \code{\link[germinationmetrics]{GermSynchrony}},
#'   \code{\link[germinationmetrics:GermSynchrony]{GermUncertainty}})
#'
#' @import data.table
#' @export
germination.indices <- function(data, total.seeds.col, counts.intervals.cols,
                                intervals, partial = TRUE,
                                PeakGermPercent = TRUE,
                                FirstGermTime = TRUE, LastGermTime = TRUE,
                                PeakGermTime = TRUE, TimeSpreadGerm = TRUE,
                                t50 = TRUE, MeanGermTime = TRUE,
                                VarGermTime = TRUE, SEGermTime = TRUE,
                                CVGermTime = TRUE, MeanGermRate = TRUE,
                                CVG = TRUE, VarGermRate = TRUE,
                                SEGermRate = TRUE, GermRateRecip = TRUE,
                                GermSpeed = TRUE, GermSpeedAccumulated = TRUE,
                                GermSpeedCorrected = TRUE,
                                WeightGermPercent = TRUE,
                                MeanGermPercent = TRUE, MeanGermNumber = TRUE,
                                TimsonsIndex = TRUE, GermRateGeorge = TRUE,
                                max.int, GermIndex = TRUE,
                                EmergenceRateIndex = TRUE, PeakValue = TRUE,
                                GermValue = TRUE, gv.k = 10,
                                CUGerm = TRUE, GermSynchrony = TRUE,
                                GermUncertainty = TRUE) {
  # Check if data.frame
  if (!is.data.frame(data)) {
    stop('"data" should be a data frame object.')
  }

  if (any(c("tbl_dataf", "tbl") %in% class(data))) {
    warning('"data" is of type tibble\nCoercing to data frame.')
    data <- as.data.frame(data)
  }

  # Check if total.seeds.col column present in data
  if (!(total.seeds.col %in% colnames(data))) {
    stop(paste('Column ', total.seeds.col,
               ' specified as the total seeds column is not present in "data".',
               sep = ""))
  }

  # Check if intervals.cols columns present in data
  if (FALSE %in% (counts.intervals.cols %in% colnames(data))) {
    stop(paste('The following column(s) specified as germination count per interval are not present in "data":\n',
               paste(counts.intervals.cols[!(counts.intervals.cols %in% colnames(data))], collapse = ", "),
               sep = ""))
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals and counts.intervals.cols are of equal length
  if (length(intervals) != length(counts.intervals.cols)) {
    stop('"intervals" and "counts.intervals.cols" are not of equal length.')
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if total.seeds.col is of type numeric
  if (!is.numeric(data[, total.seeds.col])) {
    stop('"total.seeds.col" is not of type numeric.')
  }

  # Check if total.seeds.col is of type integer
  if (!isTRUE(all.equal(data[, total.seeds.col],
                        as.integer(data[, total.seeds.col])))) {
    warning('"total.seeds.col" possesses non-integer data.')
  }

  # Check for missing values
  missvcols <- unlist(lapply(data[, c(counts.intervals.cols,
                                      total.seeds.col)],
                             function(x) TRUE %in% is.na(x)))
  if (TRUE %in% missvcols) {
    stop(paste('The following column(s) in "data" have missing values:\n',
               paste(names(missvcols[missvcols]), collapse = ", ")))
  }

  # Check if counts.intervals.cols are of type integer or numeric
  inttraitcols <- unlist(lapply(data[, counts.intervals.cols],
                                function(x) FALSE %in% (is.vector(x, mode = "integer") | is.vector(x, mode = "numeric"))))
  if (TRUE %in% inttraitcols) {
    stop(paste('The following column(s) specified as germination count per interval in "data" are not of type numeric:\n',
               paste(names(inttraitcols[inttraitcols]), collapse = ", ")))
  }

  # Warning if counts.intervals.cols are not of type integer
  inttraitcols2 <- unlist(lapply(data[, counts.intervals.cols],
                                 function(x) isTRUE(all.equal(x, as.integer(x)))))
  if (FALSE %in% inttraitcols2) {
    warning(paste('The following column(s) specified as germination count per interval in "data" possess non-integer data:\n',
                  paste(names(inttraitcols2[!inttraitcols2]), collapse = ", ")))
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Convert cumulative to partial
  if (!partial) {
    data[, counts.intervals.cols] <- t(apply(data[, counts.intervals.cols], 1,
           function(x) c(x[1], diff(x))))
  }

  # Check if total.seeds > no. of germinated seeds
  if (any(apply(data[, counts.intervals.cols], 1,
                sum) > data[, total.seeds.col])) {
    inderrorrows <- (1:nrow(data))[apply(data[, counts.intervals.cols], 1,
                                         sum) > data[, total.seeds.col]]
    warning(paste('For the following rows in "data", the total',
                  'number of seeds tested ("total.seeds.col")',
                  'is less than the total number of germinated seeds:',
                  paste(inderrorrows, collapse = ", ")))
  }


  data <- data.table::as.data.table(data)

  data[, GermPercent := GermPercent(germ.counts = unlist(mget(counts.intervals.cols)),
                                    total.seeds = unlist(mget(total.seeds.col)),
                                    partial = TRUE), by = 1:nrow(data)]

  if (PeakGermPercent) {
    data[, PeakGermPercent := PeakGermPercent(germ.counts = unlist(mget(counts.intervals.cols)),
                                              intervals = intervals,
                                              total.seeds = unlist(mget(total.seeds.col)),
                                              partial = TRUE), by = 1:nrow(data)]
  }

  if (FirstGermTime) {
    data[, FirstGermTime := FirstGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                          intervals = intervals,
                                          partial = TRUE), by = 1:nrow(data)]
  }

  if (LastGermTime) {
    data[, LastGermTime := LastGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                        intervals = intervals,
                                        partial = TRUE), by = 1:nrow(data)]
  }

  if (PeakGermTime) {
    data[, PeakGermTime := paste(PeakGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                              intervals = intervals,
                                              partial = TRUE), collapse = ", "),
         by = 1:nrow(data)]
  }

  if (TimeSpreadGerm) {
    data[, TimeSpreadGerm := TimeSpreadGerm(germ.counts = unlist(mget(counts.intervals.cols)),
                                            intervals = intervals,
                                            partial = TRUE), by = 1:nrow(data)]
  }

  if (t50) {
    data[, t50_Coolbear := t50(germ.counts = unlist(mget(counts.intervals.cols)),
                               intervals = intervals,
                               partial = TRUE,
                               method = "coolbear"), by = 1:nrow(data)]

    data[, t50_Farooq := t50(germ.counts = unlist(mget(counts.intervals.cols)),
                               intervals = intervals,
                               partial = TRUE,
                               method = "farooq"), by = 1:nrow(data)]
  }

  if (MeanGermTime) {
    data[, MeanGermTime := MeanGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                        intervals = intervals,
                                        partial = TRUE), by = 1:nrow(data)]
  }

  if (VarGermTime) {
    data[, VarGermTime := VarGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                      intervals = intervals,
                                      partial = TRUE), by = 1:nrow(data)]
  }

  if (SEGermTime) {
    data[, SEGermTime := SEGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                    intervals = intervals,
                                    partial = TRUE), by = 1:nrow(data)]
  }

  if (CVGermTime) {
    data[, CVGermTime := CVGermTime(germ.counts = unlist(mget(counts.intervals.cols)),
                                        intervals = intervals,
                                        partial = TRUE), by = 1:nrow(data)]
  }

  if (MeanGermRate) {
    data[, MeanGermRate := MeanGermRate(germ.counts = unlist(mget(counts.intervals.cols)),
                                    intervals = intervals,
                                    partial = TRUE), by = 1:nrow(data)]
  }

  if (VarGermRate) {
    data[, VarGermRate := VarGermRate(germ.counts = unlist(mget(counts.intervals.cols)),
                                    intervals = intervals,
                                    partial = TRUE), by = 1:nrow(data)]
  }

  if (SEGermRate) {
    data[, SEGermRate := SEGermRate(germ.counts = unlist(mget(counts.intervals.cols)),
                                    intervals = intervals,
                                    partial = TRUE), by = 1:nrow(data)]
  }

  if (CVG) {
    data[, CVG := CVG(germ.counts = unlist(mget(counts.intervals.cols)),
                      intervals = intervals,
                      partial = TRUE), by = 1:nrow(data)]
  }

  if (GermRateRecip) {
    data[, GermRateRecip_Coolbear := GermRateRecip(germ.counts = unlist(mget(counts.intervals.cols)),
                               intervals = intervals,
                               partial = TRUE,
                               method = "coolbear"), by = 1:nrow(data)]

    data[, GermRateRecip_Farooq := GermRateRecip(germ.counts = unlist(mget(counts.intervals.cols)),
                             intervals = intervals,
                             partial = TRUE,
                             method = "farooq"), by = 1:nrow(data)]
  }

  if (GermSpeed) {
    data[, GermSpeed_Count := GermSpeed(germ.counts = unlist(mget(counts.intervals.cols)),
                                         intervals = intervals, percent = FALSE,
                                         partial = TRUE), by = 1:nrow(data)]
    data[, GermSpeed_Percent := GermSpeed(germ.counts = unlist(mget(counts.intervals.cols)),
                                          intervals = intervals, percent = TRUE,
                                          total.seeds = unlist(mget(total.seeds.col)),
                                          partial = TRUE), by = 1:nrow(data)]
  }

  if (GermSpeedAccumulated) {
    data[, GermSpeedAccumulated_Count := GermSpeedAccumulated(germ.counts = unlist(mget(counts.intervals.cols)),
                                                        intervals = intervals,
                                                        percent = FALSE,
                                                        partial = TRUE),
         by = 1:nrow(data)]
    data[, GermSpeedAccumulated_Percent := GermSpeedAccumulated(germ.counts = unlist(mget(counts.intervals.cols)),
                                                        intervals = intervals,
                                                        percent = TRUE,
                                                        total.seeds = unlist(mget(total.seeds.col)),
                                                        partial = TRUE),
         by = 1:nrow(data)]
  }

  if (GermSpeedCorrected) {
    data[, GermSpeedCorrected_Normal := GermSpeedCorrected(germ.counts = unlist(mget(counts.intervals.cols)),
                                                           intervals = intervals,
                                                           partial = TRUE,
                                                           method = "normal",
                                                           total.seeds = unlist(mget(total.seeds.col))),
         by = 1:nrow(data)]
    data[, GermSpeedCorrected_Accumulated := GermSpeedCorrected(germ.counts = unlist(mget(counts.intervals.cols)),
                                                                intervals = intervals,
                                                                partial = TRUE,
                                                                method = "accumulated",
                                                                total.seeds = unlist(mget(total.seeds.col))),
         by = 1:nrow(data)]
  }

  if (WeightGermPercent) {
    data[, WeightGermPercent := WeightGermPercent(germ.counts = unlist(mget(counts.intervals.cols)),
                                              intervals = intervals,
                                              partial = TRUE,
                                              total.seeds = unlist(mget(total.seeds.col))),
         by = 1:nrow(data)]
  }

  if (MeanGermPercent) {
    data[, MeanGermPercent := MeanGermPercent(germ.counts = unlist(mget(counts.intervals.cols)),
                                              intervals = intervals,
                                              partial = TRUE,
                                              total.seeds = unlist(mget(total.seeds.col))),
         by = 1:nrow(data)]
  }

  if (MeanGermNumber) {
    data[, MeanGermNumber := MeanGermNumber(germ.counts = unlist(mget(counts.intervals.cols)),
                                            intervals = intervals,
                                            partial = TRUE), by = 1:nrow(data)]
  }

  if (any(TimsonsIndex, GermRateGeorge)) {
    # Set max.int
    if (missing(max.int)) {
      max.int <- intervals[length(intervals)]
    }

    # Check if argument max is of type numeric with unit length
    if (!is.numeric(max.int) || length(max.int) != 1) {
      stop("'max.int' should be a logical vector of length 1.")
    }
  }

  if (TimsonsIndex) {
    data[, TimsonsIndex := TimsonsIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                        intervals = intervals,
                                        partial = TRUE,
                                        total.seeds = unlist(mget(total.seeds.col)),
                                        max = max.int, modification = "none"),
         by = 1:nrow(data)]

    data[, TimsonsIndex_Labouriau := TimsonsIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                                  intervals = intervals,
                                                  partial = TRUE,
                                                  total.seeds = unlist(mget(total.seeds.col)),
                                                  max = max.int,
                                                  modification = "labouriau"),
         by = 1:nrow(data)]

    data[, TimsonsIndex_KhanUngar := TimsonsIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                                  intervals = intervals,
                                                  partial = TRUE,
                                                  total.seeds = unlist(mget(total.seeds.col)),
                                                  max = max.int,
                                                  modification = "khanungar"),
         by = 1:nrow(data)]
  }

  if (GermRateGeorge) {
    data[, GermRateGeorge := GermRateGeorge(germ.counts = unlist(mget(counts.intervals.cols)),
                                            intervals = intervals,
                                            partial = TRUE, max = max.int),
         by = 1:nrow(data)]
  }

  if (GermIndex) {
    data[, GermIndex := GermIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                  intervals = intervals, partial = TRUE,
                                  total.seeds = unlist(mget(total.seeds.col)),
                                  modification = "none"),
         by = 1:nrow(data)]

    data[, GermIndex_mod := GermIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                      intervals = intervals, partial = TRUE,
                                      total.seeds = unlist(mget(total.seeds.col)),
                                      modification = "santanaranal"),
         by = 1:nrow(data)]
  }

  if (EmergenceRateIndex) {
    data[, EmergenceRateIndex_SG := EmergenceRateIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                                       intervals = intervals, partial = TRUE,
                                                       total.seeds = unlist(mget(total.seeds.col)),
                                                       method = "shmueligoldberg"),
         by = 1:nrow(data)]

    data[, EmergenceRateIndex_SG_mod := EmergenceRateIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                                           intervals = intervals, partial = TRUE,
                                                           total.seeds = unlist(mget(total.seeds.col)),
                                                           method = "sgsantanaranal"),
         by = 1:nrow(data)]

    data[, EmergenceRateIndex_BilbroWanjura := EmergenceRateIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                                                  intervals = intervals, partial = TRUE,
                                                                  total.seeds = unlist(mget(total.seeds.col)),
                                                                  method = "bilbrowanjura"),
         by = 1:nrow(data)]

    data[, EmergenceRateIndex_Fakorede := EmergenceRateIndex(germ.counts = unlist(mget(counts.intervals.cols)),
                                                             intervals = intervals, partial = TRUE,
                                                             total.seeds = unlist(mget(total.seeds.col)),
                                                             method = "fakorede"),
         by = 1:nrow(data)]

  }

  if (PeakValue) {
    data[, PeakValue := PeakValue(germ.counts = unlist(mget(counts.intervals.cols)),
                                              intervals = intervals,
                                              partial = TRUE,
                                              total.seeds = unlist(mget(total.seeds.col))),
         by = 1:nrow(data)]
  }

  if (GermValue) {
    # Check if argument gv.k is of type numeric with unit length
    if (!is.numeric(gv.k) || length(gv.k) != 1) {
      stop("'gv.k' should be a numeric vector of length 1.")
    }

    data[, GermValue_Czabator := GermValue(germ.counts = unlist(mget(counts.intervals.cols)),
                                           intervals = intervals,
                                           partial = TRUE, k = gv.k,
                                           from.onset = TRUE,
                                           method = "czabator",
                                           total.seeds = unlist(mget(total.seeds.col)))$`Germination Value`,
         by = 1:nrow(data)]
    data[, GermValue_DP := GermValue(germ.counts = unlist(mget(counts.intervals.cols)),
                                     intervals = intervals,
                                     partial = TRUE, k = gv.k,
                                     from.onset = TRUE, method = "dp",
                                     total.seeds = unlist(mget(total.seeds.col)))$`Germination Value`,
         by = 1:nrow(data)]

    data[, GermValue_Czabator_mod := GermValue(germ.counts = unlist(mget(counts.intervals.cols)),
                                               intervals = intervals,
                                               partial = TRUE, k = gv.k,
                                               from.onset = FALSE,
                                               method = "czabator",
                                               total.seeds = unlist(mget(total.seeds.col)))$`Germination Value`,
         by = 1:nrow(data)]
    data[, GermValue_DP_mod := GermValue(germ.counts = unlist(mget(counts.intervals.cols)),
                                         intervals = intervals,
                                         partial = TRUE, k = gv.k,
                                         from.onset = FALSE, method = "dp",
                                         total.seeds = unlist(mget(total.seeds.col)))$`Germination Value`,
         by = 1:nrow(data)]
  }

  if (CUGerm) {
    data[, CUGerm := CUGerm(germ.counts = unlist(mget(counts.intervals.cols)),
                            intervals = intervals,
                            partial = TRUE), by = 1:nrow(data)]
  }

  if (GermSynchrony) {
    data[, GermSynchrony := GermSynchrony(germ.counts = unlist(mget(counts.intervals.cols)),
                                          intervals = intervals,
                                          partial = TRUE), by = 1:nrow(data)]
  }

  if (GermUncertainty) {
    data[, GermUncertainty := GermUncertainty(germ.counts = unlist(mget(counts.intervals.cols)),
                                              intervals = intervals,
                                              partial = TRUE),
         by = 1:nrow(data)]
  }

  data.table::setDF(data)
  return(data)
}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Plot the four-parameter hill function fitted cumulative germination curve
#' from a \code{FourPHFfit} object
#'
#' \code{plot.FourPHFfit} plots the four-parameter hill function fitted
#' cumulative germination curve (FPHF curve) from a \code{FourPHFfit} object as
#' an object of class \code{ggplot}. Further, the rate of germination curve (RoG
#' curve) is plotted and different parameters annotated as specified in the
#' different arguments. \loadmathjax
#'
#' @param x An object of class \code{FourPHFfit} obtained as output from the
#'   \code{\link[germinationmetrics]{FourPHFfit}} function.
#' @param rog If \code{TRUE}, plots the Rate of Germination curve (RoG). Default
#'   is \code{TRUE}.
#' @param t50.total If \code{TRUE}, highlights the time required for 50\% of
#'   total seeds to germinate (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Total}}})
#'   as a vertical line. Default is \code{TRUE}.
#' @param t50.germ If \code{TRUE}, highlights the time required for 50\% of
#'   viable/germinated seeds to germinate
#'   (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Germ}}}) as a vertical line.
#'   Default is \code{TRUE}.
#' @param tmgr If \code{TRUE}, highlights the Time at Maximum Germination Rate
#'   (TMGR) as a vertical line. Default is \code{TRUE}.
#' @param mgt If \code{TRUE}, highlights the Mean Germination Time (MGT) as a
#'   vertical line. Default is \code{TRUE}.
#' @param uniformity If \code{TRUE}, highlights the uniformity value
#'   (\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}})
#'   as a horizontal line. Default is \code{TRUE}.
#' @param limits logical. If \code{TRUE}, set the limits of y axis (germination
#'   percentage) between 0 and 100 in the germination curve plot. If
#'   \code{FALSE}, limits are set according to the data. Default is \code{TRUE}.
#' @param plotlabels logical. If \code{TRUE}, adds labels to the germination
#'   curve plot. Default is \code{TRUE}.
#' @param \dots Default plot arguments.
#'
#' @return The plot of the cumulative germination curve as an object of class
#'   \code{ggplot}.
#'
#' @seealso \code{\link[germinationmetrics]{FourPHFfit}}
#'
#' @import ggplot2
#' @import ggrepel
#' @method plot FourPHFfit
#' @export
#'
#' @examples
#'
#' \donttest{
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#' total.seeds = 50
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' fit1 <- FourPHFfit(germ.counts = x, intervals = int,
#'                    total.seeds = 50, tmax = 20)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' fit2 <- FourPHFfit(germ.counts = y, intervals = int,
#'                    total.seeds = 50, tmax = 20, partial = FALSE)
#'
#' # Default plots
#' plot(fit1)
#' plot(fit2)
#'
#' # No labels
#' plot(fit1, plotlabels = FALSE)
#' plot(fit2, plotlabels = FALSE)
#'
#' # Only the FPHF curve
#' plot(fit1, rog = FALSE, t50.total = FALSE, t50.germ = FALSE,
#'      tmgr = FALSE, mgt = FALSE, uniformity = FALSE)
#' plot(fit2, rog = FALSE, t50.total = FALSE, t50.germ = FALSE,
#'      tmgr = FALSE, mgt = FALSE, uniformity = FALSE)
#'
#' # Without y axis limits adjustment
#' plot(fit1, limits = FALSE)
#' plot(fit2, limits = FALSE)
#' }
#'
plot.FourPHFfit <- function(x, rog = TRUE, t50.total = TRUE, t50.germ = TRUE,
                            tmgr = TRUE, mgt = TRUE, uniformity = TRUE,
                            limits = TRUE, plotlabels = TRUE, ...){

  df <- x$data
  a <- x$a
  b <- x$b
  c <- x$c
  y0 <- x$y0
  TMGR <- x$TMGR
  MGT <- x$MGT
  Ufm <- unname(x$Uniformity[3])
  UfmMax <- unname(x$Uniformity[1])
  UfmMin <- unname(x$Uniformity[2])
  UfmMid <- UfmMax - (Ufm/2)
  t50.Germinated <- x$t50.Germinated
  t50.totalseeds <- x$t50.total
  umax = names(x$Uniformity)[1]
  umin = names(x$Uniformity)[2]


  # labels and uniformity positions
  if (limits == TRUE) {
    ypos <-  90
    ypos2 <-  80
  } else {
    ypos <-  plyr::round_any(max(df$csgp), 10, floor)
    ypos2 <- plyr::round_any(max(df$csgp), 10, floor)
  }

  if (max(df$csgp) == 0) {
    Gplot <- ggplot(data = df, aes(x = intervals, y = csgp)) +
      geom_point(alpha = 0.5) +
      labs(x = "Time", y = "Germination (%)") +
      theme_bw()
  } else {

    # curves + TMGR + MGT + t50.G + Ufm
    # Plot points + FPHF fit curve
    Gplot <- ggplot(data = df, aes(x = intervals, y = csgp)) +
      geom_point(alpha = 0.5) +
      stat_function(fun = FourPHF, colour = "red2",
                    args = list(a = a, bta = log(b, base = exp(1)),
                                c = c, y0 = y0)) +
      labs(x = "Time", y = "Germination (%)") +
      theme_bw()

    # Plot RoG curve
    if (rog == TRUE) {
      Gplot <- Gplot +
        stat_function(fun = RateofGerm, colour = "royalblue4",
                      args = list(a = a, b = b, c = c))
    }

    # Plot TMGR
    if (tmgr == TRUE) {
      Gplot <- Gplot +
        geom_vline(xintercept = TMGR, colour = "royalblue4", linetype = "dashed")
    }

    # Plot MGT
    if (mgt == TRUE) {
      Gplot <- Gplot +
        geom_vline(xintercept = MGT, colour = "green4", linetype = "dashed")
    }

    # Plot uniformity
    if (uniformity == TRUE) {
      Gplot <- Gplot +
        geom_segment(aes(x = UfmMin, xend = UfmMax, y = ypos2, yend = ypos2),
                     colour = "gray15", arrow = arrow(length = unit(0.2,"cm"))) +
        geom_segment(aes(x = UfmMax, xend = UfmMin, y = ypos2, yend = ypos2),
                     colour = "gray15", arrow = arrow(length = unit(0.2,"cm")))
    }

    # t50s
    if (t50.germ == TRUE | t50.total == TRUE ) {
      Gplot <- Gplot +
        geom_hline(yintercept = 50, colour = "black", linetype = "dotted")
      if (t50.germ == TRUE) {
        Gplot <- Gplot +
          geom_vline(xintercept = t50.Germinated, colour = "deeppink3",
                     linetype = "dashed")
      }
      if (t50.total == TRUE & max(df$csgp) >= 50) {
        Gplot <- Gplot +
          geom_vline(xintercept = t50.totalseeds, colour = "red2", linetype = "dashed")
      }
    }

    if (plotlabels == TRUE) {
      labdf <- data.frame(rbind(c(x = max(df$intervals), y = max(df$csgp),
                                  lab = "FPHF~curve", col = "red2")),
                          stringsAsFactors = F)
      if (rog == TRUE) {
        labdf <- rbind(labdf, c(x = max(df$intervals),
                                y = RateofGerm(x = max(df$intervals),
                                               a = a, b = b, c = c),
                                lab = "RoG~curve", col = "royalblue4"))
      }
      if (tmgr == TRUE) {
        labdf <- rbind(labdf, c(x = TMGR, y = ypos, lab = "TMGR",
                                col = "royalblue4"))
      }
      if (mgt == TRUE) {
        labdf <- rbind(labdf, c(x = MGT, y = ypos, lab = "MGT",col = "green4"))
      }
      if (uniformity == TRUE) {
        labdf <- rbind(labdf, c(x = UfmMid, y = ypos2,
                                lab =  paste("U","[", umax, "-", umin,"]",
                                             sep = ""), col = "gray15"))
      }
      if (t50.germ == TRUE) {
        labdf <- rbind(labdf, c(x = t50.Germinated, y = ypos,
                                lab = "t[50[Germ]]", col = "deeppink3"))
      }
      if (t50.total == TRUE & max(df$csgp) >= 50) {
        labdf <- rbind(labdf, c(x = t50.totalseeds, y = ypos, lab = "t[50[Total]]",
                                col = "red2"))
      }
      labdf$x <- as.numeric(as.character(as.factor((labdf$x))))
      labdf$y <- as.numeric(as.character(as.factor((labdf$y))))

      Gplot <- Gplot +
        geom_label_repel(data = labdf, aes(x, y, label = lab), size = 3,
                         nudge_x = 0, nudge_y = 0, colour = labdf$col,
                         na.rm = TRUE, parse = TRUE)
    }

  }


  # plot limits
  if (limits == TRUE) {
    Gplot <- Gplot + coord_cartesian(xlim = c(0, max(df$intervals)),
                                     ylim = c(0, 100))
  }



  Gplot <- Gplot +
    theme(axis.text = element_text(colour = "black"))

    # rm(df, a, b, c, y0, TMGR, MGT, Ufm,
    #    t50.Germinated, t50.totalseeds, umax, umin)

    return(Gplot)

}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Timson's index
#'
#' Compute George's index
#' \insertCite{george_influence_1961,tucker_estimating_1965,nichols_two_1968,chopra_effect_1980}{germinationmetrics},
#' Timson's index or Timson's germination velocity index
#' \insertCite{grose_laboratory_1958,timson_new_1965,lyon_rapid_1966,chaudhary_effect_1970,negm_effects_1978,brown_representing_1988,baskin_seeds_1998}{germinationmetrics}
#' and it's modifications by Labouriau
#' \insertCite{ranal_how_2006}{germinationmetrics} and
#' \insertCite{khan_effect_1984}{germinationmetrics}. \loadmathjax
#'
#' Timson's index \insertCite{timson_new_1965}{germinationmetrics} is computed
#' as follows
#' \insertCite{grose_laboratory_1958,brown_representing_1988,baskin_seeds_1998}{germinationmetrics}.
#'
#' \mjsdeqn{\Sigma k = \sum_{i=1}^{k}G_{i}}
#'
#' Where, \mjseqn{G_{i}} is the cumulative germination percentage in time
#' interval \mjseqn{i} and \mjseqn{k} is the total number of time intervals.
#'
#' For example ten summation (\mjseqn{\Sigma 10}) is expressed as follows.
#'
#' \mjsdeqn{\Sigma 10 = G_{1}+G_{2}+\cdots+G_{10}}
#'
#' Where \mjseqn{G_{1},G_{2},\cdots G_{10}} are the cumulative germination
#' percentage at day 1, 2, 3, \mjseqn{\cdots}, 10 respectively.
#'
#' Similarly \mjseqn{\Sigma 5} or \mjseqn{\Sigma 20} can be estimated. For
#' \mjseqn{\Sigma 10}, the value can range from 0 (no germination) to 1,000
#' (100\% germination after 24 hours).
#'
#' It is the progressive total of cumulative germination percentage recorded at
#' specific intervals for a set period of time.  It combines onset, rate and
#' total percentage of germination and estimates the area under the cumulative
#' germination percentage curve. It is same as the indices for area under time
#' course curve of germination described by
#' \insertCite{lyon_rapid_1966;textual}{germinationmetrics} as
#' well as \insertCite{negm_effects_1978;textual}{germinationmetrics}. It is
#' also described as Emergence Rate Index (\mjseqn{ERI}) by
#' \insertCite{chaudhary_effect_1970;textual}{germinationmetrics}.
#'
#' \insertCite{goodchild_method_1971;textual}{germinationmetrics}, described the
#' same in terms of partial germination percentage as follows.
#'
#' \mjsdeqn{\Sigma k = \sum_{i=1}^{k}g_{i}(k-j)}
#'
#' Where, \mjseqn{g_{i}} is the germination (not cumulative, but partial
#' germination) in time interval \mjseqn{i} (\mjseqn{i} varying from \mjseqn{0}
#' to \mjseqn{k}), \mjseqn{k} is the total number of time intervals, and
#' \mjseqn{j = i - 1}.
#'
#' Timson's index is equivalent to the Germination Energy Index \mjseqn{GEI}
#' proposed by \insertCite{grose_laboratory_1958;textual}{germinationmetrics}.
#'
#' As Timson's index is useful for comparison only when samples have similar
#' germinabilities or final germination percentage, the following modification
#' was suggested by Labouriau \insertCite{ranal_how_2006}{germinationmetrics}
#' (Specified by the argument \code{modification = "labouriau"}).
#'
#' \mjsdeqn{\Sigma k_{mod} = \frac{\Sigma k}{\sum_{i=1}^{k}g_{i}}}
#'
#' Here Timson's index (\mjseqn{\Sigma k}) is divided by the sum of partial
#' germination percentages.
#'
#' Similarly another modification was proposed by
#' \insertCite{khan_effect_1984}{germinationmetrics}, where Timson's index
#' (\mjseqn{\Sigma k}) is divided by the total time period of germination
#' (\mjseqn{T_{k}}) (Specified by the argument \code{modification =
#' "khanungar"}).
#'
#' \mjsdeqn{\Sigma k_{mod} = \frac{\Sigma k}{T_{k}}}
#'
#' Timson's index is similar to the Germination Rate (\mjseqn{GR}) proposed by
#' \insertCite{george_influence_1961;textual}{germinationmetrics} as follows
#' \insertCite{tucker_estimating_1965,nichols_two_1968}{germinationmetrics}.
#'
#' \mjsdeqn{GR = \sum_{i=1}^{k}N_{i}K_{i}}
#'
#' Where \mjseqn{N_{i}} is the number of seeds germinated by \mjseqn{i}th
#' interval, \mjseqn{K_{i}} is the number of intervals(e.g. days) until the end
#' of the test, and \mjseqn{k} is the total number of time intervals.
#'
#' This index uses number of seeds germinated instead of germination percentage.
#' It is also described as Emergence Rate Index (\mjseqn{ERI}) by
#' \insertCite{chopra_effect_1980;textual}{germinationmetrics}.
#'
#' So germination rate (\mjseqn{GR}) can also be described in terms of partial
#' (\mjseqn{N_{i}}) and cumulative (\mjseqn{\sum_{j=1}^{i}N_{j}}) number of
#' seeds germinated in each time interval (\mjseqn{i}) as follows.
#'
#' \mjsdeqn{GR = \sum_{i=1}^{k}\sum_{j=1}^{i}N_{j}}
#'
#' \mjsdeqn{GR = \sum_{i=1}^{k}N_{i}(k-j)}
#'
#' @inheritParams MeanGermTime
#' @param total.seeds Total number of seeds.
#' @param max The maximum interval value up to which Timson's index/George's
#'   germination rate is to be computed.
#' @param modification The modification to be applied. Either \code{"none"},
#'   \code{"labouriau"} or \code{"khanungar"}. Default is \code{"none"} (see
#'   \strong{Details}).
#'
#' @return For \code{TimsonsIndex}, the value of Timson's index (\%) if
#'   \code{modification = "none"}. If \code{modification = "labouriau"}, then
#'   value with no unit and if \code{modification = "khanungar"}, then value as
#'   \% \mjseqn{\mathrm{time^{-1}}}.
#'
#'   For \code{GermRateGeorge} The value of George's germination rate as
#'   \mjseqn{count time}.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name TimsonsIndex
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' # Without max specified
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50)
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'              modification = "none")
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'              modification = "labouriau")
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'              modification = "khanungar")
#' GermRateGeorge(germ.counts = x, intervals = int)
#'
#' # With max specified
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50, max = 10)
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'              max = 10, modification = "none")
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'              max = 10, modification = "labouriau")
#' TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
#'              max = 10, modification = "khanungar")
#' GermRateGeorge(germ.counts = x, intervals = int, max = 10)
#' GermRateGeorge(germ.counts = x, intervals = int, max = 14)
#'
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' # Without max specified
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50)
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50,
#'              modification = "none")
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50,
#'              modification = "labouriau")
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50,
#'              modification = "khanungar")
#' GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE)
#'
#' # With max specified
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50, max = 10)
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50,
#'              max = 10, modification = "none")
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50,
#'              max = 10, modification = "labouriau")
#' TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
#'              total.seeds = 50,
#'              max = 10, modification = "khanungar")
#' GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE,
#'                max = 10)
#' GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE,
#'                max = 14)

#' @rdname TimsonsIndex
#' @export
TimsonsIndex <- function(germ.counts, intervals, partial = TRUE,
                        total.seeds, max,
                        modification = c("none", "labouriau", "khanungar")) {

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  # Set max
  if (missing(max)) {
    max <- intervals[length(germ.counts)]
  }

  # Check if argument max is of type numeric with unit length
  if (!is.numeric(max) || length(max) != 1) {
    stop("'max' should be a logical vector of length 1.")
  }

  # Check max argument is present in intervals
  if (!(max %in% intervals)) {
    stop("'max' is not present in 'intervals'.")
  }

  # Check modification
  modification <- match.arg(modification)


  maxgc <- which(intervals == max)
  x <- germ.counts[1:maxgc]
  cgp <- cumsum(x)/total.seeds * 100
  pgp <- x/total.seeds * 100

  # interval <- unique(diff(intervals))

  #GI <- sum(cumsum(x))

  TI <- sum(cgp)
  # TI <- sum(pgp*(max - (intervals[1:maxgc] - interval)))
  TI <- sum(pgp*(length(intervals[1:maxgc]) -
                   (seq_along(intervals[1:maxgc]) - 1)))


  if (modification == "labouriau") {
    TI <- TI/sum(pgp)
  }

  if (modification == "khanungar") {
    TI <- TI/max(intervals)
  }

  return(TI)
}

#' @rdname TimsonsIndex
#' @export
GermRateGeorge <- function(germ.counts, intervals, partial = TRUE, max) {


  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  # Set max
  if (missing(max)) {
    max <- intervals[length(germ.counts)]
  }

  # Check if argument max is of type numeric with unit length
  if (!is.numeric(max) || length(max) != 1) {
    stop("'max' should be a logical vector of length 1.")
  }

  # Check max argument is present in intervals
  if (!(max %in% intervals)) {
    stop("'max' is not present in 'intervals'.")
  }

  maxgc <- which(intervals == max)
  x <- germ.counts[1:maxgc]

  GR <- sum(cumsum(x))
  GR <- sum(x*(length(intervals[1:maxgc]) -
                 (seq_along(intervals[1:maxgc]) - 1)))

  return(GR)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("TimsonsIndex works; Partial germination counts", {

  # From partial germination counts; max specified
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50),
               expected = 664)
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50,
                                     modification = "none"),
               expected = 664)
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50,
                                     modification = "labouriau"),
               expected = 8.3)
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50,
                                     modification = "khanungar"),
               expected = 47.4285714285714, tolerance = 1e-12)

  # From partial germination counts; max not specified
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50, max = 10),
               expected = 344)
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50, max = 10,
                                     modification = "none"),
               expected = 344)
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50, max = 10,
                                     modification = "labouriau"),
               expected = 4.41025641025641, tolerance = 1e-12)
  expect_equal(object = TimsonsIndex(germ.counts = x, intervals = int,
                                     total.seeds = 50, max = 10,
                                     modification = "khanungar"),
               expected = 24.5714285714286, tolerance = 1e-12)
})

test_that("TimsonsIndex works; Cumulative germination counts", {

  # From cumulative germination counts; max specified
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50),
               expected = 664)
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE,
                                     total.seeds = 50,
                                     modification = "none"),
               expected = 664)
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50,
                                     modification = "labouriau"),
               expected = 8.3)
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50,
                                     modification = "khanungar"),
               expected = 47.4285714285714, tolerance = 1e-12)

  # From cumulative germination counts; max not specified
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50,
                                     max = 10),
               expected = 344)
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50,
                                     max = 10, modification = "none"),
               expected = 344)
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50,
                                     max = 10, modification = "labouriau"),
               expected = 4.41025641025641, tolerance = 1e-12)
  expect_equal(object = TimsonsIndex(germ.counts = y, intervals = int,
                                     partial = FALSE, total.seeds = 50,
                                     max = 10, modification = "khanungar"),
               expected = 24.5714285714286, tolerance = 1e-12)
})

test_that("GermRateGeorge works; Partial germination counts", {

  # From partial germination counts; max specified
  expect_equal(object = GermRateGeorge(germ.counts = x, intervals = int),
               expected = 332)

  # From partial germination counts; max not specified
  expect_equal(object = GermRateGeorge(germ.counts = x, intervals = int,
                                       max = 10),
               expected = 172)
  expect_equal(object = GermRateGeorge(germ.counts = x, intervals = int,
                                       max = 14),
               expected = 332)
})

test_that("GermRateGeorge works; Cumulative germination counts", {

  # From cumulative germination counts; max specified
  expect_equal(object = GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE),
               expected = 332)

  # From cumulative germination counts; max not specified
  expect_equal(object = GermRateGeorge(germ.counts = y, intervals = int,
                                       partial = FALSE, max = 10),
               expected = 172)
  expect_equal(object = GermRateGeorge(germ.counts = y, intervals = int,
                                       partial = FALSE, max = 14),
               expected = 332)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50))
  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50,
                                         modification = "none"),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           modification = "none"))
  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50,
                                         modification = "labouriau"),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           modification = "labouriau"))
  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50,
                                         modification = "khanungar"),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           modification = "khanungar"))

  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50, max = 10),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           max = 10))
  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50, max = 10,
                                         modification = "none"),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           max = 10, modification = "none"))
  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50, max = 10,
                                         modification = "labouriau"),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           max = 10, modification = "labouriau"))
  expect_identical(object = TimsonsIndex(germ.counts = x, intervals = int,
                                         total.seeds = 50, max = 10,
                                         modification = "khanungar"),
                   expected = TimsonsIndex(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           max = 10, modification = "khanungar"))

  expect_identical(object = GermRateGeorge(germ.counts = x, intervals = int),
                   expected = GermRateGeorge(germ.counts = y, intervals = int,
                                             partial = FALSE))

  expect_identical(object = GermRateGeorge(germ.counts = x, intervals = int,
                                           max = 10),
                   expected = GermRateGeorge(germ.counts = y, intervals = int,
                                             partial = FALSE, max = 10))
  expect_identical(object = GermRateGeorge(germ.counts = x, intervals = int,
                                           max = 14),
                   expected = GermRateGeorge(germ.counts = y, intervals = int,
                                             partial = FALSE, max = 14))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/


if (getRversion() >= "2.15.1")  {
  utils::globalVariables(
    c("x", "y", "lab", "intervals", "csgp",
      "t50_Coolbear", "t50_Farooq", "GermRateRecip_Coolbear",
      "GermRateRecip_Farooq", "GermSpeed_Count", "GermSpeed_Percent",
      "GermSpeedAccumulated_Count", "GermSpeedAccumulated_Percent",
      "GermSpeedCorrected_Normal", "GermSpeedCorrected_Accumulated",
      "TimsonsIndex_Labouriau", "TimsonsIndex_KhanUngar", "GermValue_Czabator",
      "GermValue_DP", "GermValue_Czabator_mod",
      "GermValue_DP_mod","GermIndex_mod",
      "EmergenceRateIndex_SG", "EmergenceRateIndex_SG_mod",
      "EmergenceRateIndex_BilbroWanjura", "EmergenceRateIndex_Fakorede", ".",
      "a", "b", "curve", "dfgp", "gp", "y0", "isConv", "sel", "inc")
  )
}

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/


#' Modified geom_path and geom_line for Simultaneously Plot Points and Lines
#'
#' Not exported. Strictly internal
#'
#' @keywords internal
#'
#' @inheritParams ggplot2::draw_key_path
#'
#' @importFrom rlang `%||%`
#' @export
draw_key_path2 <- function(data, params, size) {

  if (is.null(data$linetype)) {
    data$linetype <- 0
  } else {
    data$linetype[is.na(data$linetype)] <- 0
  }

  # draw_key_point
  if (is.null(data$shape)) {
    data$shape <- 19
  } else if (is.character(data$shape)) {
    data$shape <- translate_shape_string(data$shape)
  }

  # NULL means the default stroke size, and NA means no stroke.
  stroke_size <- data$stroke %||% 0.5
  stroke_size[is.na(stroke_size)] <- 0

  grob <- segmentsGrob(0.1, 0.5, 0.9, 0.5,
                       gp = grid::gpar(
                         col = alpha(data$colour %||% data$fill %||% "black", data$alpha),
                         fill = alpha(params$arrow.fill %||% data$colour
                                      %||% data$fill %||% "black", data$alpha),
                         lwd = (data$linewidth %||% 0.5) * .pt,
                         lty = data$linetype %||% 1,
                         lineend = params$lineend %||% "butt"
                       ),
                       arrow = params$arrow
  )
  if (!is.null(params$arrow)) {
    angle <- deg2rad(params$arrow$angle)
    length <- grid::convertUnit(params$arrow$length, "cm", valueOnly = TRUE)
    attr(grob, "width")  <- cos(angle) * length * 1.25
    attr(grob, "height") <- sin(angle) * length * 2
  }

  pgrob <-
    pointsGrob(0.5, 0.5,
               pch = data$shape,
               gp = grid::gpar(
                 col = alpha(data$colour %||% "black", data$alpha),
                 fill = fill_alpha(data$fill %||% "black", data$alpha),
                 fontsize = (data$size %||% 1.5) * .pt + stroke_size * .stroke / 2,
                 lwd = stroke_size * .stroke / 2
               )
    )

  grid::gTree(children = grid::gList(grob, pgrob))
}




#' @rdname draw_key_path2
#'
#' @inheritParams ggplot2::geom_path
#'
#' @importFrom rlang list2
#' @importFrom stats complete.cases ave
#' @importFrom cli cli_warn cli_inform cli_abort
#' @importFrom grid segmentsGrob pointsGrob pointsGrob polylineGrob gTree convertUnit gpar gList addGrob
#' @export
#'
geom_path2 <- function(mapping = NULL, data = NULL,
                       stat = "identity", position = "identity",
                       ...,
                       lineend = "butt",
                       linejoin = "round",
                       linemitre = 10,
                       arrow = NULL,
                       na.rm = FALSE,
                       show.legend = NA,
                       inherit.aes = TRUE) {
  layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = GeomPath2,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list2(
      lineend = lineend,
      linejoin = linejoin,
      linemitre = linemitre,
      arrow = arrow,
      na.rm = na.rm,
      ...
    )
  )
}

#' @rdname draw_key_path2
#' @format NULL
#' @usage NULL
#' @export
GeomPath2 <- ggproto("GeomPath2", Geom,
                     required_aes = c("x", "y"),

                     default_aes = aes(colour = "black", linewidth = 0.5, linetype = 1, alpha = NA,
                                       # geom_point
                                       size = 1.5, shape = 19, fill = NA, stroke = 0.5),

                     non_missing_aes = c("linewidth", "colour", "linetype",
                                         # geom_point
                                         "size", "shape",
                                         # Add
                                         "show.points", "include.points"),

                     handle_na = function(self, data, params) {
                       # Drop missing values at the start or end of a line - can't drop in the
                       # middle since you expect those to be shown by a break in the line
                       aesthetics <- c(self$required_aes, self$non_missing_aes)
                       complete <- stats::complete.cases(data[names(data) %in% aesthetics])
                       kept <- stats::ave(complete, data$group, FUN = keep_mid_true)
                       data <- data[kept, ]

                       if (!all(kept) && !params$na.rm) {
                         cli::cli_warn(paste0(
                           "Removed {sum(!kept)} row{?s} containing missing values or values ",
                           "outside the scale range ({.fn {snake_class(self)}})."
                         ))
                       }

                       data
                     },

                     draw_panel = function(self, data, panel_params, coord, arrow = NULL,
                                           lineend = "butt", linejoin = "round", linemitre = 10,
                                           na.rm = FALSE) {

                       data <- fix_linewidth(data, snake_class(self))
                       if (!anyDuplicated(data$group)) {
                         cli::cli_inform(c(
                           "{.fn {snake_class(self)}}: Each group consists of only one observation.",
                           i = "Do you need to adjust the {.field group} aesthetic?"
                         ))
                       }

                       # must be sorted on group
                       data <- data[order(data$group), , drop = FALSE]
                       munched <- coord_munch(coord, data, panel_params)

                       # Add
                       if (!(is.null(munched$include.points)) &
                           !(is.null(munched$show.points))) {
                         munched$include.points <- ifelse(munched$show.points == FALSE,
                                                          TRUE,
                                                          munched$include.points)
                         munched <- munched[munched$include.points == TRUE, ]
                       }

                       # Silently drop lines with less than two points, preserving order
                       rows <- stats::ave(seq_len(nrow(munched)), munched$group, FUN = length)
                       munched <- munched[rows >= 2, ]
                       if (nrow(munched) < 2) return(zeroGrob())

                       # Work out whether we should use lines or segments
                       attr <- dapply(munched, "group", function(df) {
                         linetype <- unique0(df$linetype)
                         data_frame0(
                           solid = identical(linetype, 1) || identical(linetype, "solid"),
                           constant = nrow(unique0(df[, names(df) %in% c("alpha", "colour", "linewidth", "linetype")])) == 1,
                           .size = 1
                         )
                       })
                       solid_lines <- all(attr$solid)
                       constant <- all(attr$constant)
                       if (!solid_lines && !constant) {
                         cli::cli_abort("{.fn {snake_class(self)}} can't have varying {.field colour}, {.field linewidth}, and/or {.field alpha} along the line when {.field linetype} isn't solid.")
                       }

                       # Work out grouping variables for grobs
                       n <- nrow(munched)
                       group_diff <- munched$group[-1] != munched$group[-n]
                       start <- c(TRUE, group_diff)
                       end <-   c(group_diff, TRUE)

                       # geom_point
                       if (is.character(data$shape)) {
                         data$shape <- translate_shape_string(data$shape)
                       }

                       # geom_point
                       coords <- coord$transform(data, panel_params)

                       # Add
                       if (!(is.null(coords$show.points))) {
                         coords <- coords[coords$show.points == TRUE, ]
                       }

                       # geom_point
                       stroke_size <- coords$stroke
                       stroke_size[is.na(stroke_size)] <- 0

                       if (!constant) {

                         # MOD

                         out <- lapply(seq_along(unique0(munched$group)),
                                       function(i) {

                                         munched_id <- munched[munched$group == i, ]

                                         arrow <- repair_segment_arrow(arrow, munched_id$group)

                                         lineg_id <- grid::segmentsGrob(
                                           munched_id$x[!end], munched_id$y[!end], munched_id$x[!start], munched_id$y[!start],
                                           default.units = "native", arrow = arrow,
                                           gp = grid::gpar(
                                             col = alpha(munched_id$colour, munched_id$alpha)[!end],
                                             fill = alpha(munched_id$colour, munched_id$alpha)[!end],
                                             lwd = munched_id$linewidth[!end] * .pt,
                                             lty = munched_id$linetype[!end],
                                             lineend = lineend,
                                             linejoin = linejoin,
                                             linemitre = linemitre
                                           )
                                         )

                                         coords_id <- coords[coords$group == i, ]

                                         pointg_id <-
                                           # geom-point.R
                                           grid::pointsGrob(
                                             coords_id$x, coords_id$y,
                                             pch = coords_id$shape,
                                             gp = grid::gpar(
                                               col = alpha(coords_id$colour, coords_id$alpha),
                                               fill = fill_alpha(coords_id$fill, coords_id$alpha),
                                               # Stroke is added around the outside of the point
                                               fontsize = coords_id$size * .pt + stroke_size[coords$group == i] * .stroke / 2,
                                               lwd = coords_id$stroke * .stroke / 2
                                             )
                                           )


                                         list(lineg_id, pointg_id)
                                       })

                         out <- unlist(out, recursive = F)
                         out <- do.call(grid::gList, out)
                         out <- gTree(children = out)

                         out

                       } else {

                         # MOD

                         # id <- match(munched$group, unique0(munched$group))

                         out <- lapply(seq_along(unique0(munched$group)),
                                       function(i) {
                                         munched_id <- munched[munched$group == i, ]

                                         lineg_id <- grid::polylineGrob(
                                           munched_id$x, munched_id$y, #id = id,
                                           default.units = "native", arrow = arrow,
                                           gp = grid::gpar(
                                             col = alpha(munched_id$colour, munched_id$alpha)[start],
                                             fill = alpha(munched_id$colour, munched_id$alpha)[start],
                                             lwd = munched_id$linewidth[start] * .pt,
                                             lty = munched_id$linetype[start],
                                             lineend = lineend,
                                             linejoin = linejoin,
                                             linemitre = linemitre
                                           )
                                         )

                                         coords_id <- coords[coords$group == i, ]

                                         pointg_id <-
                                           # geom-point.R
                                           grid::pointsGrob(
                                             coords_id$x, coords_id$y,
                                             pch = coords_id$shape,
                                             gp = grid::gpar(
                                               col = alpha(coords_id$colour, coords_id$alpha),
                                               fill = fill_alpha(coords_id$fill, coords_id$alpha),
                                               # Stroke is added around the outside of the point
                                               fontsize = coords_id$size * .pt + stroke_size[coords$group == i] * .stroke / 2,
                                               lwd = coords_id$stroke * .stroke / 2
                                             )
                                           )

                                         list(lineg_id, pointg_id)
                                       })

                         out <- unlist(out, recursive = F)
                         out <- do.call(grid::gList, out)
                         out <- gTree(children = out)

                         out
                       }
                     },

                     draw_key = draw_key_path2,

                     rename_size = TRUE
)



#' @rdname draw_key_path2
#'
#' @inheritParams ggplot2::geom_line
#'
#' @export
geom_line2 <- function(mapping = NULL, data = NULL, stat = "identity",
                       position = "identity", na.rm = FALSE, orientation = NA,
                       show.legend = NA, inherit.aes = TRUE, ...) {


  layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = GeomLine2,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list2(
      na.rm = na.rm,
      orientation = orientation,
      ...
    )
  )
}

#' @rdname draw_key_path2
#' @format NULL
#' @usage NULL
#' @export
GeomLine2 <- ggproto("GeomLine2", GeomPath2,
                     setup_params = function(data, params) {
                       params$flipped_aes <- has_flipped_aes(data, params, ambiguous = TRUE)
                       params
                     },

                     extra_params = c("na.rm", "orientation"),

                     setup_data = function(data, params) {

                       data$flipped_aes <- params$flipped_aes
                       data <- flip_data(data, params$flipped_aes)
                       data <- data[order(data$PANEL, data$group, data$x), ]
                       flip_data(data, params$flipped_aes)
                     }
)

# Helper functions from ggplot2

keep_mid_true <- getFromNamespace("keep_mid_true", "ggplot2")
snake_class <- getFromNamespace("snake_class", "ggplot2")
fix_linewidth <- getFromNamespace("fix_linewidth", "ggplot2")
dapply <- getFromNamespace("dapply", "ggplot2")
unique0 <- getFromNamespace("unique0", "ggplot2")
data_frame0 <- getFromNamespace("data_frame0", "ggplot2")
fill_alpha <- getFromNamespace("fill_alpha", "ggplot2")
repair_segment_arrow <- getFromNamespace("repair_segment_arrow", "ggplot2")
snakeize <- getFromNamespace("snakeize", "ggplot2")
lower_ascii <- "abcdefghijklmnopqrstuvwxyz"
upper_ascii <- "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
to_lower_ascii <- getFromNamespace("to_lower_ascii", "ggplot2")
to_upper_ascii  <- getFromNamespace("to_upper_ascii", "ggplot2")
translate_shape_string  <- getFromNamespace("translate_shape_string", "ggplot2")
deg2rad  <- getFromNamespace("deg2rad", "ggplot2")

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Peak value and germination value
#'
#' Compute the Peak value (\mjseqn{PV}) or Emergence Energy (\mjseqn{EE})
#' \insertCite{czabator_germination_1962,bonner_ideal_1967}{germinationmetrics}
#' and Germination value (\mjseqn{GV})
#' \insertCite{czabator_germination_1962,djavanshir_germination_1976,brown_representing_1988}{germinationmetrics}.
#' \loadmathjax
#'
#' Peak value (\mjseqn{PV}) is the maximum quotient obtained by dividing
#' successive cumulative germination values by the relevant incubation time
#' \insertCite{czabator_germination_1962}{germinationmetrics}.
#'
#' \mjsdeqn{PV = \max\left ( \frac{G_{1}}{T_{1}},\frac{G_{2}}{T_{2}},\cdots
#' \frac{G_{k}}{T_{k}} \right )}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{G_{i}} is the cumulative germination
#' percentage in the \mjseqn{i}th time interval, and \mjseqn{k} is the total
#' number of time intervals.
#'
#' It represents the mean daily germination of the most vigorous component of
#' the seed lot, and is a mathematical expression of the break, or shoulder, of
#' a typical sigmoid  germination curve
#' \insertCite{djavanshir_germination_1976}{germinationmetrics}. It is the
#' accumulated number of seeds germinated at the point on the germination curve
#' at which the rate of germination starts to decrease. It is also described as
#' Emergence energy \insertCite{bonner_ideal_1967}{germinationmetrics}.
#'
#' For daily germination counts, germination value (\mjseqn{GV}) is computed as
#' follows \insertCite{czabator_germination_1962}{germinationmetrics} (Specified
#' by the argument \code{method = "czabator"}).
#'
#' \mjsdeqn{GV = PV \times MDG}
#'
#' Where, \mjseqn{PV} is the peak value, and \mjseqn{MDG} is the mean daily
#' germination percentage from the onset of germination.
#'
#' Germination value (\mjseqn{GV}) can also be computed for other time intervals
#' of successive germination counts, by replacing \mjseqn{MDG} with the mean
#' germination percentage per unit time (\mjseqn{\overline{GP}}).
#'
#' A new estimation of germination value was given by
#' \insertCite{djavanshir_germination_1976;textual}{germinationmetrics} as
#' follows (Specified by the argument \code{method = "dp"}).
#'
#' \mjsdeqn{GV = \frac{\sum DGS}{N} \times GP \times c}
#'
#' Where, \mjseqn{DGS} is the daily germination speed computed by dividing
#' cumulative germination percentage by the number of days since the onset of
#' germination, \mjseqn{N} is the frequency or number of DGS calculated during
#' the test, \mjseqn{GP} is the germination percentage expressed over 100, and
#' \mjseqn{c} is a constant. The value of \mjseqn{c} is decided on the basis of
#' average daily speed of germination (\mjseqn{\frac{\sum DGS}{N}}). If it is
#' less than 10, then \mjseqn{c} value of 10 can be used and if it is more than
#' 10, then value of 7 or 8 can be used for \mjseqn{c}.
#'
#' For both methods of computing \mjseqn{GV}, only the duration from the onset
#' of germination is considered by default. Alternatively, modified \mjseqn{GV}
#' (\mjseqn{GV_{mod}}), where the entire duration from the beginning of the test
#' is considered can be obtained by using the argument \code{from.onset = FALSE}
#' \insertCite{brown_representing_1988}{germinationmetrics}.
#'
#' @inheritParams MeanGermTime
#' @param total.seeds Total number of seeds.
#' @param method The method for computing germination value. Either
#'   \code{"czabator"} or \code{"dp"}.
#' @param from.onset logical. If \code{TRUE}, duration is considered only from
#'   the onset of germination. If \code{FALSE}, full duration of germination
#'   test is considered. Default is \code{TRUE}.
#' @param k Constant (See \strong{Details}). Default is 10.
#'
#' @return A list with the following components:  \item{Germination Value}{The
#'   germination value.} \item{Calculations}{The data frame of calculations.}
#'   \item{testend}{The end of test value (Only for method \code{dp}).}
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name GermValue
#'
#' @examples
#'
#' x <- c(0, 0, 34, 40, 21, 10, 4, 5, 3, 5, 8, 7, 7, 6, 6, 4, 0, 2, 0, 2)
#' y <- c(0, 0, 34, 74, 95, 105, 109, 114, 117, 122, 130, 137, 144, 150,
#'       156, 160, 160, 162, 162, 164)
#' int <- 1:length(x)
#' total.seeds = 200
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' PeakValue(germ.counts = x, intervals = int, total.seeds = 200)
#' GermValue(germ.counts = x, intervals = int, total.seeds = 200,
#'           method = "czabator")
#' GermValue(germ.counts = x, intervals = int, total.seeds = 200,
#'           method = "dp", k = 10)
#' GermValue(germ.counts = x, intervals = int, total.seeds = 200,
#'           method = "czabator", from.onset = FALSE)
#' GermValue(germ.counts = x, intervals = int, total.seeds = 200,
#'           method = "dp", k = 10, from.onset = FALSE)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' PeakValue(germ.counts = y, interval = int, total.seeds = 200,
#'           partial = FALSE)
#' GermValue(germ.counts = y, intervals = int, total.seeds = 200,
#'           partial = FALSE, method = "czabator")
#' GermValue(germ.counts = y, intervals = int, total.seeds = 200,
#'           partial = FALSE, method = "dp", k = 10)
#' GermValue(germ.counts = y, intervals = int, total.seeds = 200,
#'           partial = FALSE, method = "czabator", from.onset = FALSE)
#' GermValue(germ.counts = y, intervals = int, total.seeds = 200,
#'           partial = FALSE, method = "dp", k = 10, from.onset = FALSE)
#'

#' @rdname GermValue
#' @export
PeakValue <- function(germ.counts, intervals, total.seeds, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  x <- germ.counts
  csx <- cumsum(x)
  csxp <- (csx/total.seeds) * 100
  DGS <- csxp/intervals
  PV <- max(DGS, na.rm = TRUE)
  return(PV)
}

#' @rdname GermValue
#' @export
GermValue <- function(germ.counts, intervals, total.seeds, partial = TRUE,
                      method = c("czabator", "dp"), from.onset = TRUE, k = 10) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Check if GP is 0
  GP <- GermPercent(germ.counts = germ.counts, total.seeds = total.seeds,
                    partial = partial)

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
    partial <- TRUE
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check argument method
  method <- match.arg(method)

  # Check if argument k is of type numeric with unit length
  if (!is.numeric(k) || length(k) != 1) {
    stop("'k' should be a numeric vector of length 1.")
  }

  if (GP == 0) {
    warning("Final germination percentage is 0%.\n",
            "Computation of 'GermValue' is not possible.")
  }

  GV <- NA_integer_
  Calculations <- NULL
  GVdp <- NA_integer_
  testend <- NA_integer_

  if (GP > 0) {
    df <- data.frame(germ.counts, intervals)

    if (from.onset) {
      # Fetch only data from start of germination
      startindex <- min(which(df$germ.counts != 0))
      df <- df[startindex:nrow(df), ]
    }

    x <- df$germ.counts
    csx <- cumsum(x)
    csxp <- (csx/total.seeds) * 100
    DGS <-  csxp/df$intervals

  }

  if (method == "czabator") {

    if (GP > 0) {

      PV <- PeakValue(germ.counts, intervals, total.seeds, partial)
      MDG <-  MeanGermPercent(germ.counts = germ.counts, intervals = intervals,
                              total.seeds = total.seeds, partial = partial)

      GV <- MDG*PV

      Calculations <- data.frame(df, Cumulative.germ.counts = csx,
                                 Cumulative.germ.percent = csxp,
                                 DGS)
      #Calculations$GV <- Calculations$DGS*PV

    }

    output <- list(`Germination Value` = GV, Calculations)
  }

  if (method == "dp") {

    if (GP > 0) {

      N <- seq_along(DGS)
      # SumDGSbyN <- cumsum(DGS)/N
      # SumDGSbyN <- collapse::fcumsum(DGS)
      SumDGSbyN <- (cumsum(ifelse(is.na(DGS), 0, DGS)) + (DGS * 0)) / N

      GVdp <- SumDGSbyN*(csxp/100)*k

      Calculations <- data.frame(df, Cumulative.germ.counts = csx,
                                 Cumulative.germ.percent = csxp,
                                 DGS, `SumDGSbyN` = SumDGSbyN,
                                 GV = GVdp)

      testend <-
        Calculations[!is.na(GVdp) &
                       Calculations$GV ==  max(GVdp, na.rm = TRUE), ]$intervals

    }

    output <- list(`Germination Value` = max(GVdp, na.rm = TRUE), Calculations,
                   testend = testend)
  }

return(output)
}

x <- c(0, 0, 34, 40, 21, 10, 4, 5, 3, 5, 8, 7, 7, 6, 6, 4, 0, 2, 0, 2)
y <- c(0, 0, 34, 74, 95, 105, 109, 114, 117, 122, 130, 137, 144, 150,
       156, 160, 160, 162, 162, 164)
int <- 1:length(x)

test_that("PeakValue works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = PeakValue(germ.counts = x, intervals = int,
                                  total.seeds = 200),
               expected = 2.375, tolerance = 1e+12)
})

test_that("PeakValue works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = PeakValue(germ.counts = y, intervals = int,
                                  total.seeds = 200,
                                  partial = FALSE),
               expected = 2.375, tolerance = 1e+12)
})

test_that("GermValue works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermValue(germ.counts = x, intervals = int,
                                  total.seeds = 200,
                                  method = "dp", k = 10)[[1]],
               expected = 3.62118119975263, tolerance = 1e+12)
  expect_equal(object = GermValue(germ.counts = x, intervals = int,
                                  total.seeds = 200,
                                  method = "czabator",
                                  from.onset = FALSE)[[1]],
               expected = 3.39285714285714, tolerance = 1e+12)
  expect_equal(object = GermValue(germ.counts = x, intervals = int,
                                  total.seeds = 200,
                                  method = "dp", k = 10,
                                  from.onset = FALSE)[[1]],
               expected = 2.47254769040483, tolerance = 1e+12)
})

test_that("GermValue works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = GermValue(germ.counts = y, intervals = int,
                                  total.seeds = 200,
                                  partial = FALSE,
                                  method = "dp", k = 10)[[1]],
               expected = 3.62118119975263, tolerance = 1e+12)
  expect_equal(object = GermValue(germ.counts = y, intervals = int,
                                  total.seeds = 200,
                                  partial = FALSE,
                                  method = "czabator",
                                  from.onset = FALSE)[[1]],
               expected = 3.39285714285714, tolerance = 1e+12)
  expect_equal(object = GermValue(germ.counts = y, intervals = int,
                                  total.seeds = 200,
                                  partial = FALSE,
                                  method = "dp", k = 10,
                                  from.onset = FALSE)[[1]],
               expected = 2.47254769040483, tolerance = 1e+12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = PeakValue(germ.counts = x, intervals = int,
                                      total.seeds = 200),
                   expected = PeakValue(germ.counts = y, intervals = int,
                                        total.seeds = 200,
                                        partial = FALSE))
  expect_identical(object = GermValue(germ.counts = x, intervals = int,
                                      total.seeds = 200,
                                      method = "dp", k = 10)[[1]],
                   expected = GermValue(germ.counts = y, intervals = int,
                                        total.seeds = 200,
                                        partial = FALSE,
                                        method = "dp", k = 10)[[1]])
  expect_identical(object = GermValue(germ.counts = x, intervals = int,
                                      total.seeds = 200,
                                      method = "czabator",
                                      from.onset = FALSE)[[1]],
                   expected = GermValue(germ.counts = y, intervals = int,
                                        total.seeds = 200,
                                        partial = FALSE,
                                        method = "czabator",
                                        from.onset = FALSE)[[1]])
  expect_identical(object = GermValue(germ.counts = x, intervals = int,
                                      total.seeds = 200,
                                      method = "dp", k = 10,
                                      from.onset = FALSE)[[1]],
                   expected = GermValue(germ.counts = y, intervals = int,
                                        total.seeds = 200,
                                        partial = FALSE,
                                        method = "dp", k = 10,
                                        from.onset = FALSE)[[1]])
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Germination percentage
#'
#' Compute the following metrics: \describe{ \item{\code{GermPercent}}{
#' Germination percentage or Final germination percentage or Germinability
#' \insertCite{ista_chapter_2015}{germinationmetrics}.}
#' \item{\code{PeakGermPercent}}{Peak germination percentage
#' \insertCite{vallance_studies_1950,roh_maturity_2004}{germinationmetrics}.} }
#'
#' Germination percentage is computed as follows
#' \insertCite{ista_chapter_2015}{germinationmetrics}. \loadmathjax
#'
#' \mjsdeqn{GP = \frac{N_{g}}{N_{t}} \times 100}
#'
#' Where, \mjseqn{N_{g}} is the number of germinated seeds and \mjseqn{N_{t}} is
#' the total number of seeds.
#'
#' The value of \mjseqn{N_{g}} can be either specified using the argument
#' \code{germinated.seeds} or is computed from the \code{germ.counts} argument.
#'
#' Peak germination percentage is computed as follows
#' \insertCite{vallance_studies_1950,roh_maturity_2004}{germinationmetrics}.
#'
#' \mjsdeqn{PGP = \frac{N_{max}}{N_{t}} \times 100}
#'
#' Where, \mjseqn{N_{max}} is the maximum number of seeds germinated per
#' interval.
#'
#' @param germinated.seeds Number of germinated seeds
#' @param germ.counts Germination counts at each time interval. Can be partial
#'   or cumulative as specified in the argument \code{partial}.
#' @param intervals The time intervals.
#' @param total.seeds Total number of seeds.
#' @param partial logical. If \code{TRUE}, \code{germ.counts} is considered as
#'   partial and if \code{FALSE}, it is considered as cumulative. Default is
#'   \code{TRUE}.
#'
#' @return The germination percentage (\%) value.
#' @export
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' z <- c(0, 0, 0, 0, 11, 11, 9, 7, 1, 0, 1, 0, 0, 0)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' GermPercent(germ.counts = x, total.seeds = 50)
#' PeakGermPercent(germ.counts = x, intervals = int, total.seeds = 50)
#' # For multiple peak germination times
#' PeakGermPercent(germ.counts = z, intervals = int, total.seeds = 50)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' GermPercent(germ.counts = y, total.seeds = 50, partial = FALSE)
#' PeakGermPercent(germ.counts = y, intervals = int, total.seeds = 50,
#'                 partial = FALSE)
#' # For multiple peak germination times
#' PeakGermPercent(germ.counts = cumsum(z), intervals = int, total.seeds = 50,
#'                 partial = FALSE)
#'
#' # From number of germinated seeds
#' #----------------------------------------------------------------------------
#' GermPercent(germinated.seeds = 40, total.seeds = 50)
#'


#' @rdname GermPercent
#' @export
GermPercent <- function(germinated.seeds, germ.counts, total.seeds,
                        partial = TRUE) {
  gcheck <- c(missing(germinated.seeds), missing(germ.counts))

  # Check if both germinated.seeds and germ.counts are provided
  if (identical(gcheck, c(FALSE, FALSE))) {
    stop("Provide only either one of the two arguments\n'germinated.seeds' or 'germ.counts' and not both")
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  if (!missing(germinated.seeds)) { # When germinated.seeds is provided
    # Check if argument germinated.seeds is of type numeric with unit length
    if (!is.numeric(germinated.seeds) || length(germinated.seeds) != 1) {
      stop("'germinated.seeds' should be a numeric vector of length 1.")
    }
  } else {# When germ.counts is provided

    # Check if argument germ.counts is of type numeric
    if (!is.numeric(germ.counts)) {
      stop("'germ.counts' should be a numeric vector.")
    }

    # Check if argument partial is of type logical with unit length
    if (!is.logical(partial) || length(partial) != 1) {
      stop("'partial' should be a logical vector of length 1.")
    }

    # Check if data is cumulative
    if (!partial) {
      if(is.unsorted(germ.counts)) {
        stop("'germ.counts' is not cumulative.")
      }
    }

    # Convert cumulative to partial
    if (!partial) {
      germ.counts <- c(germ.counts[1], diff(germ.counts))
    }
    germinated.seeds <- sum(germ.counts)
  }

  GP <- (germinated.seeds/total.seeds)*100
  return(GP)
}

#' @rdname GermPercent
#' @export
PeakGermPercent <- function(germ.counts, intervals, total.seeds,
                            partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts
  PGT <- intervals[which(x %in% max(x))]

  if (length(PGT) > 1) {
    warning("Multiple peak germination times exist.")
  }

  PGP <- (germ.counts[PGT[1]]/total.seeds)*100
  return(PGP)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
z <- c(0, 0, 0, 0, 11, 11, 9, 7, 1, 0, 1, 0, 0, 0)
int <- 1:length(x)

test_that("GermPercent works; Number of germinated seeds", {

  # From number of germinated seeds
  expect_equal(GermPercent(germinated.seeds = 40, total.seeds = 50),
               expected = 80)

  #-----------------------------------------------------------------------------

  # Error in case germinated.seeds' is not a numeric vector of length 1
  expect_error(object = GermPercent(germinated.seeds = as.character(40),
                                    total.seeds = 50),
               regexp = "'germinated.seeds' should be a numeric vector of length 1.")
  expect_error(object = GermPercent(germinated.seeds = c(20, 40),
                                    total.seeds = 50),
               regexp = "'germinated.seeds' should be a numeric vector of length 1.")

  # Error in case total.seeds' is not a numeric vector of length 1
  expect_error(object = GermPercent(germinated.seeds = 40,
                                    total.seeds = as.character(50)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = GermPercent(germinated.seeds = 40,
                                    total.seeds = c(50, 25)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")


})

test_that("GermPercent works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermPercent(germ.counts = x, total.seeds = 50),
               expected = 80)

  #-----------------------------------------------------------------------------

  # Error in case both germ.counts and germinated.seeds are provided
  expect_error(object = GermPercent(germ.counts = x, total.seeds = 50,
                                    germinated.seeds = 40),
               regexp = "Provide only either one of the two arguments\n'germinated.seeds' or 'germ.counts' and not both")

    # Error in case germ.counts is not a numeric vector
  expect_error(object = GermPercent(germ.counts = as.character(x),
                                    total.seeds = 50),
               regexp = "'germ.counts' should be a numeric vector.")

  # Error in case total.seeds' is not a numeric vector of length 1
  expect_error(object = GermPercent(germ.counts = x,
                                    total.seeds = as.character(50)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = GermPercent(germ.counts = x,
                                    total.seeds = c(50, 25)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")

  # Error in case partial is not a numeric vector of length 1
  expect_error(object = GermPercent(germ.counts = x, total.seeds = 50,
                                    partial = 25),
               regexp = "'partial' should be a logical vector of length 1.")
  expect_error(object = GermPercent(germ.counts = x, total.seeds = 50,
                                    partial = c(TRUE, FALSE)),
               regexp = "'partial' should be a logical vector of length 1.")

})

test_that("GermPercent works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = GermPercent(germ.counts = y, total.seeds = 50,
                                    partial = FALSE),
               expected = 80)

  #-----------------------------------------------------------------------------

  # Error in case both germ.counts and germinated.seeds are provided
  expect_error(object = GermPercent(germ.counts = y, total.seeds = 50,
                                    germinated.seeds = 40, partial = FALSE),
               regexp = "Provide only either one of the two arguments\n'germinated.seeds' or 'germ.counts' and not both")


  # Error in case germ.counts is not a numeric vector
  expect_error(object = GermPercent(germ.counts = as.character(y),
                                    total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' should be a numeric vector.")

  # Error in case total.seeds' is not a numeric vector of length 1
  expect_error(object = GermPercent(germ.counts = x,
                                    total.seeds = as.character(50),
                                    partial = FALSE),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = GermPercent(germ.counts = x,
                                    total.seeds = c(50, 25),
                                    partial = FALSE),
               regexp = "'total.seeds' should be a numeric vector of length 1.")

  # Error in case partial is not a numeric vector of length 1
  expect_error(object = GermPercent(germ.counts = y, total.seeds = 50,
                                    partial = 25),
               regexp = "'partial' should be a logical vector of length 1.")
  expect_error(object = GermPercent(germ.counts = y, total.seeds = 50,
                                    partial = c(TRUE, FALSE)),
               regexp = "'partial' should be a logical vector of length 1.")

  # Error in case germ.counts is not cumulative
  expect_error(object = GermPercent(germ.counts = x, total.seeds = 50,
                                    partial = FALSE),
               regexp = "'germ.counts' is not cumulative.")
})

test_that("PeakGermPercent works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = PeakGermPercent(germ.counts = x, intervals = int,
                                        total.seeds = 50),
               expected = 34)

  #-----------------------------------------------------------------------------

  # Error in case germ.counts is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = as.character(x),
                                        intervals = int,
                                        total.seeds = 50),
               regexp = "'germ.counts' should be a numeric vector.")

  # Error in case intervals is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = x,
                                        intervals = as.character(int),
                                        total.seeds = 50),
               regexp = "'intervals' should be a numeric vector.")

  # Error in case germ.counts and intervals are of equal length
  expect_error(object = PeakGermPercent(germ.counts = x, intervals = int[-1],
                                        total.seeds = 50),
               regexp = "'germ.counts' and 'intervals' lengths differ.")
  expect_error(object = PeakGermPercent(germ.counts = x[-1], intervals = int,
                                        total.seeds = 50),
               regexp = "'germ.counts' and 'intervals' lengths differ.")

  # Error in case total.seeds' is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = x, intervals = int,
                                        total.seeds = as.character(50)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = x, intervals = int,
                                        total.seeds = c(50, 25)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")

  # Error in case partial is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = x, intervals = int,
                                        total.seeds = 50,
                                        partial = 25),
               regexp = "'partial' should be a logical vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = x, intervals = int,
                                        total.seeds = 50,
                                        partial = c(TRUE, FALSE)),
               regexp = "'partial' should be a logical vector of length 1.")
})

test_that("PeakGermPercent works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = PeakGermPercent(germ.counts = y, intervals = int,
                                        total.seeds = 50, partial = FALSE),
               expected = 34)

  #-----------------------------------------------------------------------------

  # Error in case germ.counts is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = as.character(y),
                                        intervals = int,
                                        total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' should be a numeric vector.")

  # Error in case intervals is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = y,
                                        intervals = as.character(int),
                                        total.seeds = 50, partial = FALSE),
               regexp = "'intervals' should be a numeric vector.")

  # Error in case germ.counts and intervals are of equal length
  expect_error(object = PeakGermPercent(germ.counts = y, intervals = int[-1],
                                        total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' and 'intervals' lengths differ.")
  expect_error(object = PeakGermPercent(germ.counts = y[-1], intervals = int,
                                        total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' and 'intervals' lengths differ.")

  # Error in case total.seeds is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = y, intervals = int,
                                        total.seeds = as.character(50),
                                        partial = FALSE),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = y, intervals = int,
                                        total.seeds = c(50, 25),
                                        partial = FALSE),
               regexp = "'total.seeds' should be a numeric vector of length 1.")

  # Error in case partial is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = y, intervals = int,
                                        total.seeds = 50,
                                        partial = 25),
               regexp = "'partial' should be a logical vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = y, intervals = int,
                                        total.seeds = 50,
                                        partial = c(TRUE, FALSE)),
               regexp = "'partial' should be a logical vector of length 1.")

  # Error in case germ.counts is not cumulative
  expect_error(object = PeakGermPercent(germ.counts = x, intervals = int,
                                        total.seeds = 50,
                                        partial = FALSE),
               regexp = "'germ.counts' is not cumulative.")
})

test_that("PeakGermPercent with multiple peaks works; Partial germination counts", {

  # From partial germination counts
  expect_warning(object = out1 <- PeakGermPercent(germ.counts = z,
                                                  intervals = int,
                                                  total.seeds = 50),
                 regexp = "Multiple peak germination times exist.")
  expect_equal(object = out1, expected = 22)

  #-----------------------------------------------------------------------------

  # Error in case germ.counts is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = as.character(z),
                                        intervals = int,
                                        total.seeds = 50),
               regexp = "'germ.counts' should be a numeric vector.")

  # Error in case intervals is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = z,
                                        intervals = as.character(int),
                                        total.seeds = 50),
               regexp = "'intervals' should be a numeric vector.")

  # Error in case germ.counts and intervals are of equal length
  expect_error(object = PeakGermPercent(germ.counts = z, intervals = int[-1],
                                        total.seeds = 50),
               regexp = "'germ.counts' and 'intervals' lengths differ.")
  expect_error(object = PeakGermPercent(germ.counts = z[-1], intervals = int,
                                        total.seeds = 50),
               regexp = "'germ.counts' and 'intervals' lengths differ.")

  # Error in case total.seeds' is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = z, intervals = int,
                                        total.seeds = as.character(50)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = z, intervals = int,
                                        total.seeds = c(50, 25)),
               regexp = "'total.seeds' should be a numeric vector of length 1.")

  # Error in case partial is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = z, intervals = int,
                                        total.seeds = 50,
                                        partial = 25),
               regexp = "'partial' should be a logical vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = z, intervals = int,
                                        total.seeds = 50,
                                        partial = c(TRUE, FALSE)),
               regexp = "'partial' should be a logical vector of length 1.")

})

test_that("PeakGermPercent with multiple peaks works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_warning(object = out2 <- PeakGermPercent(germ.counts = cumsum(z),
                                                  intervals = int,
                                                  total.seeds = 50,
                                                  partial = FALSE),
                 regexp = "Multiple peak germination times exist.")
  expect_equal(object = out2, expected = 22)

  #-----------------------------------------------------------------------------

  # Error in case germ.counts is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = as.character(cumsum(z)),
                                        intervals = int,
                                        total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' should be a numeric vector.")

  # Error in case intervals is not a numeric vector
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z),
                                        intervals = as.character(int),
                                        total.seeds = 50, partial = FALSE),
               regexp = "'intervals' should be a numeric vector.")

  # Error in case germ.counts and intervals are of equal length
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z),
                                        intervals = int[-1],
                                        total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' and 'intervals' lengths differ.")
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z)[-1],
                                        intervals = int,
                                        total.seeds = 50, partial = FALSE),
               regexp = "'germ.counts' and 'intervals' lengths differ.")

  # Error in case total.seeds is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z),
                                        intervals = int,
                                        total.seeds = as.character(50),
                                        partial = FALSE),
               regexp = "'total.seeds' should be a numeric vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z),
                                        intervals = int,
                                        total.seeds = c(50, 25),
                                        partial = FALSE),
               regexp = "'total.seeds' should be a numeric vector of length 1.")

  # Error in case partial is not a numeric vector of length 1
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z),
                                        intervals = int,
                                        total.seeds = 50,
                                        partial = 25),
               regexp = "'partial' should be a logical vector of length 1.")
  expect_error(object = PeakGermPercent(germ.counts = cumsum(z),
                                        intervals = int,
                                        total.seeds = 50,
                                        partial = c(TRUE, FALSE)),
               regexp = "'partial' should be a logical vector of length 1.")

  # Error in case germ.counts is not cumulative
  expect_error(object = PeakGermPercent(germ.counts = z, intervals = int,
                                        total.seeds = 50,
                                        partial = FALSE),
               regexp = "'germ.counts' is not cumulative.")
})


test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = GermPercent(germ.counts = x, total.seeds = 50),
                   expected = GermPercent(germ.counts = y, total.seeds = 50,
                                          partial = FALSE))
  expect_identical(object = PeakGermPercent(germ.counts = x, intervals = int,
                                            total.seeds = 50),
                   expected = PeakGermPercent(germ.counts = y, intervals = int,
                                              total.seeds = 50,
                                              partial = FALSE))

  expect_warning(object = out1 <- PeakGermPercent(germ.counts = z,
                                                  intervals = int,
                                                  total.seeds = 50),
                 regexp = "Multiple peak germination times exist.")
  expect_warning(object = out2 <- PeakGermPercent(germ.counts = cumsum(z),
                                                  intervals = int,
                                                  total.seeds = 50,
                                                  partial = FALSE),
                 regexp = "Multiple peak germination times exist.")
  expect_equal(object = out1, expected = out2)
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Germination time
#'
#' Compute the following metrics:\describe{ \item{\code{MeanGermTime}}{Mean
#' germination time (\mjseqn{\overline{T}}) or Mean length of incubation time
#' \insertCite{edmond_effects_1958,czabator_germination_1962,mock_cold_1972,ellis_improved_1980,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
#' \item{\code{VarGermTime}}{Variance of germination time (\mjseqn{s_{T}^{2}})
#' \insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
#' \item{\code{SEGermTime}}{Standard error of germination time
#' (\mjseqn{s_{\overline{T}}})
#' \insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
#' \item{\code{CVGermTime}}{Coefficient of variation of the germination time
#' (\mjseqn{CV_{T}})
#' \insertCite{gomes_curso_1960,ranal_how_2006}{germinationmetrics}.} }
#' \loadmathjax
#'
#' \code{MeanGermTime} computes the mean germination time according to the
#' following formula
#' \insertCite{edmond_effects_1958,czabator_germination_1962,smith_germinating_1964,ellis_improved_1980,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{\overline{T} = \frac{\sum_{i=1}^{k}N_{i}T_{i}}{\sum_{i=1}^{k}N_{i}}}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
#' the \mjseqn{i}th time interval (not the accumulated number, but the number
#' corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
#' number of time intervals.
#'
#' It is the same as Sprouting Index (\mjseqn{SI}) or Emergence Index
#' (\mjseqn{EI}) described by
#' \insertCite{smith_germinating_1964;textual}{germinationmetrics} and
#' \insertCite{mock_cold_1972;textual}{germinationmetrics} as well as
#' Germination Resistance (\mjseqn{GR}) described by
#' \insertCite{gordon_observations_1969,gordon_germination_1971;textual}{germinationmetrics}.
#'
#' It is the inverse of mean germination rate (\mjseqn{\overline{V}}).
#'
#' \mjsdeqn{\overline{T} = \frac{1}{\overline{V}}}
#'
#' It indicates the average length of time required for maximum germination of a
#' seed lot. Lower the \mjseqn{\overline{T}}, faster the sample has germinated
#' and reflects seed vigor.
#'
#' \code{VarGermTime} computes the variance of germination time according to the
#' following formula
#' \insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{s_{T}^{2} =
#' \frac{\sum_{i=1}^{k}N_{i}(T_{i}-\overline{T})^{2}}{\sum_{i=1}^{k}N_{i}-1}}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
#' the \mjseqn{i}th time interval (not the accumulated number, but the number
#' corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
#' number of time intervals.
#'
#' \code{SEGermTime} computes the standard error of germination time
#' (\mjseqn{s_{\overline{T}}}) according to the following formula
#' \insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{s_{\overline{T}} = \sqrt{\frac{s_{T}^{2}}{\sum_{i=1}^{k}N_{i}}}}
#'
#' Where, \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th
#' time interval (not the accumulated number, but the number corresponding to
#' the \mjseqn{i}th interval), and \mjseqn{k} is the total number of time
#' intervals.
#'
#' It signifies the accuracy of the calculation of the mean germination time.
#'
#' \code{CVGermTime} computes the coefficient of variation of germination time
#' (\mjseqn{CV_{T}}) according to the following formula
#' \insertCite{gomes_curso_1960,ranal_how_2006}{germinationmetrics}.
#'
#' \mjsdeqn{CV_{T} = \frac{\sqrt{s_{T}^{2}}}{\overline{T}}}
#'
#' This indicates the uniformity of germination and permits comparisons
#' irrespective of the magnitude of mean germination time
#' (\mjseqn{\overline{T}}).
#'
#' @param germ.counts Germination counts at each time interval. Can be partial
#'   or cumulative as specified in the argument \code{partial}.
#' @param intervals The time intervals.
#' @param partial logical. If \code{TRUE}, \code{germ.counts} is considered as
#'   partial and if \code{FALSE}, it is considered as cumulative. Default is
#'   \code{TRUE}.
#'
#' @return For \code{MeanGermTime}, the mean germination time value in the same
#'   unit of time as specified in the argument \code{intervals}.
#'
#'   For \code{VarGermTime}, the variance of germination time value as
#'   \mjseqn{\mathrm{time^{2}}}.
#'
#'   For \code{SEGermTime}, the standard error of germination time in the same
#'   unit of time specified in the argument \code{intervals}.
#'
#'   For \code{CVGermTime}, the value of coefficient of variation of the
#'   germination time.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name MeanGermTime
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' MeanGermTime(germ.counts = x, intervals = int)
#' VarGermTime(germ.counts = x, intervals = int)
#' SEGermTime(germ.counts = x, intervals = int)
#' CVGermTime(germ.counts = x, intervals = int)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' MeanGermTime(germ.counts = y, intervals = int, partial = FALSE)
#' VarGermTime(germ.counts = y, intervals = int, partial = FALSE)
#' SEGermTime(germ.counts = y, intervals = int, partial = FALSE)
#' CVGermTime(germ.counts = y, intervals = int, partial = FALSE)
#'
#' @seealso \code{\link[germinationmetrics]{MeanGermRate}}

#' @rdname MeanGermTime
#' @export
MeanGermTime <- function(germ.counts, intervals, partial = TRUE) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Check if data is cumulative
  if (!partial) {
    if(is.unsorted(germ.counts)) {
      stop("'germ.counts' is not cumulative.")
    }
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts
  MGT <- sum(x*intervals)/sum(x)

  return(MGT)
}


#' @rdname MeanGermTime
#' @export
VarGermTime <- function(germ.counts, intervals, partial = TRUE) {

  MGT <- MeanGermTime(germ.counts, intervals, partial)
  intervalsdiff <- (intervals - MGT)^2

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  VGT <- sum(germ.counts*intervalsdiff)/(sum(germ.counts) - 1)

  return(VGT)
}

#' @rdname MeanGermTime
#' @export
SEGermTime <- function(germ.counts, intervals, partial = TRUE) {

  VGT <- VarGermTime(germ.counts, intervals, partial)

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  SEGT <- sqrt(VGT/sum(germ.counts))

  return(SEGT)
}

#' @rdname MeanGermTime
#' @export
CVGermTime <- function(germ.counts, intervals, partial = TRUE) {

  MGT <- MeanGermTime(germ.counts, intervals, partial)
  VGT <- VarGermTime(germ.counts, intervals, partial)
  CVGT <- sqrt(VGT)/MGT

  return(CVGT)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

test_that("MeanGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = MeanGermTime(germ.counts = x, intervals = int),
               expected = 6.7, tolerance = 1e-12)
})

test_that("MeanGermTime works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = MeanGermTime(germ.counts = y, intervals = int,
                                     partial = FALSE),
               expected = 6.7, tolerance = 1e-12)
})

test_that("VarGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = VarGermTime(germ.counts = x, intervals = int),
               expected = 1.44615384615385, tolerance = 1e-12)
})

test_that("VarGermTime works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = VarGermTime(germ.counts = y, intervals = int,
                                     partial = FALSE),
               expected = 1.44615384615385, tolerance = 1e-12)
})

test_that("SEGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = SEGermTime(germ.counts = x, intervals = int),
               expected = 0.190141647604743, tolerance = 1e-12)
})

test_that("SEGermTime works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = SEGermTime(germ.counts = y, intervals = int,
                                    partial = FALSE),
               expected = 0.190141647604743, tolerance = 1e-12)
})

test_that("CVGermTime works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = CVGermTime(germ.counts = x, intervals = int),
               expected = 0.179486771488981, tolerance = 1e-12)
})

test_that("CVGermTime works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = CVGermTime(germ.counts = y, intervals = int,
                                   partial = FALSE),
               expected = 0.179486771488981, tolerance = 1e-12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = MeanGermTime(germ.counts = x, intervals = int),
                   expected = MeanGermTime(germ.counts = y, intervals = int,
                                           partial = FALSE))
  expect_identical(object = VarGermTime(germ.counts = x, intervals = int),
                   expected = VarGermTime(germ.counts = y, intervals = int,
                                          partial = FALSE))
  expect_identical(object = SEGermTime(germ.counts = x, intervals = int),
                   expected = SEGermTime(germ.counts = y, intervals = int,
                                         partial = FALSE))
  expect_identical(object = CVGermTime(germ.counts = x, intervals = int),
                   expected = CVGermTime(germ.counts = y, intervals = int,
                                         partial = FALSE))
})

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Germination count data
#'
#' An example germination count dataset. It includes germination count data over
#' 14 days from five genotypes (G1 to G5) in three replications
#'
#' @format A data frame with 17 columns: \describe{ \item{Genotype}{The
#'   genotype.} \item{Rep}{Replication.} \item{Day01}{Number of seeds that
#'   germinated (germination counts) on Day 1.} \item{Day02}{Number of seeds
#'   that germinated (germination counts) on Day 2.} \item{Day03}{Number of
#'   seeds that germinated (germination counts) on Day 3.} \item{Day04}{Number
#'   of seeds that germinated (germination counts) on Day 4.}
#'   \item{Day05}{Number of seeds that germinated (germination counts) on Day
#'   5.} \item{Day06}{Number of seeds that germinated (germination counts) on
#'   Day 6.} \item{Day07}{Number of seeds that germinated (germination counts)
#'   on Day 7.} \item{Day08}{Number of seeds that germinated (germination
#'   counts) on Day 8.} \item{Day09}{Number of seeds that germinated
#'   (germination counts) on Day 9.} \item{Day10}{Number of seeds that
#'   germinated (germination counts) on Day 10.} \item{Day11}{Number of seeds
#'   that germinated (germination counts) on Day 11.} \item{Day12}{Number of
#'   seeds that germinated (germination counts) on Day 12.} \item{Day13}{Number
#'   of seeds that germinated (germination counts) on Day 13.}
#'   \item{Day14}{Number of seeds that germinated (germination counts) on Day
#'   14.} \item{Total Seeds}{Total number of seeds tested.} }
#'
#' @examples
#'
#' \donttest{
#' data(gcdata)
#' library(ggplot2)
#' library(reshape2)
#'
#' # Plot partial germination counts over time
#' #----------------------------------------------------------------------------
#' # Convert wide-from to long-form
#' gcdatamelt <- melt(gcdata[, !names(gcdata) %in% c("Total Seeds")],
#'                    id.vars = c("Genotype", "Rep"))
#'
#' ggplot(gcdatamelt, aes(x = variable, y = value,
#'                        group = interaction(Genotype, Rep),
#'                        colour = Genotype)) +
#'   geom_point(alpha = 0.7) +
#'   geom_line(alpha = 0.7) +
#'   ylab("Germination count (Partial)") +
#'   xlab("Intervals") +
#'   theme_bw()
#'
#' # Plot partial germination counts over time
#' #----------------------------------------------------------------------------
#'
#' # Convert wide-from to long-form
#' # Compute cumulative germination counts
#' gcdata2 <- gcdata
#' gcdata2[, !names(gcdata2) %in% c("Genotype", "Rep", "Total Seeds")] <-
#'   t(apply(gcdata2[, !names(gcdata2) %in% c("Genotype", "Rep", "Total Seeds")], 1, cumsum))
#'
#'
#' gcdatamelt2 <- melt(gcdata2[, !names(gcdata2) %in% c("Total Seeds")],
#'                     id.vars = c("Genotype", "Rep"))
#'
#' ggplot(gcdatamelt2, aes(x = variable, y = value,
#'                         group = interaction(Genotype, Rep),
#'                         colour = Genotype)) +
#'   geom_point(alpha = 0.7) +
#'   geom_line(alpha = 0.7) +
#'   ylab("Germination count (Cumulative)") +
#'   xlab("Intervals") +
#'   theme_bw()
#'
#' # Compute germination indices
#' #----------------------------------------------------------------------------
#'
#' counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
#'                           "Day06", "Day07", "Day08", "Day09", "Day10",
#'                           "Day11", "Day12", "Day13", "Day14")
#' germination.indices(gcdata, total.seeds.col = "Total Seeds",
#'                     counts.intervals.cols = counts.per.intervals,
#'                     intervals = 1:14, partial = TRUE, max.int = 5)
#' }
#'
"gcdata"

### This file is part of 'germinationmetrics' package for R.

### Copyright (C) 2017-2026, ICAR-NBPGR.
#
# germinationmetrics is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# germinationmetrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# https://www.r-project.org/Licenses/

#' Speed of germination
#'
#' Compute the following metrics: \describe{ \item{\code{GermSpeed}}{Speed of
#' germination (\mjseqn{S}) or Germination rate index or Index of velocity of germination or
#' Germination index or Emergence rate index
#' \insertCite{throneberry_relation_1955,maguire_speed_1962,allan_seedling_1962,kendrick_photocontrol_1969,bouton_germination_1976,erbach_tillage_1982,aosa_seed_1983,khandakar_jute_1983,hsu_planting_1986,bradbeer_seed_1988,wardle_allelopathic_1991}{germinationmetrics}.
#' } \item{\code{GermSpeedAccumulated}}{Speed of accumulated germination (\mjseqn{S_{accumulated}})
#' \insertCite{bradbeer_seed_1988,wardle_allelopathic_1991,haugland_experiments_1996,de_santana_alise_2004}{germinationmetrics}.
#' } \item{\code{GermSpeedCorrected}}{Corrected speed of germination (\mjseqn{\hat{S}}) or
#' Corrected germination rate index
#' \insertCite{evetts_germination_1972}{germinationmetrics}.} } \loadmathjax
#'
#' \code{GermSpeed} computes the speed of germination according to the following
#' formula
#' \insertCite{throneberry_relation_1955,maguire_speed_1962,allan_seedling_1962,kendrick_photocontrol_1969,bouton_germination_1976,erbach_tillage_1982,aosa_seed_1983,khandakar_jute_1983,hsu_planting_1986,bradbeer_seed_1988,wardle_allelopathic_1991}{germinationmetrics}.
#'
#' \mjsdeqn{S = \frac{N_{1}}{T_{1}} + \frac{N_{2}}{T_{2}} + \frac{N_{3}}{T_{3}}
#' + \cdots + \frac{N_{k}}{T_{k}}}
#'
#' Where, \mjseqn{N_{1}}, \mjseqn{N_{2}}, \mjseqn{N_{3}}, \mjseqn{\cdots},
#' \mjseqn{N_{k}} are the number of germinated seeds observed at time (days or
#' hours) \mjseqn{T_{1}}, \mjseqn{T_{2}}, \mjseqn{T_{3}}, \mjseqn{\cdots},
#' \mjseqn{T_{k}} after sowing (Not accumulated/cumulative number, but the
#' number of seeds that germinated at the specific time) and \mjseqn{k} is the
#' total number of time intervals.
#'
#' It is the same as Emergence Rate Index (\mjseqn{ERI}) described by
#' \insertCite{allan_seedling_1962;textual}{germinationmetrics},
#' \insertCite{erbach_tillage_1982;textual}{germinationmetrics} and
#' \insertCite{hsu_planting_1986;textual}{germinationmetrics} as well as
#' Germination Index (\mjseqn{GI}) according to
#' \insertCite{aosa_seed_1983;textual}{germinationmetrics}.
#'
#' The formula can also be described as follows.
#'
#' \mjsdeqn{S = \sum_{i=1}^{k}\frac{N_{i}}{T_{i}}}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
#' the \mjseqn{i}th time interval (not the accumulated number, but the number
#' corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
#' number of time intervals.
#'
#' \code{GermSpeedAccumulated} computes the speed of accumulated germination as
#' follows
#' \insertCite{bradbeer_seed_1988,wardle_allelopathic_1991,haugland_experiments_1996,de_santana_alise_2004}{germinationmetrics}.
#'
#' \mjsdeqn{S_{accumulated} = \frac{N_{1}}{T_{1}} + \frac{N_{1} + N_{2}}{T_{2}}
#' + \frac{N_{1} + N_{2} + N_{3}}{T_{3}} + \cdots +\frac{N_{1} + N_{2} + \cdots
#' + N_{k}}{T_{k}}}
#'
#' Where, \mjseqn{N_{1}}, \mjseqn{N_{2}}, \mjseqn{N_{3}}, \mjseqn{\cdots},
#' \mjseqn{N_{k}} are the number of germinated seeds observed at time (days or
#' hours) \mjseqn{T_{1}}, \mjseqn{T_{2}}, \mjseqn{T_{3}}, \mjseqn{\cdots},
#' \mjseqn{T_{k}} after sowing (Not accumulated/cumulative number, but the
#' number of seeds that germinated at the specific time), and \mjseqn{k} is the
#' total number of time intervals.
#'
#' The formula can also be described as follows.
#'
#' \mjsdeqn{S_{accumulated} = \sum_{i=1}^{k}\frac{\sum_{j=1}^{i}N_{j}}{T_{i}}}
#'
#' Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
#' \mjseqn{i}th interval, \mjseqn{\sum_{j=1}^{i}N_{j}} is the
#' cumuative/accumulated number of seeds germinated in the \mjseqn{i}th time
#' interval, and \mjseqn{k} is the total number of time intervals.
#'
#' Speed of germination expresses the rate of germination in terms of the total
#' number of seeds that germinate in a time interval. Higher values indicate
#' greater and faster germination. This is useful for comparisons only when
#' samples or treatments possess similar germinabilities. This is overcome by
#' either using the corrected speed of germination or by using germination
#' percentages instead of counts for computing speed.
#'
#' \code{GermSpeedCorrected} computes the corrected speed of germination
#' \mjseqn{S} or corrected speed of accumulated germination \mjseqn{\hat{S}} as
#' follows \insertCite{evetts_germination_1972}{germinationmetrics}.
#'
#' \mjsdeqn{\hat{S} = \frac{S}{FGP}}
#'
#' \mjsdeqn{\hat{S}_{accumulated} = \frac{S_{accumulated}}{FGP}}
#'
#' Where, \mjseqn{S} is the germination speed or \mjseqn{\hat{S}} is the
#' accumulated speed of germination (as
#' specified by the argument  \code{method = "normal"} or \code{method =
#' "accumulated"} respectively) computed with germination percentage instead of
#' counts and \mjseqn{FGP} is the final germination percentage or germinability.
#'
#' With \code{percent = TRUE}, germination percentage is used instead of counts
#' for computation in \code{GermSpeed} and \code{GermSpeedAccumulated}. In case
#' of \code{GermSpeedCorrected}, germination percentage is always used for the
#' numerator.
#'
#' @inheritParams MeanGermTime
#' @param percent logical. If \code{TRUE}, germination percentage is used
#'   instead of counts for computation. Default is \code{FALSE}.
#' @param total.seeds Total number of seeds. Mandatory for computation when
#'   \code{percent = TRUE}.
#' @param method The method for computing germination speed in
#'   \code{GermSpeedCorrected}. Either \code{"normal"} (\code{GermSpeed}) or
#'   \code{"accumulated"} (\code{GermSpeedAccumulated}).
#'
#' @return For \code{GermSpeed}, the value of germination speed as
#'   \mjseqn{\mathrm{count \, time^{-1}}} or \% \mjseqn{\mathrm{time^{-1}}}.
#'
#'   For \code{GermSpeedAccumulated}, the value of accumulated germination speed
#'   as \mjseqn{\mathrm{count \, time^{-1}}} or \% \mjseqn{\mathrm{time^{-1}}}.
#'
#'   For \code{GermSpeedCorrected}, the corrected speed of germination as
#'   \mjseqn{\mathrm{time^{-1}}}.
#'
#' @references
#'
#' \insertAllCited{}
#'
#' @name GermSpeed
#'
#' @examples
#' x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
#' y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
#' int <- 1:length(x)
#'
#' # From partial germination counts
#' #----------------------------------------------------------------------------
#' GermSpeed(germ.counts = x, intervals = int)
#' GermSpeedAccumulated(germ.counts = x, intervals = int)
#' GermSpeedCorrected(germ.counts = x, intervals = int, total.seeds = 50,
#'                    method = "normal")
#' GermSpeedCorrected(germ.counts = x, intervals = int, total.seeds = 50,
#'                    method = "accumulated")
#'
#' # From partial germination counts (with percentages instead of counts)
#' #----------------------------------------------------------------------------
#' GermSpeed(germ.counts = x, intervals = int,
#'           percent = TRUE, total.seeds = 50)
#' GermSpeedAccumulated(germ.counts = x, intervals = int,
#'                      percent = TRUE, total.seeds = 50)
#'
#' # From cumulative germination counts
#' #----------------------------------------------------------------------------
#' GermSpeed(germ.counts = y, intervals = int, partial = FALSE)
#' GermSpeedAccumulated(germ.counts = y, intervals = int, partial = FALSE)
#' GermSpeedCorrected(germ.counts = y, intervals = int,
#'                    partial = FALSE, total.seeds = 50, method = "normal")
#' GermSpeedCorrected(germ.counts = y, intervals = int,
#'                    partial = FALSE, total.seeds = 50, method = "accumulated")
#'
#' # From cumulative germination counts (with percentages instead of counts)
#' #----------------------------------------------------------------------------
#' GermSpeed(germ.counts = y, intervals = int, partial = FALSE,
#'           percent = TRUE, total.seeds = 50)
#' GermSpeedAccumulated(germ.counts = y, intervals = int, partial = FALSE,
#'                      percent = TRUE, total.seeds = 50)

#' @rdname GermSpeed
#' @export
#GerminationIndexAOSA
GermSpeed <- function(germ.counts, intervals, partial = TRUE,
                      percent = FALSE, total.seeds = NULL) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts

  # Check if argument percent is of type logical with unit length
  if (!is.logical(percent) || length(percent) != 1) {
    stop("'percent' should be a logical vector of length 1.")
  }

  if (percent) {
    # Check if argument total.seeds is of type numeric with unit length
    if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
      stop("'total.seeds' should be a numeric vector of length 1.")
    }
  x <- (x / total.seeds) * 100
  }

  speed <- sum(x/intervals, na.rm = TRUE)
  return(speed)
}

#' @rdname GermSpeed
#' @export
GermSpeedAccumulated <- function(germ.counts, intervals, partial = TRUE,
                                 percent = FALSE, total.seeds = NULL) {
  # Check if argument germ.counts is of type numeric
  if (!is.numeric(germ.counts)) {
    stop("'germ.counts' should be a numeric vector.")
  }

  # Check if argument intervals is of type numeric
  if (!is.numeric(intervals)) {
    stop("'intervals' should be a numeric vector.")
  }

  # Check if intervals are uniform
  idiff <- diff(intervals)
  if (!all(abs(idiff - idiff[[1]]) < .Machine$double.eps ^ 0.5)) {
    warning("'intervals' are not uniform.")
  }

  # Check if germ.counts and intervals are of equal length
  if (length(germ.counts) != length(intervals)) {
    stop("'germ.counts' and 'intervals' lengths differ.")
  }

  # Check if argument partial is of type logical with unit length
  if (!is.logical(partial) || length(partial) != 1) {
    stop("'partial' should be a logical vector of length 1.")
  }

  # Convert cumulative to partial
  if (!partial) {
    germ.counts <- c(germ.counts[1], diff(germ.counts))
  }

  x <- germ.counts

  # Check if argument percent is of type logical with unit length
  if (!is.logical(percent) || length(percent) != 1) {
    stop("'percent' should be a logical vector of length 1.")
  }

  if (percent) {
    # Check if argument total.seeds is of type numeric with unit length
    if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
      stop("'total.seeds' should be a numeric vector of length 1.")
    }
    x <- (x / total.seeds) * 100
  }

  aspeed <- sum(cumsum(x)/intervals, na.rm = TRUE)
  return(aspeed)
}

#' @rdname GermSpeed
#' @export
GermSpeedCorrected <- function(germ.counts, intervals, partial = TRUE,
                               total.seeds,
                               method = c("normal", "accumulated")) {
  # Check if argument total.seeds is of type numeric with unit length
  if (!is.numeric(total.seeds) || length(total.seeds) != 1) {
    stop("'total.seeds' should be a numeric vector of length 1.")
  }

  # Check method
  method <- match.arg(method)

  if (method == "normal") {
    speed <- GermSpeed(germ.counts, intervals, partial,
                       total.seeds = total.seeds, percent = TRUE)
  }

  if (method == "accumulated") {
    speed <- GermSpeedAccumulated(germ.counts, intervals, partial,
                                  total.seeds = total.seeds, percent = TRUE)
  }

  gp <- GermPercent(germ.counts = germ.counts, total.seeds = total.seeds,
                    partial = partial)
  cspeed <- speed/gp

  return(cspeed)
}

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)


test_that("GermSpeed works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermSpeed(germ.counts = x, intervals = int),
               expected = 6.13892496392496, tolerance = 1e-12)

  # From partial germination counts (with percentages instead of counts)
  expect_equal(object = GermSpeed(germ.counts = x, intervals = int,
                                  percent = TRUE, total.seeds = 50),
               expected = 12.2778499278499, tolerance = 1e-12)
})

test_that("GermSpeed works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = GermSpeed(germ.counts = y, intervals = int,
                                  partial = FALSE),
               expected = 6.13892496392496, tolerance = 1e-12)

  # From partial germination counts (with percentages instead of counts)
  expect_equal(object = GermSpeed(germ.counts = y, intervals = int,
                                  partial = FALSE, percent = TRUE,
                                  total.seeds = 50),
               expected = 12.2778499278499, tolerance = 1e-12)
})

test_that("GermSpeedAccumulated works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermSpeedAccumulated(germ.counts = x, intervals = int),
               expected = 34.6156676656677, tolerance = 1e-12)

  # From partial germination counts (with percentages instead of counts)
  expect_equal(object = GermSpeedAccumulated(germ.counts = x, intervals = int,
                                             percent = TRUE, total.seeds = 50),
               expected = 69.2313353313353, tolerance = 1e-12)
})

test_that("GermSpeedAccumulated works; Cumulative germination counts", {

  # From partial germination counts
  expect_equal(object = GermSpeedAccumulated(germ.counts = y, intervals = int,
                                             partial = FALSE),
               expected = 34.6156676656677, tolerance = 1e-12)

  # From partial germination counts (with percentages instead of counts)
  expect_equal(object = GermSpeedAccumulated(germ.counts = y, intervals = int,
                                             partial = FALSE, percent = TRUE,
                                             total.seeds = 50),
               expected = 69.2313353313353, tolerance = 1e-12)
})

test_that("GermSpeedCorrected works; Partial germination counts", {

  # From partial germination counts
  expect_equal(object = GermSpeedCorrected(germ.counts = x, intervals = int,
                                           total.seeds = 50, method = "normal"),
               expected = 0.153473124098124, tolerance = 1e-12)
  expect_equal(object = GermSpeedCorrected(germ.counts = x, intervals = int,
                                           total.seeds = 50,
                                           method = "accumulated"),
               expected = 0.865391691641692, tolerance = 1e-12)

})

test_that("GermSpeedCorrected works; Cumulative germination counts", {

  # From cumulative germination counts
  expect_equal(object = GermSpeedCorrected(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           method = "normal"),
               expected = 0.153473124098124, tolerance = 1e-12)
  expect_equal(object = GermSpeedCorrected(germ.counts = y, intervals = int,
                                           partial = FALSE, total.seeds = 50,
                                           method = "accumulated"),
               expected = 0.865391691641692, tolerance = 1e-12)
})

test_that("Identical results with Cumulative and Partial germination counts", {

  expect_identical(object = GermSpeed(germ.counts = x, intervals = int),
                   expected = GermSpeed(germ.counts = y, intervals = int,
                                        partial = FALSE))
  expect_identical(object = GermSpeed(germ.counts = x, intervals = int,
                                      percent = TRUE, total.seeds = 50),
                   expected = GermSpeed(germ.counts = y, intervals = int,
                                        partial = FALSE, percent = TRUE,
                                        total.seeds = 50))

  expect_identical(object = GermSpeedAccumulated(germ.counts = x, intervals = int),
                   expected =  GermSpeedAccumulated(germ.counts = y, intervals = int,
                                                    partial = FALSE))
  expect_identical(object = GermSpeedAccumulated(germ.counts = x, intervals = int,
                                                 percent = TRUE, total.seeds = 50),
                   expected = GermSpeedAccumulated(germ.counts = y, intervals = int,
                                                   partial = FALSE, percent = TRUE,
                                                   total.seeds = 50))

  expect_identical(object = GermSpeedCorrected(germ.counts = x, intervals = int,
                                               total.seeds = 50, method = "normal"),
                   expected = GermSpeedCorrected(germ.counts = y, intervals = int,
                                                 partial = FALSE, total.seeds = 50,
                                                 method = "normal"))
  expect_identical(object = GermSpeedCorrected(germ.counts = x, intervals = int,
                                               total.seeds = 50,
                                               method = "accumulated"),
                   expected = GermSpeedCorrected(germ.counts = y, intervals = int,
                                                 partial = FALSE, total.seeds = 50,
                                                 method = "accumulated"))

})

---
title: "The `germinationmetrics` Package: A Brief Introduction"
author: "Aravind, J., Vimala Devi, S., Radhamani, J., Jacob, S. R., and Kalyani Srinivasan"
date: "`r Sys.Date()`"
classoption: table, twoside
header-includes:
- \usepackage{fancyhdr}
- \usepackage{wrapfig}
- \pagestyle{fancy}
- \fancyhead[LE,RO]{\slshape \rightmark}
- \fancyfoot[C]{\thepage}
- \usepackage{xcolor}
- \usepackage{hyperref}
- \hypersetup{colorlinks=true}
- \hypersetup{linktoc=all}
- \hypersetup{linkcolor=blue}
- \usepackage{pdflscape}
- \usepackage{booktabs}
- \newcommand{\blandscape}{\begin{landscape}}
- \newcommand{\elandscape}{\end{landscape}}
- \fancyhead[LO,RE]{The \texttt{germinationmetrics} Package{:} A Brief Introduction}
output:
  pdf_document:
    fig_caption: no
    toc: no
    keep_tex:  no
  html_document:
    df_print: paged
    toc: yes
csl: frontiers.csl
bibliography: REFERENCES.bib
link-citations: yes
vignette: |
  %\VignetteIndexEntry{Introduction}
  %\usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown_notangle}
---

```{r, echo=FALSE}
out_type <- knitr::opts_knit$get("rmarkdown.pandoc.to")

r = getOption("repos")
r["CRAN"] = "https://cran.rstudio.com/"
#r["CRAN"] = "https://cloud.r-project.org/"
#r["CRAN"] = "https://ftp.iitm.ac.in/cran/"
options(repos = r)

# Workaround for missing pandoc in CRAN OSX build machines
out_type <- ifelse(out_type == "", "latex", out_type)

# Workaround for missing pandoc in Solaris build machines
out_type <- ifelse(identical (out_type, vector(mode = "logical", length = 0)),
                   "latex", out_type)
```

```{r, results='asis', echo=FALSE}
switch(out_type,
    html = {cat("<p> ICAR-National Bureau of Plant Genetic Resources, New Delhi.</p>")},
    latex = cat("\\begin{center}
 ICAR-National Bureau of Plant Genetic Resources, New Delhi.

\\end{center}" )
)
```

\begin{center}
\vspace{6pt}
\hrule
\end{center}

```{r, echo = FALSE}
knitr::opts_chunk$set(
  comment = "",
  fig.cap = ""
)

```

\tableofcontents

\begin{wrapfigure}{r}{0.35\textwidth}
  \vspace{-20pt}
  \begin{center}
    \includegraphics[width=0.33\textwidth]{`r system.file("extdata", "germinationmetrics.png", package = "germinationmetrics")`}
  \end{center}
    \vspace{-20pt}
\end{wrapfigure}

<img src="https://raw.githubusercontent.com/aravind-j/germinationmetrics/master/inst/extdata/germinationmetrics.png" align="right" alt="logo" width="173" height = "200" style = "padding: 10px; border: none; float: right;">

## Overview

The package `germinationmetrics` is a collection of functions which implements various methods for describing the time-course of germination in terms of single-value germination indices as well as fitted curves.

The goal of this vignette is to introduce the users to these functions and get started in describing sequentially recorded germination count data. This document assumes a basic knowledge of `R` programming language.

## Installation
The package can be installed using the following functions:

```{r, eval=FALSE}
# Install from CRAN
install.packages('germinationmetrics', dependencies=TRUE)

# Install development version from Github
devtools::install_github("aravind-j/germinationmetrics")

```

Then the package can be loaded using the function
```{r}
library(germinationmetrics)
```

\clearpage

## Version History
```{r, results='asis', echo=FALSE}

# Fetch release version
rver <- ifelse(test = gsub("(.\\.)(\\d+)(\\..)", "", getNamespaceVersion("germinationmetrics")) == "",
               yes = getNamespaceVersion("germinationmetrics"),
               no = ifelse("germinationmetrics" %in% available.packages()[, 1],
                           as.vector(available.packages()["germinationmetrics",]["Version"]),
                           getNamespaceVersion("germinationmetrics")))

```

The current version of the package is `r rver`. The previous versions are as follows.

**Table 1.** Version history of `germinationmetrics` `R` package.

```{r, echo=FALSE, message=FALSE}
if (requireNamespace("curl", quietly = TRUE) & 
    requireNamespace("httr", quietly = TRUE) & 
    requireNamespace("XML", quietly = TRUE)) {
  
  pkg <- "germinationmetrics"
  link <- paste0("https://cran.r-project.org/src/contrib/Archive/", pkg, "/")
  
  url_exists <- function(url) {
    h <- curl::new_handle(nobody = TRUE)
    tryCatch({
      fetch <- curl::curl_fetch_memory(url, handle = h)
      status <- fetch$status_code
      status == 200 # Strict
      # status >= 200 && status < 400 # Lenient
    }, error = function(e) {
      FALSE
    })
  }
  
  if (curl::has_internet()) {

    if (url_exists(link)) {
      # cafile <- system.file("CurlSSL", "cacert.pem", package = "RCurl")
      # page <- httr::GET(link, httr::config(cainfo = cafile))
      page <- httr::GET(link)
      page <- httr::content(page, as = 'text')
      # page <- RCurl::getURL(link)
      
      VerHistory <- XML::readHTMLTable(page)[[1]][,2:3]
      colnames(VerHistory) <- c("Version", "Date")
      VerHistory <- VerHistory[VerHistory$Version != "Parent Directory",]
      VerHistory <- VerHistory[!is.na(VerHistory$Version), ]
      VerHistory$Date <- as.Date(VerHistory$Date)
      VerHistory$Version <- gsub("germinationmetrics_", "", VerHistory$Version)
      VerHistory$Version <- gsub(".tar.gz", "", VerHistory$Version)
      
      VerHistory <- VerHistory[order(VerHistory$Date), c("Version", "Date")]
      rownames(VerHistory) <- NULL
      
      knitr::kable(VerHistory)
      
    } else {
      print("Access to CRAN page for 'germinationmetrics' is required to generate this table.'")
    }
  } else {
    "Internet connection is required to generate this table."
  }
  
} else {
  print("Packages 'curl', 'httr' and 'XML' are required to generate this table.")
}

```

To know detailed history of changes use `news(package='germinationmetrics')`.


## Germination count data

Typically in a germination test, the germination count data of a fixed number of seeds is recorded at regular intervals for a definite period of time or until all the seeds have germinated. These germination count data can be either partial or cumulative (Table 2).

**Table 2 :** A typical germination count data.

```{r, echo = FALSE}
data <- data.frame(intervals = 1:14,
                   counts = c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0))
data$cumulative.counts = cumsum(data$counts)

knitr::kable(data)

```

The time-course of germination can be plotted as follows.

```{r, eval = TRUE}
data <- data.frame(intervals = 1:14,
                   counts = c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0))

# Partial germination counts
x <- data$counts
# Cumulative germination counts
y <- cumsum(x)
# Time intervals of observations
int <- data$intervals

plot(int, cumsum(x))
```

## Single-value germination indices

The details about the single-value germination indices implemented in `germinationmetrics` are described in Table 3.

\newpage
\blandscape

\rowcolors{2}{gray!25}{white}
\renewcommand{\arraystretch}{1.3}

**Table 3 :** Single-value germination indices implemented in `germinationmetrics`.

\footnotesize

```{r, echo = FALSE, eval = TRUE}
load("Indices.rda")

indices$`Function` <- paste("`", indices$`Function`, "`", sep = "")

# indices$Details <- gsub("\\r\\n\\r\\n", "$$\\\\, $$", indices$Details)
# indices$Details <- gsub("$$\\, $$$$", "$$", indices$Details, fixed = TRUE)
# indices$Details <- paste(indices$Details, "$$\\\\, $$", sep = "")

indices$Details <- gsub("\\r\\n", "\\\\\n", indices$Details)

#indices$Details <- gsub("\\r\\n\\r\\n", "##", indices$Details)
#indices$Details <- gsub("##", "\\\n", indices$Details)

indices$Reference <- gsub("\\[|\\]", "", indices$Reference)
#indices <- indices[-27,]

```

```{r, echo = FALSE, eval = TRUE}
# if (requireNamespace("pander", quietly = TRUE) & Sys.which('pandoc') != "" & Sys.which('pandoc-citeproc') != ""){
if (requireNamespace("pander", quietly = TRUE) & rmarkdown::pandoc_available()){
  pander::pander(indices, style='grid',
               split.cell = c(15, 18, 50, 10, 12, 8),
               justify = c('left'), split.table = Inf, keep.line.breaks = TRUE,
               use.hyphening = FALSE, missing = "")
} else {
  print("Package 'pander' and pandoc are required to generate this table.")
}

```

\elandscape

\newpage

#### Examples

```{r, echo = FALSE}
# Function saved in functions.R file
getCode <- function(myFunction, myPackage) {
  out <- example(myFunction, myPackage, ask = FALSE, character.only = TRUE,
          prompt.prefix = "", give.lines = TRUE)
  maxn <- grep(pattern = "### ** Examples", x = out, fixed = TRUE)
  out <- out[-(1:maxn)]
  
  out <- out[!(out %in% c("## No test: ", "## End(No test)"))]
  
  out <- gsub("^##D ", "", out)
  
  return(out)
}
library(germinationmetrics)
require(germinationmetrics)
```

#### `GermPercent()`
```{r, runCode-GermPercent, code = getCode("GermPercent", "germinationmetrics")}
```

#### `FirstGermTime(), LastGermTime(), PeakGermTime(), TimeSpreadGerm()`
```{r, runCode-FirstGermTime, code = getCode("FirstGermTime", "germinationmetrics")}
```

#### `t50()`
```{r, runCode-t50, code = getCode("t50", "germinationmetrics")}
```

#### `MeanGermTime(), VarGermTime(), SEGermTime(), CVGermTime()`
```{r, runCode-MeanGermTime, code = getCode("MeanGermTime", "germinationmetrics")}
```

#### `MeanGermRate(), CVG(), VarGermRate(), SEGermRate(), GermRateRecip()`
```{r, runCode-MeanGermRate, code = getCode("MeanGermRate", "germinationmetrics")}
```

#### `GermSpeed(), GermSpeedAccumulated(), GermSpeedCorrected() `
```{r, runCode-GermSpeed, code = getCode("GermSpeed", "germinationmetrics")}
```

#### `WeightGermPercent()`
```{r, runCode-WeightGermPercent, code = getCode("WeightGermPercent", "germinationmetrics")}
```

#### `MeanGermPercent(), MeanGermNumber()`
```{r, runCode-MeanGermPercent, code = getCode("MeanGermPercent", "germinationmetrics")}
```

#### `TimsonsIndex(), GermRateGeorge() `
```{r, runCode-TimsonsIndex, code = getCode("TimsonsIndex", "germinationmetrics")}
```

#### `GermIndex() `
```{r, runCode-GermIndex, code = getCode("GermIndex", "germinationmetrics")}
```

#### `EmergenceRateIndex() `
```{r, runCode-EmergenceRateIndex, code = getCode("EmergenceRateIndex", "germinationmetrics")}
```

#### `PeakValue(), GermValue()`
```{r, runCode-PeakValue, code = getCode("PeakValue", "germinationmetrics")}
```

#### `CUGerm()`
```{r, runCode-CUGerm, code = getCode("CUGerm", "germinationmetrics")}
```

#### `GermSynchrony(), GermUncertainty()`
```{r, runCode-GermSynchrony, code = getCode("GermSynchrony", "germinationmetrics")}
```

## Non-linear regression analysis

Several mathematical functions have been used to fit the cumulative germination count data and describe the germination process  by non-linear regression analysis. They include functions such as Richard's, Weibull, logistic, log-logistic, gaussian, four-parameter hill function etc. Currently `germinationmetrics` implements the four-parameter hill function to fit the count data and computed various associated metrics.

### Four-parameter hill function
The four-parameter hill function defined as follows [@el-kassaby_seed_2008].

$$
f(x) = y = y_0 + \frac{ax^b}{x^b+c^b}
$$
Where, $y$ is the cumulative germination percentage at time $x$, $y_{0}$ is the intercept on the y axis, $a$ is the asymptote, $b$ is a mathematical parameter controlling the shape and steepness of the germination curve and $c$ is the "half-maximal activation level".

this function can also be be reparameterized by substituting $b$ with $e^{\beta}$ to constraint $b$ to positive values only.

$$
y = y_{0}+\frac{ax^{e^{\beta}}}{c^{e^{\beta}}+x^{e^{\beta}}}
$$
Where, $b = e^{\beta}$ and $\beta = \log_{e}(b)$.

The details of various parameters that are computed from this function are given in Table 4.

**Table 4** Germination parameters estimated from the four-parameter hill function.

\rowcolors{2}{gray!25}{white}
\renewcommand{\arraystretch}{1.3}
```{r, echo = FALSE, eval = TRUE}
load("Indices.rda")
fphf$Details <- gsub("\\r\\n", "\\\\\n", fphf$Details)
if (requireNamespace("pander", quietly = TRUE) & rmarkdown::pandoc_available()){
# if (requireNamespace("pander", quietly = TRUE) & Sys.which('pandoc') != "" & Sys.which('pandoc-citeproc') != ""){
  pander::pander(fphf, style='grid',
               split.cell = c(15, 50, 10, 10),
               justify = c('left'), split.table = Inf, keep.line.breaks = TRUE,
               use.hyphening = FALSE, missing = "")
} else {
  print("Package 'pander' and pandoc are required to generate this table.")
}
```

#### Examples

```{r, echo = FALSE}
# Function saved in functions.R file
getCode <- function(myFunction, myPackage) {
  out <- example(myFunction, myPackage, ask = FALSE, character.only = TRUE,
          prompt.prefix = "", give.lines = TRUE)
  maxn <- grep(pattern = "### ** Examples", x = out, fixed = TRUE)
  out <- out[-(1:maxn)]
  
  out <- out[!(out %in% c("## No test: ", "## End(No test)"))]
  
  out <- gsub("^##D ", "", out)
  
  return(out)
}
library(germinationmetrics)
require(germinationmetrics)
```

#### `FourPHFfit()`
```{r, runCode-FourPHFfit, code = getCode("FourPHFfit", "germinationmetrics")}
```

```{r, runCode, fig.width = 6, fig.height = 4, code = getCode("plot.FourPHFfit", "germinationmetrics"), eval =FALSE, echo = FALSE}
```

```{r, fig.width = 6, fig.height = 4, code = getCode("plot.FourPHFfit", "germinationmetrics")}
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)
total.seeds = 50

# From partial germination counts
#----------------------------------------------------------------------------
fit1 <- FourPHFfit(germ.counts = x, intervals = int,
                   total.seeds = 50, tmax = 20)

# From cumulative germination counts
#----------------------------------------------------------------------------
fit2 <- FourPHFfit(germ.counts = y, intervals = int,
                   total.seeds = 50, tmax = 20, partial = FALSE)

# Default plots
plot(fit1)
plot(fit2)

# No labels
plot(fit1, plotlabels = FALSE)
plot(fit2, plotlabels = FALSE)

# Only the FPHF curve
plot(fit1, rog = FALSE, t50.total = FALSE, t50.germ = FALSE,
     tmgr = FALSE, mgt = FALSE, uniformity = FALSE)
plot(fit2, rog = FALSE, t50.total = FALSE, t50.germ = FALSE,
     tmgr = FALSE, mgt = FALSE, uniformity = FALSE)

# Without y axis limits adjustment
plot(fit1, limits = FALSE)
plot(fit2, limits = FALSE)
```

## Wrapper functions

Wrapper functions `germination.indices()` and `FourPHFfit.bulk()` are available in the package for computing results for multiple samples in batch from a data frame of germination counts recorded at specific time intervals.

#### `germination.indices()`
This wrapper function can be used to compute several germination indices simultaneously for multiple samples in batch.

```{r, runCode-germination.indices, code = getCode("germination.indices", "germinationmetrics")}
```

#### `FourPHFfit.bulk()`
This wrapper function can be used to fit the four-parameter hill function for multiple samples in batch.

```{r, runCode-FourPHFfit.bulk, code = getCode("FourPHFfit.bulk", "germinationmetrics")}
```

Multiple fitted curves generated in batch can also be plotted.

```{r, runCode-plot.FourPHFfit.bulk, fig.width = 6, fig.height = 4, code = getCode("plot.FourPHFfit.bulk", "germinationmetrics")}
```

## Citing `germinationmetrics`

```{r, echo = FALSE, collapse = TRUE}
detach("package:germinationmetrics", unload=TRUE)
suppressPackageStartupMessages(library(germinationmetrics))
cit <- citation("germinationmetrics")
# yr <- format(Sys.Date(), "%Y")
# cit[1]$year <- yr
# oc <- class(cit)
# 
# cit <- unclass(cit)
# attr(cit[[1]],"textVersion") <- gsub("\\(\\)",
#                                      paste("\\(", yr, "\\)", sep = ""),
#                                      attr(cit[[1]],"textVersion"))
# class(cit) <- oc
if (is.null(cit[1]$year)) {
  cit[1]$year <- format(Sys.Date(), "%Y")
  cit$textVersion <- gsub("()", 
                          paste("(", format(Sys.Date(), "%Y"), ")", sep =""), 
                          cit$textVersion, fixed = TRUE)
}
cit
```

## Session Info

```{r}
sessionInfo()
```

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/germination.indices.R
\name{germination.indices}
\alias{germination.indices}
\title{Germination Indices}
\usage{
germination.indices(
  data,
  total.seeds.col,
  counts.intervals.cols,
  intervals,
  partial = TRUE,
  PeakGermPercent = TRUE,
  FirstGermTime = TRUE,
  LastGermTime = TRUE,
  PeakGermTime = TRUE,
  TimeSpreadGerm = TRUE,
  t50 = TRUE,
  MeanGermTime = TRUE,
  VarGermTime = TRUE,
  SEGermTime = TRUE,
  CVGermTime = TRUE,
  MeanGermRate = TRUE,
  CVG = TRUE,
  VarGermRate = TRUE,
  SEGermRate = TRUE,
  GermRateRecip = TRUE,
  GermSpeed = TRUE,
  GermSpeedAccumulated = TRUE,
  GermSpeedCorrected = TRUE,
  WeightGermPercent = TRUE,
  MeanGermPercent = TRUE,
  MeanGermNumber = TRUE,
  TimsonsIndex = TRUE,
  GermRateGeorge = TRUE,
  max.int,
  GermIndex = TRUE,
  EmergenceRateIndex = TRUE,
  PeakValue = TRUE,
  GermValue = TRUE,
  gv.k = 10,
  CUGerm = TRUE,
  GermSynchrony = TRUE,
  GermUncertainty = TRUE
)
}
\arguments{
\item{data}{A data frame with the germination count data. It should possess
columns with \itemize{ \item Partial or cumulative germination counts per
time interval (to be indicated by the argument \code{counts.intervals.cols}
and \item Total number of seeds tested (to be indicated by the argument
\code{total.seeds.col}). }}

\item{total.seeds.col}{The name of the column in \code{data} with the total
number of seeds tested.}

\item{counts.intervals.cols}{The names of columns in \code{data} with the
germination counts (partial or cumulative, as indicated by the argument
\code{partial}) per time interval (indicated by the argument
\code{intervals}).}

\item{intervals}{The time intervals at which germination counts (in the
columns specified in argument \code{counts.intervals.cols})  were recorded.}

\item{partial}{logical. If \code{TRUE}, germination counts in
\code{counts.intervals.cols} is considered as partial and if \code{FALSE},
it is considered as cumulative. Default is \code{TRUE}.}

\item{PeakGermPercent}{logical. If \code{TRUE}, then the Peak germination
percentage
\insertCite{vallance_studies_1950,roh_maturity_2004}{germinationmetrics} is
computed. Default is \code{TRUE}.}

\item{FirstGermTime}{logical. If \code{TRUE}, the Time of first germination
or Germination time lag (\mjseqn{t_{0}})
\insertCite{edwards_temperature_1932,czabator_germination_1962,goloff_germination_1975,labouriau_germinacao_1983,ranal_effects_1999,quintanilla_effect_2000}{germinationmetrics}
 is computed. Default is \code{TRUE}.}

\item{LastGermTime}{logical. If \code{TRUE}, the Time of last germination
(\mjseqn{t_{g}})
\insertCite{edwards_temperature_1932,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
 is computed. Default is \code{TRUE}.}

\item{PeakGermTime}{logical. If \code{TRUE}, the Peak time of germination or
Modal time of germination \insertCite{ranal_how_2006}{germinationmetrics}
is computed. Default is \code{TRUE}.}

\item{TimeSpreadGerm}{logical. If \code{TRUE}, the Time spread of germination
\insertCite{al-mudaris_notes_1998,kader_comparison_2005}{germinationmetrics}
 or Germination distribution
\insertCite{schrader_seed_2000}{germinationmetrics}. is computed. Default
is \code{TRUE}.}

\item{t50}{logical. If \code{TRUE}, the Median germination time
(\mjseqn{t_{50}})
\insertCite{coolbear_effect_1984,farooq_thermal_2005}{germinationmetrics}
is computed. Default is \code{TRUE}.}

\item{MeanGermTime}{logical. If \code{TRUE}, the Mean germination time
(\mjseqn{\overline{T}}) or Mean length of incubation time
\insertCite{edmond_effects_1958,czabator_germination_1962,ellis_improved_1980,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
 or Sprouting index (\mjseqn{SI})
\insertCite{smith_germinating_1964}{germinationmetrics} or Germination
Resistance (\mjseqn{GR})
\insertCite{gordon_observations_1969,gordon_germination_1971;textual}{germinationmetrics}
 is computed. Default is \code{TRUE}.}

\item{VarGermTime}{logical. If \code{TRUE}, the Variance of germination time
(\mjseqn{s_{T}^{2}})
\insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}
is computed. Default is \code{TRUE}.}

\item{SEGermTime}{logical. If \code{TRUE}, the Standard error of germination
time (\mjseqn{s_{\overline{T}}})
\insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.
is computed. Default is \code{TRUE}.}

\item{CVGermTime}{logical. If \code{TRUE}, the Coefficient of variation of
the germination time (\mjseqn{CV_{T}})
\insertCite{ranal_how_2006}{germinationmetrics} is computed. Default is
\code{TRUE}.}

\item{MeanGermRate}{logical. If \code{TRUE}, the Mean germination rate
(\mjseqn{\overline{V}})
\insertCite{labouriau_germination_1976,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.
 is computed. Default is \code{TRUE}.}

\item{CVG}{logical. If \code{TRUE}, the Coefficient of velocity/rate of
germination or Kotowski's coefficient of velocity (\mjseqn{CVG})
\insertCite{kotowski_temperature_1926,nichols_two_1968,labouriau_uma_1983,scott_review_1984,bewley_seeds_1994}{germinationmetrics}.
 is computed. Default is \code{TRUE}.}

\item{VarGermRate}{logical. If \code{TRUE}, the Variance of germination rate
(\mjseqn{s_{V}^{2}})
\insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics} is
computed. Default is \code{TRUE}.}

\item{SEGermRate}{logical. If \code{TRUE}, the Standard error of germination
rate (\mjseqn{s_{\overline{V}}})
\insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics} is
computed. Default is \code{TRUE}.}

\item{GermRateRecip}{logical. If \code{TRUE}, the Germination rate as
reciprocal of median time (\mjseqn{v_{50}})
\insertCite{went_experimental_1957,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}
 is computed.}

\item{GermSpeed}{logical. If \code{TRUE}, the Speed of germination or
Germination rate index or Index of velocity of germination or Germination
index or Emergence rate index
\insertCite{throneberry_relation_1955,maguire_speed_1962,allan_seedling_1962,kendrick_photocontrol_1969,bouton_germination_1976,aosa_seed_1983,khandakar_jute_1983,bradbeer_seed_1988,wardle_allelopathic_1991}{germinationmetrics}
 considering both counts and percentages is computed.}

\item{GermSpeedAccumulated}{logical. If \code{TRUE}, the Speed of accumulated
germination
\insertCite{bradbeer_seed_1988,wardle_allelopathic_1991,haugland_experiments_1996,de_santana_alise_2004}{germinationmetrics}
 considering both counts and percentages is computed.}

\item{GermSpeedCorrected}{logical. If \code{TRUE}, the Corrected speed of
germination or Corrected germination rate index and the Corrected speed of
accumulated germination
\insertCite{evetts_germination_1972}{germinationmetrics} is computed.}

\item{WeightGermPercent}{logical. If \code{TRUE}, the Weighted germination
percentage \mjseqn{WGP} or Weighted germination index \mjseqn{WGI}
\insertCite{reddy_effect_1978,reddy_effect_1985}{germinationmetrics} is
computed.}

\item{MeanGermPercent}{logical. If \code{TRUE}, the Mean/average germination
percentage per unit time (\mjseqn{\overline{GP}})
\insertCite{czabator_germination_1962}{germinationmetrics} is computed.}

\item{MeanGermNumber}{logical. If \code{TRUE}, the Number of seeds germinated
per unit time (\mjseqn{\overline{N}})
\insertCite{khamassi_optimal_2013}{germinationmetrics} is computed.}

\item{TimsonsIndex}{logical. If \code{TRUE}, the Timson's index or Timson's
germination velocity index
\insertCite{grose_laboratory_1958,timson_new_1965,brown_representing_1988,baskin_seeds_1998}{germinationmetrics}
 and it's modifications by Labouriau
\insertCite{ranal_how_2006}{germinationmetrics} and
\insertCite{khan_effect_1984}{germinationmetrics} is computed.}

\item{GermRateGeorge}{logical. If \code{TRUE}, the George's index
\insertCite{george_influence_1961;textual}{germinationmetrics} is computed.}

\item{max.int}{The maximum interval value up to which Timson's index/George's
germination rate is to be computed.}

\item{GermIndex}{logical. If \code{TRUE}, the Germination index according to
\insertCite{melville_seed_1980;textual}{germinationmetrics}
and its modification by
\insertCite{de_santana_alise_2004;textual}{germinationmetrics}
\insertCite{ranal_how_2006}{germinationmetrics} are computed.}

\item{EmergenceRateIndex}{logical. If \code{TRUE}, the Emergence rate index
or Germination rate index are computed according to \itemize{ \item
\insertCite{shmueli_emergence_1971;textual}{germinationmetrics}
and it's modification by
\insertCite{de_santana_alise_2004;textual}{germinationmetrics}
\insertCite{ranal_how_2006}{germinationmetrics}; \item
\insertCite{bilbro_soil_1982;textual}{germinationmetrics}; and
\item
\insertCite{fakorede_relation_1980;textual}{germinationmetrics},
\insertCite{fakorede_variability_1981;textual}{germinationmetrics},
 and
\insertCite{fakorede_heterotic_1983;textual}{germinationmetrics}.}}

\item{PeakValue}{logical. If \code{TRUE}, the Peak value (\mjseqn{PV}) or
Emergence Energy (\mjseqn{EE})
\insertCite{czabator_germination_1962,bonner_ideal_1967}{germinationmetrics}
 is computed.}

\item{GermValue}{logical. If \code{TRUE}, the Germination value
\insertCite{czabator_germination_1962,djavanshir_germination_1976}{germinationmetrics}
 and its modification considering duration from start of test instead of
from onset of germination
\insertCite{brown_representing_1988}{germinationmetrics} is computed.}

\item{gv.k}{The k Constant for computation of germination value. Default is
10.}

\item{CUGerm}{logical. If \code{TRUE}, the Coefficient of uniformity of
germination (CUG)
\insertCite{heydecker_seed_1972,bewley_seeds_1994}{germinationmetrics} is
computed.}

\item{GermSynchrony}{logical. If \code{TRUE}, the Synchrony of germination
(\mjseqn{Z} index)
\insertCite{primack_longevity_1985,ranal_how_2006}{germinationmetrics} is
computed.}

\item{GermUncertainty}{logical. If \code{TRUE}, the Synchronization index
(\mjseqn{\overline{E}}) or Uncertainty of the germination process
(\mjseqn{U}) or Informational entropy (\mjseqn{H})
\insertCite{shannon_mathematical_1948,labouriau_germination_1976,labouriau_uma_1983}{germinationmetrics}
 is computed.}
}
\value{
A data frame with the original data along with the various
  single-value germination indices specified in the function arguments.
}
\description{
Compute germination indices from a data frame of germination counts recorded
at specific time intervals for multiple samples in batch. \loadmathjax
}
\examples{

\donttest{
data(gcdata)

counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
                          "Day06", "Day07", "Day08", "Day09", "Day10",
                          "Day11", "Day12", "Day13", "Day14")
germination.indices(gcdata, total.seeds.col = "Total Seeds",
                    counts.intervals.cols = counts.per.intervals,
                    intervals = 1:14, partial = TRUE, max.int = 5)
}

}
\references{
\insertAllCited{}
}
\seealso{
This function is a wrapper around the different functions for
  computation of single-value germination indices in
  \href{https://cran.r-project.org/package=germinationmetrics}{germinationmetrics}
   (\code{\link[germinationmetrics]{PeakGermPercent}},
  \code{\link[germinationmetrics]{FirstGermTime}},
  \code{\link[germinationmetrics:FirstGermTime]{LastGermTime}},
  \code{\link[germinationmetrics:FirstGermTime]{PeakGermTime}},
  \code{\link[germinationmetrics:FirstGermTime]{TimeSpreadGerm}},
  \code{\link[germinationmetrics]{t50}},
  \code{\link[germinationmetrics]{MeanGermTime}},
  \code{\link[germinationmetrics:MeanGermTime]{VarGermTime}},
  \code{\link[germinationmetrics:MeanGermTime]{SEGermTime}},
  \code{\link[germinationmetrics:MeanGermTime]{CVGermTime}},
  \code{\link[germinationmetrics]{MeanGermRate}},
  \code{\link[germinationmetrics:MeanGermRate]{CVG}},
  \code{\link[germinationmetrics:MeanGermRate]{VarGermRate}},
  \code{\link[germinationmetrics:MeanGermRate]{SEGermRate}},
  \code{\link[germinationmetrics:MeanGermRate]{GermRateRecip}},
  \code{\link[germinationmetrics]{GermSpeed}},
  \code{\link[germinationmetrics:GermSpeed]{GermSpeedAccumulated}},
  \code{\link[germinationmetrics:GermSpeed]{GermSpeedCorrected}},
  \code{\link[germinationmetrics]{WeightGermPercent}},
  \code{\link[germinationmetrics]{MeanGermPercent}},
  \code{\link[germinationmetrics:GermSpeed]{MeanGermPercent}},
  \code{\link[germinationmetrics]{TimsonsIndex}},
  \code{\link[germinationmetrics:TimsonsIndex]{GermRateGeorge}},
  \code{\link[germinationmetrics]{GermIndex}},
  \code{\link[germinationmetrics]{EmergenceRateIndex}},
  \code{\link[germinationmetrics]{GermValue}},
  \code{\link[germinationmetrics:GermValue]{PeakValue}},
  \code{\link[germinationmetrics]{CUGerm}},
  \code{\link[germinationmetrics]{GermSynchrony}},
  \code{\link[germinationmetrics:GermSynchrony]{GermUncertainty}})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FourPHFfit.bulk.R
\name{FourPHFfit.bulk}
\alias{FourPHFfit.bulk}
\title{Fit four-parameter hill function for multiple samples}
\usage{
FourPHFfit.bulk(
  data,
  total.seeds.col,
  counts.intervals.cols,
  intervals,
  partial = TRUE,
  fix.y0 = TRUE,
  fix.a = TRUE,
  tmax,
  xp = c(10, 60),
  umin = 10,
  umax = 90,
  tries = 3
)
}
\arguments{
\item{data}{A data frame with the germination count data. It should possess
columns with \itemize{ \item Partial or cumulative germination counts per
time interval (to be indicated by the argument \code{counts.intervals.cols}
and \item Total number of seeds tested (to be indicated by the argument
\code{total.seeds.col}). }}

\item{total.seeds.col}{The name of the column in \code{data} with the total
number of seeds tested.}

\item{counts.intervals.cols}{The names of columns in \code{data} with the
germination counts (partial or cumulative, as indicated by the argument
\code{partial}) per time interval (indicated by the argument
\code{intervals}).}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{fix.y0}{Force the intercept of the y axis through 0.}

\item{fix.a}{Fix a as the actual maximum germination percentage at the end of
the experiment.}

\item{tmax}{The time up to which AUC is to be computed.}

\item{xp}{Germination percentage value(s) for which the corresponding time is
to be computed as a numeric vector. Default is \code{c(10, 60)}.}

\item{umin}{The minimum germination percentage value for computing
uniformity. Default is \code{10}. See \strong{\code{Details}}.}

\item{umax}{The maximum germination percentage value for computing
uniformity. Default is \code{90}. See \strong{\code{Details}}.}

\item{tries}{The number of tries to be attempted to fit the curve. Default is
\code{3}.}
}
\value{
A data frame with the original data along with the various parameters
  of the respective fitted four-parameter hill function.
}
\description{
Fit four-parameter hill function from a data frame of germination counts
recorded at specific time intervals for multiple samples in batch.
}
\examples{

\donttest{
data(gcdata)

counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
                          "Day06", "Day07", "Day08", "Day09", "Day10",
                          "Day11", "Day12", "Day13", "Day14")

FourPHFfit.bulk(gcdata, total.seeds.col = "Total Seeds",
                    counts.intervals.cols = counts.per.intervals,
                    intervals = 1:14, partial = TRUE,
                    fix.y0 = TRUE, fix.a = TRUE, xp = c(10, 60),
                    tmax = 20, tries = 3, umax = 90, umin = 10)
}

}
\references{
\insertRef{el-kassaby_seed_2008}{germinationmetrics}
}
\seealso{
This function is a wrapper around the
  \code{\link[germinationmetrics]{FourPHFfit}} function for fitting
  four-parameter hill curve.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcdata.R
\docType{data}
\name{gcdata}
\alias{gcdata}
\title{Germination count data}
\format{
A data frame with 17 columns: \describe{ \item{Genotype}{The
  genotype.} \item{Rep}{Replication.} \item{Day01}{Number of seeds that
  germinated (germination counts) on Day 1.} \item{Day02}{Number of seeds
  that germinated (germination counts) on Day 2.} \item{Day03}{Number of
  seeds that germinated (germination counts) on Day 3.} \item{Day04}{Number
  of seeds that germinated (germination counts) on Day 4.}
  \item{Day05}{Number of seeds that germinated (germination counts) on Day
  5.} \item{Day06}{Number of seeds that germinated (germination counts) on
  Day 6.} \item{Day07}{Number of seeds that germinated (germination counts)
  on Day 7.} \item{Day08}{Number of seeds that germinated (germination
  counts) on Day 8.} \item{Day09}{Number of seeds that germinated
  (germination counts) on Day 9.} \item{Day10}{Number of seeds that
  germinated (germination counts) on Day 10.} \item{Day11}{Number of seeds
  that germinated (germination counts) on Day 11.} \item{Day12}{Number of
  seeds that germinated (germination counts) on Day 12.} \item{Day13}{Number
  of seeds that germinated (germination counts) on Day 13.}
  \item{Day14}{Number of seeds that germinated (germination counts) on Day
  14.} \item{Total Seeds}{Total number of seeds tested.} }
}
\usage{
gcdata
}
\description{
An example germination count dataset. It includes germination count data over
14 days from five genotypes (G1 to G5) in three replications
}
\examples{

\donttest{
data(gcdata)
library(ggplot2)
library(reshape2)

# Plot partial germination counts over time
#----------------------------------------------------------------------------
# Convert wide-from to long-form
gcdatamelt <- melt(gcdata[, !names(gcdata) \%in\% c("Total Seeds")],
                   id.vars = c("Genotype", "Rep"))

ggplot(gcdatamelt, aes(x = variable, y = value,
                       group = interaction(Genotype, Rep),
                       colour = Genotype)) +
  geom_point(alpha = 0.7) +
  geom_line(alpha = 0.7) +
  ylab("Germination count (Partial)") +
  xlab("Intervals") +
  theme_bw()

# Plot partial germination counts over time
#----------------------------------------------------------------------------

# Convert wide-from to long-form
# Compute cumulative germination counts
gcdata2 <- gcdata
gcdata2[, !names(gcdata2) \%in\% c("Genotype", "Rep", "Total Seeds")] <-
  t(apply(gcdata2[, !names(gcdata2) \%in\% c("Genotype", "Rep", "Total Seeds")], 1, cumsum))


gcdatamelt2 <- melt(gcdata2[, !names(gcdata2) \%in\% c("Total Seeds")],
                    id.vars = c("Genotype", "Rep"))

ggplot(gcdatamelt2, aes(x = variable, y = value,
                        group = interaction(Genotype, Rep),
                        colour = Genotype)) +
  geom_point(alpha = 0.7) +
  geom_line(alpha = 0.7) +
  ylab("Germination count (Cumulative)") +
  xlab("Intervals") +
  theme_bw()

# Compute germination indices
#----------------------------------------------------------------------------

counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
                          "Day06", "Day07", "Day08", "Day09", "Day10",
                          "Day11", "Day12", "Day13", "Day14")
germination.indices(gcdata, total.seeds.col = "Total Seeds",
                    counts.intervals.cols = counts.per.intervals,
                    intervals = 1:14, partial = TRUE, max.int = 5)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GermPercent.R
\name{GermPercent}
\alias{GermPercent}
\alias{PeakGermPercent}
\title{Germination percentage}
\usage{
GermPercent(germinated.seeds, germ.counts, total.seeds, partial = TRUE)

PeakGermPercent(germ.counts, intervals, total.seeds, partial = TRUE)
}
\arguments{
\item{germinated.seeds}{Number of germinated seeds}

\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{total.seeds}{Total number of seeds.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{intervals}{The time intervals.}
}
\value{
The germination percentage (\%) value.
}
\description{
Compute the following metrics: \describe{ \item{\code{GermPercent}}{
Germination percentage or Final germination percentage or Germinability
\insertCite{ista_chapter_2015}{germinationmetrics}.}
\item{\code{PeakGermPercent}}{Peak germination percentage
\insertCite{vallance_studies_1950,roh_maturity_2004}{germinationmetrics}.} }
}
\details{
Germination percentage is computed as follows
\insertCite{ista_chapter_2015}{germinationmetrics}. \loadmathjax

\mjsdeqn{GP = \frac{N_{g}}{N_{t}} \times 100}

Where, \mjseqn{N_{g}} is the number of germinated seeds and \mjseqn{N_{t}} is
the total number of seeds.

The value of \mjseqn{N_{g}} can be either specified using the argument
\code{germinated.seeds} or is computed from the \code{germ.counts} argument.

Peak germination percentage is computed as follows
\insertCite{vallance_studies_1950,roh_maturity_2004}{germinationmetrics}.

\mjsdeqn{PGP = \frac{N_{max}}{N_{t}} \times 100}

Where, \mjseqn{N_{max}} is the maximum number of seeds germinated per
interval.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
z <- c(0, 0, 0, 0, 11, 11, 9, 7, 1, 0, 1, 0, 0, 0)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
GermPercent(germ.counts = x, total.seeds = 50)
PeakGermPercent(germ.counts = x, intervals = int, total.seeds = 50)
# For multiple peak germination times
PeakGermPercent(germ.counts = z, intervals = int, total.seeds = 50)

# From cumulative germination counts
#----------------------------------------------------------------------------
GermPercent(germ.counts = y, total.seeds = 50, partial = FALSE)
PeakGermPercent(germ.counts = y, intervals = int, total.seeds = 50,
                partial = FALSE)
# For multiple peak germination times
PeakGermPercent(germ.counts = cumsum(z), intervals = int, total.seeds = 50,
                partial = FALSE)

# From number of germinated seeds
#----------------------------------------------------------------------------
GermPercent(germinated.seeds = 40, total.seeds = 50)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MeanGermRate.R
\name{MeanGermRate}
\alias{MeanGermRate}
\alias{CVG}
\alias{VarGermRate}
\alias{SEGermRate}
\alias{GermRateRecip}
\title{Germination rate}
\usage{
MeanGermRate(germ.counts, intervals, partial = TRUE)

CVG(germ.counts, intervals, partial = TRUE)

VarGermRate(germ.counts, intervals, partial = TRUE)

SEGermRate(germ.counts, intervals, partial = TRUE)

GermRateRecip(
  germ.counts,
  intervals,
  partial = TRUE,
  method = c("coolbear", "farooq")
)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{method}{The method for computing median germination time. Either
\code{"coolbear"} or \code{"farooq"}.}
}
\value{
For \code{MeanGermRate}, the mean germination rate value as
  \mjseqn{\mathrm{time^{-1}}}.

  For \code{VarGermTime}, the variance of germination rate value as
  \mjseqn{\mathrm{time^{-2}}}.

  For \code{SEGermTime}, the standard error of germination rate as
  \mjseqn{\mathrm{time^{-1}}}.

  For \code{CVG}, the value of Coefficient of of velocity/rate of germination
  or Kotowski's coefficient of velocity as \% \mjseqn{\mathrm{time^{-1}}}.

  For \code{GermRateRecip}, the value of germination rate as
  \mjseqn{\mathrm{time^{-1}}}.
}
\description{
Compute the following metrics: \describe{ \item{\code{MeanGermRate}}{Mean
germination rate (\mjseqn{\overline{V}})
\insertCite{labouriau_germination_1976,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.}
\item{\code{VarGermRate}}{Variance of germination rate (\mjseqn{s_{V}^{2}})
\insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}. }
\item{\code{SEGermRate}}{Standard error of germination rate
(\mjseqn{s_{\overline{V}}})
\insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}. }
\item{\code{CVG}}{Coefficient of velocity/rate of germination or Kotowski's
coefficient of velocity (\mjseqn{CVG})
\insertCite{kotowski_temperature_1926,nichols_two_1968,labouriau_uma_1983,scott_review_1984,bewley_seeds_1994}{germinationmetrics}.
} \item{\code{GermRateRecip}}{Germination rate as reciprocal of median time
(\mjseqn{v_{50}})
\insertCite{went_experimental_1957,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.}
} \loadmathjax
}
\details{
\code{MeanGermRate} computes the mean germination rate
(\mjseqn{\overline{V}}) according to the following formula
\insertCite{labouriau_germination_1976,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{\overline{V} = \frac{\sum_{i=1}^{k}N_{i}}{\sum_{i=1}^{k}N_{i}T_{i}}}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
the \mjseqn{i}th time interval (not the accumulated number, but the number
corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
number of time intervals.

It is the inverse of mean germination time (\mjseqn{\overline{T}}).

\mjsdeqn{\overline{V} = \frac{1}{\overline{T}}}

\code{VarGermRate} computes the variance of germination rate
(\mjseqn{s_{V}^{2}}) according to the following formula
\insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{s_{V}^{2} = \overline{V}^{4} \times s_{T}^{2}}

Where, \mjseqn{s_{T}^{2}} is the variance of germination time.

\code{SEGermRate} computes the standard error of germination time
(\mjseqn{s_{\overline{V}}}) according to the following formula
\insertCite{labouriau_uma_1983,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{s_{\overline{V}} = \sqrt{\frac{s_{V}^{2}}{\sum_{i=1}^{k}N_{i}}}}

Where, \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th
time interval (not the accumulated number, but the number corresponding to
the \mjseqn{i}th interval), and \mjseqn{k} is the total number of time
intervals.

\code{CVG} computes the coefficient of velocity/rate of germination or
Kotowski's coefficient of velocity (\mjseqn{CVG}) according to the following
formula
\insertCite{kotowski_temperature_1926,nichols_two_1968,labouriau_uma_1983,scott_review_1984,bewley_seeds_1994}{germinationmetrics}.

\mjsdeqn{CVG = \frac{\sum_{i=1}^{k}N_{i}}{\sum_{i=1}^{k}N_{i}T_{i}} \times
100}

\mjsdeqn{CVG = \overline{V} \times 100}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
the \mjseqn{i}th time interval (not the accumulated number, but the number
corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
number of time intervals.

\code{GermRateRecip} computes the germination rate (\mjseqn{v_{50}}) as the
reciprocal of the median germination time (\mjseqn{t_{50}})
\insertCite{went_experimental_1957,labouriau_uma_1983,ranal_how_2006}{germinationmetrics}
according to the methods of
\insertCite{coolbear_effect_1984;textual}{germinationmetrics} (Specified by
the argument \code{method = "coolbear"}) or
\insertCite{farooq_thermal_2005;textual}{germinationmetrics} (Specified by
the argument \code{method = "farooq"}) as follows.

\mjsdeqn{v_{50} = \frac{1}{t_{50}}}
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
MeanGermRate(germ.counts = x, intervals = int)
CVG(germ.counts = x, intervals = int)
VarGermRate(germ.counts = x, intervals = int)
SEGermRate(germ.counts = x, intervals = int)
GermRateRecip(germ.counts = x, intervals = int, method = "coolbear")
GermRateRecip(germ.counts = x, intervals = int, method = "farooq")

# From cumulative germination counts
#----------------------------------------------------------------------------
MeanGermRate(germ.counts = y, intervals = int, partial = FALSE)
CVG(germ.counts = y, intervals = int, partial = FALSE)
VarGermRate(germ.counts = y, intervals = int, partial = FALSE)
SEGermRate(germ.counts = y, intervals = int, partial = FALSE)
GermRateRecip(germ.counts = y, intervals = int,
              method = "coolbear", partial = FALSE)
GermRateRecip(germ.counts = y, intervals = int,
              method = "farooq", partial = FALSE)

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{MeanGermTime}},
  \code{\link[germinationmetrics]{t50}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FourPHFfit.R
\name{FourPHF}
\alias{FourPHF}
\alias{FourPHF_fixa}
\alias{FourPHF_fixy0}
\alias{FourPHF_fixa_fixy0}
\alias{RateofGerm}
\title{Four paramter hill function}
\usage{
FourPHF(x, a, bta, c, y0)

FourPHF_fixa(x, a = 100, bta, c, y0)

FourPHF_fixy0(x, a, bta, c)

FourPHF_fixa_fixy0(x, a = 100, bta, c)

RateofGerm(x, a, b, c)
}
\arguments{
\item{x}{The explanatory/independent variable value.}

\item{a}{Parameter \mjseqn{a}.}

\item{bta}{Parameter \mjseqn{\beta} where \mjseqn{b = e^{\beta}}.}

\item{c}{Parameter \mjseqn{c}.}

\item{y0}{Parameter \mjseqn{y_{0}}.}

\item{b}{Parameter \mjseqn{b}.}
}
\value{
The calculated response/dependent value value.
}
\description{
To be used by
\code{\link[germinationmetrics]{FourPHFfit}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WeightGermPercent.R
\name{WeightGermPercent}
\alias{WeightGermPercent}
\title{Weighted germination percentage}
\usage{
WeightGermPercent(germ.counts, total.seeds, intervals, partial = TRUE)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{total.seeds}{Total number of seeds tested.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}
}
\value{
The weighted germination percentage (\%) value.
}
\description{
Compute the Weighted germination percentage
\mjseqn{WGP} or Weighted germination index
\mjseqn{WGI}
\insertCite{reddy_effect_1978,reddy_effect_1985}{germinationmetrics}. \loadmathjax
}
\details{
Weighted germination percentage was calculated by giving maximum weight to
the seeds that germinate first and decreasing weight to the seeds that
germinate subsequently. It is computed as follows
\insertCite{reddy_effect_1978,reddy_effect_1985}{germinationmetrics}.

\mjsdeqn{WGP =
\frac{\sum_{i=1}^{k}(k-i+1)N_{i}}{k \times N} \times 100}

Where, \mjseqn{N_{i}}  is the  number
of seeds that germinated in the time interval
\mjseqn{i} (not cumulative, but partial count),
\mjseqn{N} is the total number of seeds tested,
and \mjseqn{k} is the  total number of time
intervals.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
WeightGermPercent(germ.counts = x, total.seeds = 50, intervals = int)

# From cumulative germination counts
#----------------------------------------------------------------------------
WeightGermPercent(germ.counts = y, total.seeds = 50, intervals = int,
                  partial = FALSE)


}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/t50.R
\name{t50}
\alias{t50}
\title{Median germination time}
\usage{
t50(germ.counts, intervals, partial = TRUE, method = c("coolbear", "farooq"))
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{method}{The method for computing median germination time. Either
\code{"coolbear"} or \code{"farooq"}.}
}
\value{
The median germination time (\mjseqn{t_{50}}) value in the same unit
  of time as specified in the argument \code{intervals}.
}
\description{
Compute the median germination time (\mjseqn{t_{50}}). Median germination
time is the time to reach 50\% of final/maximum germination. \loadmathjax
}
\details{
With argument \code{method} specified as \code{"coolbear"}, median
germination time is computed according to the formula by
\insertCite{coolbear_effect_1984;textual}{germinationmetrics} as follows.

\mjsdeqn{t_{50}=T_{i}+
\frac{(\frac{N+1}{2}-N_{i})(T_{j}-T_{i})}{N_{j}-N_{i}}}

Where, \mjseqn{t_{50}} is the median germination time, \mjseqn{N} is the
final number of germinated seeds, and \mjseqn{N_{i}} and \mjseqn{N_{j}} are
the total number of seeds germinated in adjacent counts at time
\mjseqn{T_{i}} and \mjseqn{T_{j}} respectively, when \mjteqn{N_{i} < \frac{N
+ 1}{2} < N_{j}}{N_{i} &lt; \frac{N + 1}{2} &lt; N_{j}}{N_i < ((N + 1) / 2) <
N_j}.

Similarly with argument \code{method} specified as \code{"farooq"}, median
germination time is computed according to the formula by  by
\insertCite{farooq_thermal_2005;textual}{germinationmetrics} as follows.

\mjsdeqn{t_{50}=T_{i}+ \frac{(\frac{N}{2}-N_{i})(T_{j}-T_{i})}{N_{j}-N_{i}}}

Where, \mjseqn{t_{50}} is the median germination time, \mjseqn{N} is the
final number of germinated seeds, and \mjseqn{N_{i}} and \mjseqn{N_{j}} are
the total number of seeds germinated in adjacent counts at time
\mjseqn{T_{i}} and \mjseqn{T_{j}} respectively, when \mjteqn{N_{i} <
\frac{N}{2} < N_{j}}{N_{i} &lt; \frac{N}{2} &lt; N_{j}}{N_i < (N / 2) < N_j}.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
t50(germ.counts = x, intervals = int, method = "coolbear")
t50(germ.counts = x, intervals = int, method = "farooq")

# From cumulative germination counts
#----------------------------------------------------------------------------
t50(germ.counts = y, intervals = int, partial = FALSE, method = "coolbear")
t50(germ.counts = y, intervals = int, partial = FALSE, method = "farooq")

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{MeanGermRate}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GermSynchrony.R
\name{GermSynchrony}
\alias{GermSynchrony}
\alias{GermUncertainty}
\title{Synchrony and uncertainty of germination}
\usage{
GermSynchrony(germ.counts, intervals, partial = TRUE)

GermUncertainty(germ.counts, intervals, partial = TRUE)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}
}
\value{
For \code{GermUncertainty}, the value of uncertainty of germination
  process in \mjseqn{\mathrm{bit}}.

  For \code{GermSynchrony}, the value of synchrony of germination.
}
\description{
Compute the following metrics:\describe{ \item{\code{GermSynchrony}}{
Synchrony of germination (\mjseqn{Z} index)
\insertCite{primack_longevity_1985,ranal_how_2006}{germinationmetrics}. }
\item{\code{GermUncertainty}}{Synchronization index (\mjseqn{\overline{E}})
or Uncertainty of the germination process (\mjseqn{U}) or Informational
entropy (\mjseqn{H})
\insertCite{shannon_mathematical_1948,labouriau_germination_1976,labouriau_uma_1983}{germinationmetrics}.}
} \loadmathjax
}
\details{
\code{GermSynchrony} computes the value of synchrony of germination
(\mjseqn{Z} index) as follows
\insertCite{primack_longevity_1985,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{Z=\frac{\sum_{i=1}^{k}C_{N_{i},2}}{C_{\Sigma N_{i},2}}}

Where, \mjseqn{C_{N_{i},2}} is the partial combination of the two germinated
seeds from among \mjseqn{N_{i}}, the number of seeds germinated on the
\mjseqn{i}th time interval (estimated as
\mjseqn{C_{N_{i},2}=\frac{N_{i}(N_{i}-1)}{2}}), and \mjseqn{C_{\Sigma
N_{i},2}} is the partial combination of the two germinated seeds from among
the total number of seeds germinated at the final count, assuming that all
seeds that germinated did so simultaneously.

\code{GermUncertainty} computes the value of synchronization index
(\mjseqn{\overline{E}}) or uncertainty of the germination process
(\mjseqn{U}) or informational entropy (\mjseqn{H}) as follows
\insertCite{shannon_mathematical_1948,labouriau_germination_1976,labouriau_uma_1983}{germinationmetrics}.

\mjsdeqn{\overline{E} = -\sum_{i=1}^{k}f_{i}\log_{2}f_{i}}

Where, \mjseqn{f_{i}} is the relative frequency of germination (estimated as
\mjseqn{f_{i}=\frac{N_{i}}{\sum_{i=1}^{k}N_{i}}}), \mjseqn{N_{i}} is the
number of seeds germinated on the \mjseqn{i}th time interval, and \mjseqn{k}
is the total number of time intervals.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
GermSynchrony(germ.counts = x, intervals = int)
GermUncertainty(germ.counts = x, intervals = int)

# From cumulative germination counts
#----------------------------------------------------------------------------
GermSynchrony(germ.counts = y, intervals = int, partial = FALSE)
GermUncertainty(germ.counts = y, intervals = int, partial = FALSE)

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{CUGerm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MeanGermTime.R
\name{MeanGermTime}
\alias{MeanGermTime}
\alias{VarGermTime}
\alias{SEGermTime}
\alias{CVGermTime}
\title{Germination time}
\usage{
MeanGermTime(germ.counts, intervals, partial = TRUE)

VarGermTime(germ.counts, intervals, partial = TRUE)

SEGermTime(germ.counts, intervals, partial = TRUE)

CVGermTime(germ.counts, intervals, partial = TRUE)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}
}
\value{
For \code{MeanGermTime}, the mean germination time value in the same
  unit of time as specified in the argument \code{intervals}.

  For \code{VarGermTime}, the variance of germination time value as
  \mjseqn{\mathrm{time^{2}}}.

  For \code{SEGermTime}, the standard error of germination time in the same
  unit of time specified in the argument \code{intervals}.

  For \code{CVGermTime}, the value of coefficient of variation of the
  germination time.
}
\description{
Compute the following metrics:\describe{ \item{\code{MeanGermTime}}{Mean
germination time (\mjseqn{\overline{T}}) or Mean length of incubation time
\insertCite{edmond_effects_1958,czabator_germination_1962,mock_cold_1972,ellis_improved_1980,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
\item{\code{VarGermTime}}{Variance of germination time (\mjseqn{s_{T}^{2}})
\insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
\item{\code{SEGermTime}}{Standard error of germination time
(\mjseqn{s_{\overline{T}}})
\insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
\item{\code{CVGermTime}}{Coefficient of variation of the germination time
(\mjseqn{CV_{T}})
\insertCite{gomes_curso_1960,ranal_how_2006}{germinationmetrics}.} }
\loadmathjax
}
\details{
\code{MeanGermTime} computes the mean germination time according to the
following formula
\insertCite{edmond_effects_1958,czabator_germination_1962,smith_germinating_1964,ellis_improved_1980,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{\overline{T} = \frac{\sum_{i=1}^{k}N_{i}T_{i}}{\sum_{i=1}^{k}N_{i}}}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
the \mjseqn{i}th time interval (not the accumulated number, but the number
corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
number of time intervals.

It is the same as Sprouting Index (\mjseqn{SI}) or Emergence Index
(\mjseqn{EI}) described by
\insertCite{smith_germinating_1964;textual}{germinationmetrics} and
\insertCite{mock_cold_1972;textual}{germinationmetrics} as well as
Germination Resistance (\mjseqn{GR}) described by
\insertCite{gordon_observations_1969,gordon_germination_1971;textual}{germinationmetrics}.

It is the inverse of mean germination rate (\mjseqn{\overline{V}}).

\mjsdeqn{\overline{T} = \frac{1}{\overline{V}}}

It indicates the average length of time required for maximum germination of a
seed lot. Lower the \mjseqn{\overline{T}}, faster the sample has germinated
and reflects seed vigor.

\code{VarGermTime} computes the variance of germination time according to the
following formula
\insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{s_{T}^{2} =
\frac{\sum_{i=1}^{k}N_{i}(T_{i}-\overline{T})^{2}}{\sum_{i=1}^{k}N_{i}-1}}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
the \mjseqn{i}th time interval (not the accumulated number, but the number
corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
number of time intervals.

\code{SEGermTime} computes the standard error of germination time
(\mjseqn{s_{\overline{T}}}) according to the following formula
\insertCite{labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{s_{\overline{T}} = \sqrt{\frac{s_{T}^{2}}{\sum_{i=1}^{k}N_{i}}}}

Where, \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th
time interval (not the accumulated number, but the number corresponding to
the \mjseqn{i}th interval), and \mjseqn{k} is the total number of time
intervals.

It signifies the accuracy of the calculation of the mean germination time.

\code{CVGermTime} computes the coefficient of variation of germination time
(\mjseqn{CV_{T}}) according to the following formula
\insertCite{gomes_curso_1960,ranal_how_2006}{germinationmetrics}.

\mjsdeqn{CV_{T} = \frac{\sqrt{s_{T}^{2}}}{\overline{T}}}

This indicates the uniformity of germination and permits comparisons
irrespective of the magnitude of mean germination time
(\mjseqn{\overline{T}}).
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
MeanGermTime(germ.counts = x, intervals = int)
VarGermTime(germ.counts = x, intervals = int)
SEGermTime(germ.counts = x, intervals = int)
CVGermTime(germ.counts = x, intervals = int)

# From cumulative germination counts
#----------------------------------------------------------------------------
MeanGermTime(germ.counts = y, intervals = int, partial = FALSE)
VarGermTime(germ.counts = y, intervals = int, partial = FALSE)
SEGermTime(germ.counts = y, intervals = int, partial = FALSE)
CVGermTime(germ.counts = y, intervals = int, partial = FALSE)

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{MeanGermRate}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EmergenceRateIndex.R
\name{EmergenceRateIndex}
\alias{EmergenceRateIndex}
\title{Emergence Rate Index or Germination Rate Index}
\usage{
EmergenceRateIndex(
  germ.counts,
  intervals,
  partial = TRUE,
  total.seeds = NULL,
  method = c("shmueligoldberg", "sgsantanaranal", "bilbrowanjura", "fakorede")
)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{total.seeds}{Total number of seeds.}

\item{method}{The method to be used. Either \code{"shmueligoldberg"},
\code{"sgsantanaranal"}, \code{"bilbrowanjura"} or \code{"fakorede"}.
Default is \code{"shmueligoldberg"} (see \strong{Details}).}
}
\value{
The value of the Emergence rate index with the units according to
  \code{method} as follows.

  \item{\code{shmueligoldberg}}{\mjseqn{\mathrm{count}}}
  \item{\code{sgsantanaranal}}{\mjseqn{\mathrm{no \, unit}}}
  \item{\code{bilbrowanjura}}{\mjseqn{\mathrm{count} \, time^{-1}}}
  \item{\code{fakorede}}{\mjseqn{time \, \mathrm{count}^{-1}}}
}
\description{
Compute the Emergence rate index or Germination rate index according to
\itemize{ \item
\insertCite{shmueli_emergence_1971;textual}{germinationmetrics} and
it's modification by
\insertCite{de_santana_alise_2004;textual}{germinationmetrics}
\insertCite{ranal_how_2006}{germinationmetrics}; \item
\insertCite{bilbro_soil_1982;textual}{germinationmetrics}; and
\item
\insertCite{fakorede_relation_1980;textual}{germinationmetrics},
\insertCite{fakorede_variability_1981;textual}{germinationmetrics},
and
\insertCite{fakorede_heterotic_1983;textual}{germinationmetrics}.}
\loadmathjax
}
\section{Details}{
 Several germination indices are referred to as Emergence
  rate Index (\mjseqn{ERI}).

  \subsection{\insertCite{shmueli_emergence_1971;textual}{germinationmetrics}}{
   With argument \code{method} specified as \code{"shmueligoldberg"}
  \mjseqn{ERI} is estimated as follows.

  \mjsdeqn{ERI = \sum_{i=i_{0}}^{k-1}N_{i}(k-i)}

  Where, \mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th
  time interval (not the accumulated number, but the number corresponding to
  the \mjseqn{i}th interval), \mjseqn{i_{0}} is the time interval when
  emergence/germination started, and \mjseqn{k} is the total number of time
  intervals.

  A modification for the same was suggested by
  \insertCite{de_santana_alise_2004;textual}{germinationmetrics}
  \insertCite{ranal_how_2006}{germinationmetrics} and is estimated by
  dividing Emergence rate index (\mjseqn{ERI}) by total number of emerged
  seedlings (or germinated seeds) as follows with argument \code{method}
  specified as \code{"sgsantanaranal"}).

  \mjsdeqn{ERI_{mod} = \frac{\sum_{i=i_{0}}^{k-1}N_{i}(k-i)}{N_{g}} =
  \frac{ERI}{N_{g}}}

  Where, \mjseqn{N_{g}} is the total number of germinated seeds at the end of
  the test, \mjseqn{N_{i}} is the number of seeds germinated in the
  \mjseqn{i}th time interval (not the accumulated number, but the number
  corresponding to the \mjseqn{i}th interval), \mjseqn{i_{0}} is the time
  interval when emergence/germination started, and \mjseqn{k} is the total
  number of time intervals. }

  \subsection{\insertCite{bilbro_soil_1982;textual}{germinationmetrics}}{
   With argument \code{method} specified as \code{"bilbrowanjura"}
  \mjseqn{ERI} is the estimated as follows.

  \mjsdeqn{ERI = \frac{\sum_{i=1}^{k}N_{i}}{\overline{T}} =
  \frac{N_{g}}{\overline{T}}}

  Where, \mjseqn{N_{g}} is the total number of germinated seeds at the end of
  the test, \mjseqn{N_{i}} is the number of seeds germinated in the
  \mjseqn{i}th time interval (not the accumulated number, but the number
  corresponding to the \mjseqn{i}th interval), and \mjseqn{\overline{T}} is
  the mean germination time or mean emergence time. }

  \subsection{\insertCite{fakorede_relation_1980,fakorede_variability_1981,fakorede_heterotic_1983;textual}{germinationmetrics}}{
   With argument \code{method} specified as \code{"fakorede"} \mjseqn{ERI} is
  the estimated as follows.

  \mjsdeqn{ERI = \frac{\overline{T}}{FGP/100}}

  Where, \mjseqn{\overline{T}} is the Mean germination time and \mjseqn{FGP}
  is the final germination time. }

  \subsection{Others}{ Apart from the above the above, the name Emergence
  rate index (\mjseqn{ERI}) has also been used to for the mathematical
  expressions for Speed of germination
  (\code{\link[germinationmetrics]{GermSpeed}})
  \insertCite{allan_seedling_1962,erbach_tillage_1982}{germinationmetrics},
   Timson's index (\code{\link[germinationmetrics]{TimsonsIndex}})
  \insertCite{chaudhary_effect_1970}{germinationmetrics},
  and George's index
  (\code{\link[germinationmetrics:TimsonsIndex]{GermRateGeorge}})
  \insertCite{chopra_effect_1980}{germinationmetrics}. }
}

\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50)
EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
                   method = "shmueligoldberg")
EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
                   method = "sgsantanaranal")
EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
                   method = "bilbrowanjura")
EmergenceRateIndex(germ.counts = x, intervals = int, total.seeds = 50,
                   method = "fakorede")

# From cumulative germination counts
#----------------------------------------------------------------------------
EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
                   partial = FALSE)
EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
                   partial = FALSE,
                   method = "shmueligoldberg")
EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
                   partial = FALSE,
                   method = "sgsantanaranal")
EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
                   partial = FALSE,
                   method = "bilbrowanjura")
EmergenceRateIndex(germ.counts = y, intervals = int, total.seeds = 50,
                   partial = FALSE,
                   method = "fakorede")

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{GermSpeed}},
  \code{\link[germinationmetrics]{TimsonsIndex}},
  \code{\link[germinationmetrics:TimsonsIndex]{GermRateGeorge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.FourPHFfit.bulk.R
\name{plot.FourPHFfit.bulk}
\alias{plot.FourPHFfit.bulk}
\title{Plot the multiple four-parameter hill function fitted cumulative germination
curves from a \code{FourPHFfit.bulk} object}
\usage{
\method{plot}{FourPHFfit.bulk}(
  x,
  rog = FALSE,
  annotate = c("none", "t50.total", "t50.germ", "tmgr", "mgt", "uniformity"),
  limits = TRUE,
  group.col,
  show.points = FALSE,
  ...
)
}
\arguments{
\item{x}{An data frame object of class \code{FourPHFfit.bulk} obtained as
output from the \code{\link[germinationmetrics]{FourPHFfit.bulk}} function.}

\item{rog}{If \code{TRUE}, plots the Rate of Germination curves (RoG) instead
of the Four-Parameter Hill Function fitted cumulative germination curves
(FPHF). Default is \code{FALSE}.}

\item{annotate}{The parameter to be annotated in the plot. Either
\code{"t50.total"} (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Total}}}),
\code{"t50.germ"} (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Germ}}}),
\code{"tmgr"} (TMGR), \code{"mgt"} (MGT), or \code{"uniformity"}
(\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}}).
 All the parameters are annotated as vertical lines, except for
(\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}}),
 which is annotated as horizontal lines.}

\item{limits}{logical. If \code{TRUE}, set the limits of y axis (germination
percentage) between 0 and 100 in the germination curve plot. If
\code{FALSE}, limits are set according to the data. Default is \code{TRUE}.}

\item{group.col}{The names of the column in data frame \code{x} according to
which grouping has to be done.}

\item{show.points}{logical. If \code{TRUE}, shows the original data points
(only for the FPHF curve). Default is \code{TRUE}.}

\item{\dots}{Default plot arguments.}
}
\value{
The plot of the cumulative germination curve as an object of class
  \code{ggplot}.
}
\description{
\code{plot.FourPHFfit} plots multiple four-parameter hill function fitted
cumulative germination curves (FPHF curves) or  the rate of germination
curves (RoG curves) from a \code{FourPHFfit.bulk} object as an object of
class \code{ggplot}. Further, either one of the parameters such as time
required for 50\% of total seeds to germinate
(\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Total}}}), time required for 50\% of
viable/germinated seeds to germinate
(\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Germ}}}), Time at Maximum
Germination Rate (TMGR), Mean Germination Time (MGT), or uniformity value
(\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}})
can also be annotated in the plot as specified in the \code{annotate}
argument. \loadmathjax
}
\examples{
\donttest{
data(gcdata)

counts.per.intervals <- c("Day01", "Day02", "Day03", "Day04", "Day05",
                          "Day06", "Day07", "Day08", "Day09", "Day10",
                          "Day11", "Day12", "Day13", "Day14")

fits <- FourPHFfit.bulk(gcdata, total.seeds.col = "Total Seeds",
                    counts.intervals.cols = counts.per.intervals,
                    intervals = 1:14, partial = TRUE,
                    fix.y0 = TRUE, fix.a = TRUE, xp = c(10, 60),
                    tmax = 20, tries = 3, umax = 90, umin = 10)

# Plot FPHF curves
plot(fits, group.col = "Genotype")

# Plot ROG curves
plot(fits, rog = TRUE, group.col = "Genotype")

# Plot FPHF curves with points
plot(fits, group.col = "Genotype", show.points = TRUE)

# Plot FPHF curves with annotations
plot(fits, group.col = "Genotype", annotate = "t50.total")
plot(fits, group.col = "Genotype", annotate = "t50.germ")
plot(fits, group.col = "Genotype", annotate = "tmgr")
plot(fits, group.col = "Genotype", annotate = "mgt")
plot(fits, group.col = "Genotype", annotate = "uniformity")

# Plot ROG curves with annotations
plot(fits, rog = TRUE, group.col = "Genotype", annotate = "t50.total")
plot(fits, rog = TRUE, group.col = "Genotype", annotate = "t50.germ")
plot(fits, rog = TRUE, group.col = "Genotype", annotate = "tmgr")
plot(fits, rog = TRUE, group.col = "Genotype", annotate = "mgt")
plot(fits, rog = TRUE, group.col = "Genotype", annotate = "uniformity")



# Change colour of curves using ggplot2 options
library(ggplot2)
curvesplot <- plot(fits, group.col = "Genotype")

# 'Dark2' palette from RColorBrewer
curvesplot + scale_colour_brewer(palette = "Dark2")

# Manual colours
curvesplot +
  scale_colour_manual(values = c("Coral", "Brown", "Blue",
                                 "Aquamarine", "Red"))
}

}
\seealso{
\code{\link[germinationmetrics]{FourPHFfit.bulk}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TimsonsIndex.R
\name{TimsonsIndex}
\alias{TimsonsIndex}
\alias{GermRateGeorge}
\title{Timson's index}
\usage{
TimsonsIndex(
  germ.counts,
  intervals,
  partial = TRUE,
  total.seeds,
  max,
  modification = c("none", "labouriau", "khanungar")
)

GermRateGeorge(germ.counts, intervals, partial = TRUE, max)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{total.seeds}{Total number of seeds.}

\item{max}{The maximum interval value up to which Timson's index/George's
germination rate is to be computed.}

\item{modification}{The modification to be applied. Either \code{"none"},
\code{"labouriau"} or \code{"khanungar"}. Default is \code{"none"} (see
\strong{Details}).}
}
\value{
For \code{TimsonsIndex}, the value of Timson's index (\%) if
  \code{modification = "none"}. If \code{modification = "labouriau"}, then
  value with no unit and if \code{modification = "khanungar"}, then value as
  \% \mjseqn{\mathrm{time^{-1}}}.

  For \code{GermRateGeorge} The value of George's germination rate as
  \mjseqn{count time}.
}
\description{
Compute George's index
\insertCite{george_influence_1961,tucker_estimating_1965,nichols_two_1968,chopra_effect_1980}{germinationmetrics},
Timson's index or Timson's germination velocity index
\insertCite{grose_laboratory_1958,timson_new_1965,lyon_rapid_1966,chaudhary_effect_1970,negm_effects_1978,brown_representing_1988,baskin_seeds_1998}{germinationmetrics}
and it's modifications by Labouriau
\insertCite{ranal_how_2006}{germinationmetrics} and
\insertCite{khan_effect_1984}{germinationmetrics}. \loadmathjax
}
\details{
Timson's index \insertCite{timson_new_1965}{germinationmetrics} is computed
as follows
\insertCite{grose_laboratory_1958,brown_representing_1988,baskin_seeds_1998}{germinationmetrics}.

\mjsdeqn{\Sigma k = \sum_{i=1}^{k}G_{i}}

Where, \mjseqn{G_{i}} is the cumulative germination percentage in time
interval \mjseqn{i} and \mjseqn{k} is the total number of time intervals.

For example ten summation (\mjseqn{\Sigma 10}) is expressed as follows.

\mjsdeqn{\Sigma 10 = G_{1}+G_{2}+\cdots+G_{10}}

Where \mjseqn{G_{1},G_{2},\cdots G_{10}} are the cumulative germination
percentage at day 1, 2, 3, \mjseqn{\cdots}, 10 respectively.

Similarly \mjseqn{\Sigma 5} or \mjseqn{\Sigma 20} can be estimated. For
\mjseqn{\Sigma 10}, the value can range from 0 (no germination) to 1,000
(100\% germination after 24 hours).

It is the progressive total of cumulative germination percentage recorded at
specific intervals for a set period of time.  It combines onset, rate and
total percentage of germination and estimates the area under the cumulative
germination percentage curve. It is same as the indices for area under time
course curve of germination described by
\insertCite{lyon_rapid_1966;textual}{germinationmetrics} as
well as \insertCite{negm_effects_1978;textual}{germinationmetrics}. It is
also described as Emergence Rate Index (\mjseqn{ERI}) by
\insertCite{chaudhary_effect_1970;textual}{germinationmetrics}.

\insertCite{goodchild_method_1971;textual}{germinationmetrics}, described the
same in terms of partial germination percentage as follows.

\mjsdeqn{\Sigma k = \sum_{i=1}^{k}g_{i}(k-j)}

Where, \mjseqn{g_{i}} is the germination (not cumulative, but partial
germination) in time interval \mjseqn{i} (\mjseqn{i} varying from \mjseqn{0}
to \mjseqn{k}), \mjseqn{k} is the total number of time intervals, and
\mjseqn{j = i - 1}.

Timson's index is equivalent to the Germination Energy Index \mjseqn{GEI}
proposed by \insertCite{grose_laboratory_1958;textual}{germinationmetrics}.

As Timson's index is useful for comparison only when samples have similar
germinabilities or final germination percentage, the following modification
was suggested by Labouriau \insertCite{ranal_how_2006}{germinationmetrics}
(Specified by the argument \code{modification = "labouriau"}).

\mjsdeqn{\Sigma k_{mod} = \frac{\Sigma k}{\sum_{i=1}^{k}g_{i}}}

Here Timson's index (\mjseqn{\Sigma k}) is divided by the sum of partial
germination percentages.

Similarly another modification was proposed by
\insertCite{khan_effect_1984}{germinationmetrics}, where Timson's index
(\mjseqn{\Sigma k}) is divided by the total time period of germination
(\mjseqn{T_{k}}) (Specified by the argument \code{modification =
"khanungar"}).

\mjsdeqn{\Sigma k_{mod} = \frac{\Sigma k}{T_{k}}}

Timson's index is similar to the Germination Rate (\mjseqn{GR}) proposed by
\insertCite{george_influence_1961;textual}{germinationmetrics} as follows
\insertCite{tucker_estimating_1965,nichols_two_1968}{germinationmetrics}.

\mjsdeqn{GR = \sum_{i=1}^{k}N_{i}K_{i}}

Where \mjseqn{N_{i}} is the number of seeds germinated by \mjseqn{i}th
interval, \mjseqn{K_{i}} is the number of intervals(e.g. days) until the end
of the test, and \mjseqn{k} is the total number of time intervals.

This index uses number of seeds germinated instead of germination percentage.
It is also described as Emergence Rate Index (\mjseqn{ERI}) by
\insertCite{chopra_effect_1980;textual}{germinationmetrics}.

So germination rate (\mjseqn{GR}) can also be described in terms of partial
(\mjseqn{N_{i}}) and cumulative (\mjseqn{\sum_{j=1}^{i}N_{j}}) number of
seeds germinated in each time interval (\mjseqn{i}) as follows.

\mjsdeqn{GR = \sum_{i=1}^{k}\sum_{j=1}^{i}N_{j}}

\mjsdeqn{GR = \sum_{i=1}^{k}N_{i}(k-j)}
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
# Without max specified
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50)
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
             modification = "none")
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
             modification = "labouriau")
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
             modification = "khanungar")
GermRateGeorge(germ.counts = x, intervals = int)

# With max specified
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50, max = 10)
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
             max = 10, modification = "none")
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
             max = 10, modification = "labouriau")
TimsonsIndex(germ.counts = x, intervals = int, total.seeds = 50,
             max = 10, modification = "khanungar")
GermRateGeorge(germ.counts = x, intervals = int, max = 10)
GermRateGeorge(germ.counts = x, intervals = int, max = 14)


# From cumulative germination counts
#----------------------------------------------------------------------------
# Without max specified
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50)
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50,
             modification = "none")
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50,
             modification = "labouriau")
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50,
             modification = "khanungar")
GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE)

# With max specified
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50, max = 10)
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50,
             max = 10, modification = "none")
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50,
             max = 10, modification = "labouriau")
TimsonsIndex(germ.counts = y, intervals = int, partial = FALSE,
             total.seeds = 50,
             max = 10, modification = "khanungar")
GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE,
               max = 10)
GermRateGeorge(germ.counts = y, intervals = int, partial = FALSE,
               max = 14)
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FirstGermTime.R
\name{FirstGermTime}
\alias{FirstGermTime}
\alias{LastGermTime}
\alias{PeakGermTime}
\alias{TimeSpreadGerm}
\title{Time for first, last and peak germination}
\usage{
FirstGermTime(germ.counts, intervals, partial = TRUE)

LastGermTime(germ.counts, intervals, partial = TRUE)

PeakGermTime(germ.counts, intervals, partial = TRUE)

TimeSpreadGerm(germ.counts, intervals, partial = TRUE)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}
}
\value{
For \code{FirstGermTime}, the time of first germination value in the
  same unit of time as specified in the argument \code{intervals}.

  For \code{LastGermTime}, the time of last germination value in the same
  unit of time as specified in the argument \code{intervals}.

  For \code{TimeSpreadGerm}, the time spread of germination value in the same
  unit of time as specified in the argument \code{intervals}.

  For \code{PeakGermTime}, the time(s) of peak germination value(s) as a
  numeric vector in the same unit of time as specified in the argument
  \code{intervals}.
}
\description{
Compute the following metrics: \describe{ \item{\code{FirstGermTime}}{Time of
first germination or Germination time lag (\mjseqn{t_{0}})
\insertCite{edwards_temperature_1932,czabator_germination_1962,goloff_germination_1975,labouriau_germinacao_1983,ranal_effects_1999,quintanilla_effect_2000}{germinationmetrics}.}
\item{\code{LastGermTime}}{Time of last germination (\mjseqn{t_{g}})
\insertCite{edwards_temperature_1932,labouriau_germinacao_1983,ranal_how_2006}{germinationmetrics}.}
\item{\code{TimeSpreadGerm}}{Time spread of germination
\insertCite{al-mudaris_notes_1998,kader_comparison_2005}{germinationmetrics}
or Germination distribution
\insertCite{schrader_seed_2000}{germinationmetrics}.}
\item{\code{PeakGermTime}}{Peak time of germination or Modal time of
germination (\mjseqn{t_{peak}})
\insertCite{ranal_how_2006}{germinationmetrics}.} } \loadmathjax
}
\details{
Time of first germination indicates time of germination of the faster seeds
in a seedlot.

Lower value of time of first germination indicates faster initiation of
germination and lower value of time of last germination indicates faster
termination of germination.

\mjsdeqn{t_{0} = \min \lbrace T_{i} : N_{i} \neq  0 \rbrace}

\mjsdeqn{t_{g} = \max \lbrace T_{i} : N_{i} \neq  0 \rbrace}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval and \mjseqn{N_{i}} is the number of seeds germinated in
the \mjseqn{i}th time interval (not the accumulated number, but the number
corresponding to the \mjseqn{i}th interval).


Time spread of germination (\mjseqn{t_{g}-t_{0}}) indicates difference
between faster and slower germinating members of a sample.

Peak time of germination is the time in which highest frequency of germinated
seeds are observed. Multiple peak times of germination are possible and if
detected are indicated by a warning message.

\mjsdeqn{t_{peak} = \lbrace T_{i} : N_{i} = N_{max} \rbrace}

Where, \mjseqn{N_{max}} is the maximum number of seeds germinated per
interval.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
z <- c(0, 0, 0, 0, 11, 11, 9, 7, 1, 0, 1, 0, 0, 0)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
FirstGermTime(germ.counts = x, intervals = int)
LastGermTime(germ.counts = x, intervals = int)
TimeSpreadGerm(germ.counts = x, intervals = int)
PeakGermTime(germ.counts = x, intervals = int)
# For multiple peak germination times
PeakGermTime(germ.counts = z, intervals = int)

# From cumulative germination counts
#----------------------------------------------------------------------------
FirstGermTime(germ.counts = y, intervals = int, partial = FALSE)
LastGermTime(germ.counts = y, intervals = int, partial = FALSE)
TimeSpreadGerm(germ.counts = y, intervals = int, partial = FALSE)
PeakGermTime(germ.counts = y, intervals = int, partial = FALSE)
# For multiple peak germination time
PeakGermTime(germ.counts = cumsum(z), intervals = int, partial = FALSE)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CUGerm.R
\name{CUGerm}
\alias{CUGerm}
\title{Coefficient of uniformity of germination}
\usage{
CUGerm(germ.counts, intervals, partial = TRUE)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}
}
\value{
The value of the coefficient of uniformity of germination as
  \mjseqn{\mathrm{time^{-2}}}.
}
\description{
Compute the Coefficient of uniformity of germination (CUG).
}
\details{
CUG is computed as follows
\insertCite{heydecker_seed_1972,bewley_seeds_1994}{germinationmetrics}.
\loadmathjax

\mjsdeqn{CUG =
\frac{\sum_{i=1}^{k}N_{i}}{\sum_{i=1}^{k}(\overline{T}-T_{i})^{2}N_{i}}}

Where, \mjseqn{\overline{T}} is the the mean germination time, \mjseqn{T_{i}}
is the time from the start of the experiment to the \mjseqn{i}th interval,
\mjseqn{N_{i}} is the number of seeds germinated in the \mjseqn{i}th time
interval (not the accumulated number, but the number corresponding to the
\mjseqn{i}th interval), and \mjseqn{k} is the total number of time intervals.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
CUGerm(germ.counts = x, intervals = int)

# From cumulative germination counts
#----------------------------------------------------------------------------
CUGerm(germ.counts = y, intervals = int, partial = FALSE)

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{GermSynchrony}},
  \code{\link[germinationmetrics]{MeanGermTime}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MeanGermPercent.R
\name{MeanGermPercent}
\alias{MeanGermPercent}
\alias{MeanGermNumber}
\title{Mean germination percentage and number of seeds per time interval}
\usage{
MeanGermPercent(
  germinated.seeds,
  germ.counts,
  total.seeds,
  intervals,
  partial = TRUE
)

MeanGermNumber(germ.counts, intervals, partial = TRUE)
}
\arguments{
\item{germinated.seeds}{Number of germinated seeds}

\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{total.seeds}{Total number of seeds.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}
}
\value{
The value of mean germination percentage as \%
  \mjseqn{\mathrm{time^{-1}}} or mean number of seeds per time interval as
  \mjseqn{\mathrm{count \, time^{-1}}}.
}
\description{
Compute the following metrics:\describe{ \item{\code{MeanGermPercent}}{
Mean/average germination percentage per unit time (\mjseqn{\overline{GP}})
\insertCite{czabator_germination_1962}{germinationmetrics}. }
\item{\code{MeanGermNumber}}{Number of seeds germinated per unit time
(\mjseqn{\overline{N}})
\insertCite{khamassi_optimal_2013}{germinationmetrics}.}} \loadmathjax
}
\details{
Mean germination percentage per unit time (\mjseqn{\overline{GP}}) is
computed as follows
\insertCite{czabator_germination_1962}{germinationmetrics}.

\mjsdeqn{\overline{GP} = \frac{GP}{T_{k}}}

Where, \mjseqn{GP} is the final germination percentage, \mjseqn{T_{k}} is the
time at the \mjseqn{k}th time interval, and \mjseqn{k} is the total number of
time intervals required for final germination.

Mean number of seeds germinated per unit time (\mjseqn{\overline{N}}) is
computed as follows \insertCite{khamassi_optimal_2013}{germinationmetrics}.

\mjsdeqn{\overline{N} = \frac{N_{g}}{T_{k}}}

Where, \mjseqn{N_{g}} is the number of germinated seeds at the end of the
germination test, \mjseqn{T_{k}} is the time at the \mjseqn{k}th time
interval, and \mjseqn{k} is the total number of time intervals required for
final germination.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
MeanGermPercent(germ.counts = x, total.seeds = 50, intervals = int)
MeanGermNumber(germ.counts = x, intervals = int)

# From cumulative germination counts
#----------------------------------------------------------------------------
MeanGermPercent(germ.counts = y, total.seeds = 50, intervals = int, partial = FALSE)
MeanGermNumber(germ.counts = y, intervals = int, partial = FALSE)

# From number of germinated seeds
#----------------------------------------------------------------------------
MeanGermPercent(germinated.seeds = 40, total.seeds = 50, intervals = int)

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{GermPercent}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FourPHFfit.R
\name{FourPHFfit}
\alias{FourPHFfit}
\title{Fit four-parameter hill function}
\usage{
FourPHFfit(
  germ.counts,
  intervals,
  total.seeds,
  partial = TRUE,
  fix.y0 = TRUE,
  fix.a = TRUE,
  tmax,
  xp = c(10, 60),
  umin = 10,
  umax = 90,
  tries = 3
)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{total.seeds}{Total number of seeds.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{fix.y0}{Force the intercept of the y axis through 0.}

\item{fix.a}{Fix a as the actual maximum germination percentage at the end of
the experiment.}

\item{tmax}{The time up to which AUC is to be computed.}

\item{xp}{Germination percentage value(s) for which the corresponding time is
to be computed as a numeric vector. Default is \code{c(10, 60)}.}

\item{umin}{The minimum germination percentage value for computing
uniformity. Default is \code{10}. See \strong{\code{Details}}.}

\item{umax}{The maximum germination percentage value for computing
uniformity. Default is \code{90}. See \strong{\code{Details}}.}

\item{tries}{The number of tries to be attempted to fit the curve. Default is
\code{3}.}
}
\value{
A list with the following components: \item{data}{A data frame with
  the data used for computing the model.} \item{Parameters}{A data frame of
  parameter estimates, standard errors and p value.}  \item{Fit}{A one-row
  data frame with estimates of model fitness such as log likelyhoods, Akaike
  Information Criterion, Bayesian Information Criterion, deviance and
  residual degrees of freedom.}  \item{a}{The asymptote or the maximum
  cumulative germination percentage.}  \item{b}{The mathematical parameter
  controlling the shape and steepness of the germination curve.} \item{c}{The
  half-maximal activation level.}  \item{y0}{The intercept on the y axis.}
  \item{lag}{Time at germination onset.} \item{Dlag50}{duration between the
  time at germination onset (lag) and that at 50\% germination.}
  \item{t50.total}{Time required for 50\% of total seeds to germinate. Will
  be \code{NaN} if more than 50\% of total seeds do not germinate.}
  \item{txp.total}{Time required for x\% (as specified in argument \code{xp})
  of total seeds to germinate. Will be \code{NaN} if more than x\% of total
  seeds do not germinate.} \item{t50.Germinated}{Time required for 50\%
  of viable/germinated seeds to germinate.} \item{txp.Germinated}{Time
  required for x\% (as specified in argument \code{xp}) of viable/germinated
  seeds to germinate.} \item{Uniformity}{Time interval between \code{umin}\%
  and \code{umax}\% of viable seeds to germinate.} \item{TMGR}{Time at
  maximum germination rate.} \item{AUC}{The estimate of area under the
  curve.} \item{MGT}{Mean germination time.} \item{Skewness}{Skewness of mean
  germination time.} \item{msg}{The message from
  \code{\link[gslnls]{gsl_nls}}.} \item{isConv}{Logical value indicating
  whether convergence was achieved.}
  \item{model}{The raw fitted model output as a list of class
  \code{gsl_nls}.}
}
\description{
Fit a four-parameter hill function
\insertCite{el-kassaby_seed_2008}{germinationmetrics} to cumulative
germination count data and compute the associated parameters. \loadmathjax
}
\details{
The cumulative germination count data of a seed lot can be modelled to fit a
four-parameter hill function defined as follows
\insertCite{el-kassaby_seed_2008}{germinationmetrics}.

\mjsdeqn{y = y_{0}+\frac{ax^{b}}{c^{b}+x^{b}}}

Where, \mjseqn{y} is the cumulative germination percentage at time
\mjseqn{x}, \mjseqn{y_{0}} is the intercept on the y axis, \mjseqn{a} is the
asymptote, or maximum cumulative germination percentage, which is equivalent
to germination capacity, \mjseqn{b} is a mathematical parameter controlling
the shape and steepness of the germination curve (the larger the \mjseqn{b}
parameter, the steeper the rise toward the asymptote \mjseqn{a}, and the
shorter the time between germination onset and maximum germination), and
\mjseqn{c} is the "half-maximal activation level" which represents the time
required for 50\% of viable seeds to germinate (\mjseqn{c} is equivalent to
the germination speed).

In \code{FourPHFfit}, this model has be reparameterized by substituting
\mjseqn{b} with \mjseqn{e^{\beta}} to constraint \mjseqn{b} to positive
values only.

\mjsdeqn{y = y_{0}+\frac{ax^{e^{\beta}}}{c^{e^{\beta}}+x^{e^{\beta}}}}

Where, \mjseqn{b = e^{\beta}} and \mjseqn{\beta = \log_{e}(b)}.

The curve fitting is performed with nonlinear
\code{\link[gslnls:gsl_nls]{gslnls}} package, a R interface to the
least-squares optimization with the GNU Scientific Library (GSL) with the
Levenberg-Marquardt algorithm
\insertCite{galassi_gnu_2009,chau_gslnls_2023}{germinationmetrics}.

Once this function is fitted to the curve, \code{FourPHFfit} computes the
time to 50\% germination of total seeds (\code{t50.total}) or viable seeds
(\code{t50.Germinated}). Similarly the time at any percentage of germination
(in terms of both total and viable seeds) as specified in argument \code{xp}
can be computed.

The time at germination onset (\mjseqn{lag}) can be computed as follows.

\mjsdeqn{lag = b\sqrt{\frac{-y_{0}c^{b}}{a + y_{0}}}}

The value \mjseqn{D_{lag-50}} is defined as the duration between the time at
germination onset (lag) and that at 50\% germination (\mjseqn{c}).

The time interval between the percentages of viable seeds specified in the
arguments \code{umin} and \code{umin} to germinate is computed as uniformity
(\mjseqn{U_{t_{max}-t_{min}}}).

\mjsdeqn{U_{t_{max}-t_{min}} = t_{max} - t_{min}}

The partial derivative of the four-parameter hill function gives the
instantaneous rate of germination (\mjseqn{s}) as follows.

\mjsdeqn{s = \frac{\partial y}{\partial x} =
\frac{abc^{b}x^{b-1}}{(c^{b}+x^{b})^{2}}}

From this function for instantaneous rate of germination, the time at maximum
germination rate (\mjseqn{TMGR}) can be estimated as follows.

\mjsdeqn{TMGR = b \sqrt{\frac{c^{b}(b-1)}{b+1}}}

TMGR represents the point in time when the instantaneous rate of germination
starts to decline.

The area under the curve (\mjseqn{AUC}) is obined by integration of the
fitted curve between time 0 and time specified in the argument `tmax`.

Integration of the fitted curve gives the value of mean germination time
(\mjseqn{MGT}) and the skewness of the germination curve is computed as the
ratio of \mjseqn{MGT} and the time for 50\% of viable seeds to germinate
(\mjseqn{t_{50}}).

\mjsdeqn{Skewness = \frac{MGT}{t_{50}}}

If final germination percentage is less than 10\%, a warning is given, as the
results may not be informative.
}
\examples{

x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)
total.seeds = 50

# From partial germination counts
#----------------------------------------------------------------------------
FourPHFfit(germ.counts = x, intervals = int, total.seeds = 50, tmax = 20)

# From cumulative germination counts
#----------------------------------------------------------------------------
FourPHFfit(germ.counts = y, intervals = int, total.seeds = 50, tmax = 20,
           partial = FALSE)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GermIndex.R
\name{GermIndex}
\alias{GermIndex}
\title{Germination Index}
\usage{
GermIndex(
  germ.counts,
  intervals,
  partial = TRUE,
  total.seeds,
  modification = c("none", "santanaranal")
)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{total.seeds}{Total number of seeds.}

\item{modification}{The modification to be applied. Either \code{"none"} or
\code{"santanaranal"}. Default is \code{"none"} (see \strong{Details}).}
}
\value{
The value of the Germination index in the same unit of time as
  specified in the argument \code{intervals}.
}
\description{
Compute the Germination index according to
\insertCite{melville_seed_1980;textual}{germinationmetrics} and
its modification by
\insertCite{de_santana_alise_2004;textual}{germinationmetrics}
\insertCite{ranal_how_2006}{germinationmetrics}. \loadmathjax
}
\details{
Germination index (\mjseqn{GI}) is computed as follows
\insertCite{melville_seed_1980}{germinationmetrics}.

\mjsdeqn{GI = \sum_{i=1}^{k}\frac{\left | \left ( T_{k} - T_{i} \right )
N_{i}\right |}{N_{t}}}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval (day for the example), \mjseqn{N_{i}} is the number of
seeds germinated in the \mjseqn{i}th time interval (not the accumulated
number, but the number corresponding to the \mjseqn{i}th interval),
\mjseqn{N_{t}} is the total number of seeds used in the test, and \mjseqn{k}
is the total number of time intervals.

A modified germination index (\mjseqn{GI_{mod}}) was proposed by
\insertCite{de_santana_alise_2004}{germinationmetrics} where total number of
seeds tested (\mjseqn{N_{t}}) is replaced by the total number of germinated
seeds (\mjseqn{N_{g}}) in the expression of
\insertCite{melville_seed_1980;textual}{germinationmetrics} to
remove the effect of sample size
\insertCite{ranal_how_2006}{germinationmetrics}. It is computed as follows
(Specified by the argument \code{modification = "santanaranal"}).

\mjsdeqn{GI_{mod} = \sum_{i=1}^{k}\frac{\left | \left ( T_{k} - T_{i} \right
) N_{i}\right |}{N_{g}}}

The term Germination index (\mjseqn{GI}) has also been used to describe the
mathematical expression for Speed of germination
(\code{\link[germinationmetrics]{GermSpeed}}) by
\insertCite{aosa_seed_1983;textual}{germinationmetrics}.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
GermIndex(germ.counts = x, intervals = int, total.seeds = 50)
GermIndex(germ.counts = x, intervals = int, total.seeds = 50,
          modification = "none")
GermIndex(germ.counts = x, intervals = int, total.seeds = 50,
          modification = "santanaranal")

# From cumulative germination counts
#----------------------------------------------------------------------------
GermIndex(germ.counts = y, intervals = int, partial = FALSE,
          total.seeds = 50)
GermIndex(germ.counts = y, intervals = int, partial = FALSE,
          total.seeds = 50,
          modification = "none")
GermIndex(germ.counts = y, intervals = int, partial = FALSE,
          total.seeds = 50,
          modification = "santanaranal")

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link[germinationmetrics]{GermSpeed}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GermSpeed.R
\name{GermSpeed}
\alias{GermSpeed}
\alias{GermSpeedAccumulated}
\alias{GermSpeedCorrected}
\title{Speed of germination}
\usage{
GermSpeed(
  germ.counts,
  intervals,
  partial = TRUE,
  percent = FALSE,
  total.seeds = NULL
)

GermSpeedAccumulated(
  germ.counts,
  intervals,
  partial = TRUE,
  percent = FALSE,
  total.seeds = NULL
)

GermSpeedCorrected(
  germ.counts,
  intervals,
  partial = TRUE,
  total.seeds,
  method = c("normal", "accumulated")
)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{percent}{logical. If \code{TRUE}, germination percentage is used
instead of counts for computation. Default is \code{FALSE}.}

\item{total.seeds}{Total number of seeds. Mandatory for computation when
\code{percent = TRUE}.}

\item{method}{The method for computing germination speed in
\code{GermSpeedCorrected}. Either \code{"normal"} (\code{GermSpeed}) or
\code{"accumulated"} (\code{GermSpeedAccumulated}).}
}
\value{
For \code{GermSpeed}, the value of germination speed as
  \mjseqn{\mathrm{count \, time^{-1}}} or \% \mjseqn{\mathrm{time^{-1}}}.

  For \code{GermSpeedAccumulated}, the value of accumulated germination speed
  as \mjseqn{\mathrm{count \, time^{-1}}} or \% \mjseqn{\mathrm{time^{-1}}}.

  For \code{GermSpeedCorrected}, the corrected speed of germination as
  \mjseqn{\mathrm{time^{-1}}}.
}
\description{
Compute the following metrics: \describe{ \item{\code{GermSpeed}}{Speed of
germination (\mjseqn{S}) or Germination rate index or Index of velocity of germination or
Germination index or Emergence rate index
\insertCite{throneberry_relation_1955,maguire_speed_1962,allan_seedling_1962,kendrick_photocontrol_1969,bouton_germination_1976,erbach_tillage_1982,aosa_seed_1983,khandakar_jute_1983,hsu_planting_1986,bradbeer_seed_1988,wardle_allelopathic_1991}{germinationmetrics}.
} \item{\code{GermSpeedAccumulated}}{Speed of accumulated germination (\mjseqn{S_{accumulated}})
\insertCite{bradbeer_seed_1988,wardle_allelopathic_1991,haugland_experiments_1996,de_santana_alise_2004}{germinationmetrics}.
} \item{\code{GermSpeedCorrected}}{Corrected speed of germination (\mjseqn{\hat{S}}) or
Corrected germination rate index
\insertCite{evetts_germination_1972}{germinationmetrics}.} } \loadmathjax
}
\details{
\code{GermSpeed} computes the speed of germination according to the following
formula
\insertCite{throneberry_relation_1955,maguire_speed_1962,allan_seedling_1962,kendrick_photocontrol_1969,bouton_germination_1976,erbach_tillage_1982,aosa_seed_1983,khandakar_jute_1983,hsu_planting_1986,bradbeer_seed_1988,wardle_allelopathic_1991}{germinationmetrics}.

\mjsdeqn{S = \frac{N_{1}}{T_{1}} + \frac{N_{2}}{T_{2}} + \frac{N_{3}}{T_{3}}
+ \cdots + \frac{N_{k}}{T_{k}}}

Where, \mjseqn{N_{1}}, \mjseqn{N_{2}}, \mjseqn{N_{3}}, \mjseqn{\cdots},
\mjseqn{N_{k}} are the number of germinated seeds observed at time (days or
hours) \mjseqn{T_{1}}, \mjseqn{T_{2}}, \mjseqn{T_{3}}, \mjseqn{\cdots},
\mjseqn{T_{k}} after sowing (Not accumulated/cumulative number, but the
number of seeds that germinated at the specific time) and \mjseqn{k} is the
total number of time intervals.

It is the same as Emergence Rate Index (\mjseqn{ERI}) described by
\insertCite{allan_seedling_1962;textual}{germinationmetrics},
\insertCite{erbach_tillage_1982;textual}{germinationmetrics} and
\insertCite{hsu_planting_1986;textual}{germinationmetrics} as well as
Germination Index (\mjseqn{GI}) according to
\insertCite{aosa_seed_1983;textual}{germinationmetrics}.

The formula can also be described as follows.

\mjsdeqn{S = \sum_{i=1}^{k}\frac{N_{i}}{T_{i}}}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{N_{i}} is the number of seeds germinated in
the \mjseqn{i}th time interval (not the accumulated number, but the number
corresponding to the \mjseqn{i}th interval), and \mjseqn{k} is the total
number of time intervals.

\code{GermSpeedAccumulated} computes the speed of accumulated germination as
follows
\insertCite{bradbeer_seed_1988,wardle_allelopathic_1991,haugland_experiments_1996,de_santana_alise_2004}{germinationmetrics}.

\mjsdeqn{S_{accumulated} = \frac{N_{1}}{T_{1}} + \frac{N_{1} + N_{2}}{T_{2}}
+ \frac{N_{1} + N_{2} + N_{3}}{T_{3}} + \cdots +\frac{N_{1} + N_{2} + \cdots
+ N_{k}}{T_{k}}}

Where, \mjseqn{N_{1}}, \mjseqn{N_{2}}, \mjseqn{N_{3}}, \mjseqn{\cdots},
\mjseqn{N_{k}} are the number of germinated seeds observed at time (days or
hours) \mjseqn{T_{1}}, \mjseqn{T_{2}}, \mjseqn{T_{3}}, \mjseqn{\cdots},
\mjseqn{T_{k}} after sowing (Not accumulated/cumulative number, but the
number of seeds that germinated at the specific time), and \mjseqn{k} is the
total number of time intervals.

The formula can also be described as follows.

\mjsdeqn{S_{accumulated} = \sum_{i=1}^{k}\frac{\sum_{j=1}^{i}N_{j}}{T_{i}}}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{\sum_{j=1}^{i}N_{j}} is the
cumuative/accumulated number of seeds germinated in the \mjseqn{i}th time
interval, and \mjseqn{k} is the total number of time intervals.

Speed of germination expresses the rate of germination in terms of the total
number of seeds that germinate in a time interval. Higher values indicate
greater and faster germination. This is useful for comparisons only when
samples or treatments possess similar germinabilities. This is overcome by
either using the corrected speed of germination or by using germination
percentages instead of counts for computing speed.

\code{GermSpeedCorrected} computes the corrected speed of germination
\mjseqn{S} or corrected speed of accumulated germination \mjseqn{\hat{S}} as
follows \insertCite{evetts_germination_1972}{germinationmetrics}.

\mjsdeqn{\hat{S} = \frac{S}{FGP}}

\mjsdeqn{\hat{S}_{accumulated} = \frac{S_{accumulated}}{FGP}}

Where, \mjseqn{S} is the germination speed or \mjseqn{\hat{S}} is the
accumulated speed of germination (as
specified by the argument  \code{method = "normal"} or \code{method =
"accumulated"} respectively) computed with germination percentage instead of
counts and \mjseqn{FGP} is the final germination percentage or germinability.

With \code{percent = TRUE}, germination percentage is used instead of counts
for computation in \code{GermSpeed} and \code{GermSpeedAccumulated}. In case
of \code{GermSpeedCorrected}, germination percentage is always used for the
numerator.
}
\examples{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)

# From partial germination counts
#----------------------------------------------------------------------------
GermSpeed(germ.counts = x, intervals = int)
GermSpeedAccumulated(germ.counts = x, intervals = int)
GermSpeedCorrected(germ.counts = x, intervals = int, total.seeds = 50,
                   method = "normal")
GermSpeedCorrected(germ.counts = x, intervals = int, total.seeds = 50,
                   method = "accumulated")

# From partial germination counts (with percentages instead of counts)
#----------------------------------------------------------------------------
GermSpeed(germ.counts = x, intervals = int,
          percent = TRUE, total.seeds = 50)
GermSpeedAccumulated(germ.counts = x, intervals = int,
                     percent = TRUE, total.seeds = 50)

# From cumulative germination counts
#----------------------------------------------------------------------------
GermSpeed(germ.counts = y, intervals = int, partial = FALSE)
GermSpeedAccumulated(germ.counts = y, intervals = int, partial = FALSE)
GermSpeedCorrected(germ.counts = y, intervals = int,
                   partial = FALSE, total.seeds = 50, method = "normal")
GermSpeedCorrected(germ.counts = y, intervals = int,
                   partial = FALSE, total.seeds = 50, method = "accumulated")

# From cumulative germination counts (with percentages instead of counts)
#----------------------------------------------------------------------------
GermSpeed(germ.counts = y, intervals = int, partial = FALSE,
          percent = TRUE, total.seeds = 50)
GermSpeedAccumulated(germ.counts = y, intervals = int, partial = FALSE,
                     percent = TRUE, total.seeds = 50)
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.FourPHFfit.R
\name{plot.FourPHFfit}
\alias{plot.FourPHFfit}
\title{Plot the four-parameter hill function fitted cumulative germination curve
from a \code{FourPHFfit} object}
\usage{
\method{plot}{FourPHFfit}(
  x,
  rog = TRUE,
  t50.total = TRUE,
  t50.germ = TRUE,
  tmgr = TRUE,
  mgt = TRUE,
  uniformity = TRUE,
  limits = TRUE,
  plotlabels = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{FourPHFfit} obtained as output from the
\code{\link[germinationmetrics]{FourPHFfit}} function.}

\item{rog}{If \code{TRUE}, plots the Rate of Germination curve (RoG). Default
is \code{TRUE}.}

\item{t50.total}{If \code{TRUE}, highlights the time required for 50\% of
total seeds to germinate (\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Total}}})
as a vertical line. Default is \code{TRUE}.}

\item{t50.germ}{If \code{TRUE}, highlights the time required for 50\% of
viable/germinated seeds to germinate
(\mjseqn{\textrm{t}_{\textrm{50}_\textrm{Germ}}}) as a vertical line.
Default is \code{TRUE}.}

\item{tmgr}{If \code{TRUE}, highlights the Time at Maximum Germination Rate
(TMGR) as a vertical line. Default is \code{TRUE}.}

\item{mgt}{If \code{TRUE}, highlights the Mean Germination Time (MGT) as a
vertical line. Default is \code{TRUE}.}

\item{uniformity}{If \code{TRUE}, highlights the uniformity value
(\mjseqn{\textrm{U}_{\textrm{t}_{\textrm{max}}-\textrm{t}_{\textrm{min}}}})
as a horizontal line. Default is \code{TRUE}.}

\item{limits}{logical. If \code{TRUE}, set the limits of y axis (germination
percentage) between 0 and 100 in the germination curve plot. If
\code{FALSE}, limits are set according to the data. Default is \code{TRUE}.}

\item{plotlabels}{logical. If \code{TRUE}, adds labels to the germination
curve plot. Default is \code{TRUE}.}

\item{\dots}{Default plot arguments.}
}
\value{
The plot of the cumulative germination curve as an object of class
  \code{ggplot}.
}
\description{
\code{plot.FourPHFfit} plots the four-parameter hill function fitted
cumulative germination curve (FPHF curve) from a \code{FourPHFfit} object as
an object of class \code{ggplot}. Further, the rate of germination curve (RoG
curve) is plotted and different parameters annotated as specified in the
different arguments. \loadmathjax
}
\examples{

\donttest{
x <- c(0, 0, 0, 0, 4, 17, 10, 7, 1, 0, 1, 0, 0, 0)
y <- c(0, 0, 0, 0, 4, 21, 31, 38, 39, 39, 40, 40, 40, 40)
int <- 1:length(x)
total.seeds = 50

# From partial germination counts
#----------------------------------------------------------------------------
fit1 <- FourPHFfit(germ.counts = x, intervals = int,
                   total.seeds = 50, tmax = 20)

# From cumulative germination counts
#----------------------------------------------------------------------------
fit2 <- FourPHFfit(germ.counts = y, intervals = int,
                   total.seeds = 50, tmax = 20, partial = FALSE)

# Default plots
plot(fit1)
plot(fit2)

# No labels
plot(fit1, plotlabels = FALSE)
plot(fit2, plotlabels = FALSE)

# Only the FPHF curve
plot(fit1, rog = FALSE, t50.total = FALSE, t50.germ = FALSE,
     tmgr = FALSE, mgt = FALSE, uniformity = FALSE)
plot(fit2, rog = FALSE, t50.total = FALSE, t50.germ = FALSE,
     tmgr = FALSE, mgt = FALSE, uniformity = FALSE)

# Without y axis limits adjustment
plot(fit1, limits = FALSE)
plot(fit2, limits = FALSE)
}

}
\seealso{
\code{\link[germinationmetrics]{FourPHFfit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GermValue.R
\name{GermValue}
\alias{GermValue}
\alias{PeakValue}
\title{Peak value and germination value}
\usage{
PeakValue(germ.counts, intervals, total.seeds, partial = TRUE)

GermValue(
  germ.counts,
  intervals,
  total.seeds,
  partial = TRUE,
  method = c("czabator", "dp"),
  from.onset = TRUE,
  k = 10
)
}
\arguments{
\item{germ.counts}{Germination counts at each time interval. Can be partial
or cumulative as specified in the argument \code{partial}.}

\item{intervals}{The time intervals.}

\item{total.seeds}{Total number of seeds.}

\item{partial}{logical. If \code{TRUE}, \code{germ.counts} is considered as
partial and if \code{FALSE}, it is considered as cumulative. Default is
\code{TRUE}.}

\item{method}{The method for computing germination value. Either
\code{"czabator"} or \code{"dp"}.}

\item{from.onset}{logical. If \code{TRUE}, duration is considered only from
the onset of germination. If \code{FALSE}, full duration of germination
test is considered. Default is \code{TRUE}.}

\item{k}{Constant (See \strong{Details}). Default is 10.}
}
\value{
A list with the following components:  \item{Germination Value}{The
  germination value.} \item{Calculations}{The data frame of calculations.}
  \item{testend}{The end of test value (Only for method \code{dp}).}
}
\description{
Compute the Peak value (\mjseqn{PV}) or Emergence Energy (\mjseqn{EE})
\insertCite{czabator_germination_1962,bonner_ideal_1967}{germinationmetrics}
and Germination value (\mjseqn{GV})
\insertCite{czabator_germination_1962,djavanshir_germination_1976,brown_representing_1988}{germinationmetrics}.
\loadmathjax
}
\details{
Peak value (\mjseqn{PV}) is the maximum quotient obtained by dividing
successive cumulative germination values by the relevant incubation time
\insertCite{czabator_germination_1962}{germinationmetrics}.

\mjsdeqn{PV = \max\left ( \frac{G_{1}}{T_{1}},\frac{G_{2}}{T_{2}},\cdots
\frac{G_{k}}{T_{k}} \right )}

Where, \mjseqn{T_{i}} is the time from the start of the experiment to the
\mjseqn{i}th interval, \mjseqn{G_{i}} is the cumulative germination
percentage in the \mjseqn{i}th time interval, and \mjseqn{k} is the total
number of time intervals.

It represents the mean daily germination of the most vigorous component of
the seed lot, and is a mathematical expression of the break, or shoulder, of
a typical sigmoid  germination curve
\insertCite{djavanshir_germination_1976}{germinationmetrics}. It is the
accumulated number of seeds germinated at the point on the germination curve
at which the rate of germination starts to decrease. It is also described as
Emergence energy \insertCite{bonner_ideal_1967}{germinationmetrics}.

For daily germination counts, germination value (\mjseqn{GV}) is computed as
follows \insertCite{czabator_germination_1962}{germinationmetrics} (Specified
by the argument \code{method = "czabator"}).

\mjsdeqn{GV = PV \times MDG}

Where, \mjseqn{PV} is the peak value, and \mjseqn{MDG} is the mean daily
germination percentage from the onset of germination.

Germination value (\mjseqn{GV}) can also be computed for other time intervals
of successive germination counts, by replacing \mjseqn{MDG} with the mean
germination percentage per unit time (\mjseqn{\overline{GP}}).

A new estimation of germination value was given by
\insertCite{djavanshir_germination_1976;textual}{germinationmetrics} as
follows (Specified by the argument \code{method = "dp"}).

\mjsdeqn{GV = \frac{\sum DGS}{N} \times GP \times c}

Where, \mjseqn{DGS} is the daily germination speed computed by dividing
cumulative germination percentage by the number of days since the onset of
germination, \mjseqn{N} is the frequency or number of DGS calculated during
the test, \mjseqn{GP} is the germination percentage expressed over 100, and
\mjseqn{c} is a constant. The value of \mjseqn{c} is decided on the basis of
average daily speed of germination (\mjseqn{\frac{\sum DGS}{N}}). If it is
less than 10, then \mjseqn{c} value of 10 can be used and if it is more than
10, then value of 7 or 8 can be used for \mjseqn{c}.

For both methods of computing \mjseqn{GV}, only the duration from the onset
of germination is considered by default. Alternatively, modified \mjseqn{GV}
(\mjseqn{GV_{mod}}), where the entire duration from the beginning of the test
is considered can be obtained by using the argument \code{from.onset = FALSE}
\insertCite{brown_representing_1988}{germinationmetrics}.
}
\examples{

x <- c(0, 0, 34, 40, 21, 10, 4, 5, 3, 5, 8, 7, 7, 6, 6, 4, 0, 2, 0, 2)
y <- c(0, 0, 34, 74, 95, 105, 109, 114, 117, 122, 130, 137, 144, 150,
      156, 160, 160, 162, 162, 164)
int <- 1:length(x)
total.seeds = 200

# From partial germination counts
#----------------------------------------------------------------------------
PeakValue(germ.counts = x, intervals = int, total.seeds = 200)
GermValue(germ.counts = x, intervals = int, total.seeds = 200,
          method = "czabator")
GermValue(germ.counts = x, intervals = int, total.seeds = 200,
          method = "dp", k = 10)
GermValue(germ.counts = x, intervals = int, total.seeds = 200,
          method = "czabator", from.onset = FALSE)
GermValue(germ.counts = x, intervals = int, total.seeds = 200,
          method = "dp", k = 10, from.onset = FALSE)

# From cumulative germination counts
#----------------------------------------------------------------------------
PeakValue(germ.counts = y, interval = int, total.seeds = 200,
          partial = FALSE)
GermValue(germ.counts = y, intervals = int, total.seeds = 200,
          partial = FALSE, method = "czabator")
GermValue(germ.counts = y, intervals = int, total.seeds = 200,
          partial = FALSE, method = "dp", k = 10)
GermValue(germ.counts = y, intervals = int, total.seeds = 200,
          partial = FALSE, method = "czabator", from.onset = FALSE)
GermValue(germ.counts = y, intervals = int, total.seeds = 200,
          partial = FALSE, method = "dp", k = 10, from.onset = FALSE)

}
\references{
\insertAllCited{}
}
